Method: Puppet::Settings#define_settings

Defined in:
lib/puppet/settings.rb

#define_settings(section, defs) ⇒ Object

Define a group of settings.

Parameters:

  • section (Symbol)

    a symbol to use for grouping multiple settings together into a conceptual unit. This value (and the conceptual separation) is not used very often; the main place where it will have a potential impact is when code calls Settings#use method. See docs on that method for further details, but basically that method just attempts to do any preparation that may be necessary before code attempts to leverage the value of a particular setting. This has the most impact for file/directory settings, where #use will attempt to “ensure” those files / directories.

  • defs (Hash[Hash])

    the settings to be defined. This argument is a hash of hashes; each key should be a symbol, which is basically the name of the setting that you are defining. The value should be another hash that specifies the parameters for the particular setting. Legal values include:

    [:default] => not required; this is the value for the setting if no other value is specified (via cli, config file, etc.)
       For string settings this may include "variables", demarcated with $ or ${} which will be interpolated with values of other settings.
       The default value may also be a Proc that will be called only once to evaluate the default when the setting's value is retrieved.
    [:desc] => required; a description of the setting, used in documentation / help generation
    [:type] => not required, but highly encouraged!  This specifies the data type that the setting represents.  If
       you do not specify it, it will default to "string".  Legal values include:
       :string - A generic string setting
       :boolean - A boolean setting; values are expected to be "true" or "false"
       :file - A (single) file path; puppet may attempt to create this file depending on how the settings are used.  This type
           also supports additional options such as "mode", "owner", "group"
       :directory - A (single) directory path; puppet may attempt to create this file depending on how the settings are used.  This type
           also supports additional options such as "mode", "owner", "group"
       :path - This is intended to be used for settings whose value can contain multiple directory paths, represented
           as strings separated by the system path separator (e.g. system path, module path, etc.).
     [:mode] => an (optional) octal value to be used as the permissions/mode for :file and :directory settings
     [:owner] => optional owner username/uid for :file and :directory settings
     [:group] => optional group name/gid for :file and :directory settings
    


889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
923
924
925
# File 'lib/puppet/settings.rb', line 889

def define_settings(section, defs)
  section = section.to_sym
  call = []
  defs.each do |name, hash|
    raise ArgumentError, "setting definition for '#{name}' is not a hash!" unless hash.is_a? Hash

    name = name.to_sym
    hash[:name] = name
    hash[:section] = section
    raise ArgumentError, "Setting #{name} is already defined" if @config.include?(name)
    tryconfig = newsetting(hash)
    if short = tryconfig.short
      if other = @shortnames[short]
        raise ArgumentError, "Setting #{other.name} is already using short name '#{short}'"
      end
      @shortnames[short] = tryconfig
    end
    @config[name] = tryconfig

    # Collect the settings that need to have their hooks called immediately.
    # We have to collect them so that we can be sure we're fully initialized before
    # the hook is called.
    if tryconfig.has_hook?
      if tryconfig.call_hook_on_define?
        call << tryconfig
      elsif tryconfig.call_hook_on_initialize?
        @hooks_to_call_on_application_initialization |= [ tryconfig ]
      end
    end

    @deprecated_setting_names << name if tryconfig.deprecated?
  end

  call.each do |setting|
    setting.handle(self.value(setting.name))
  end
end