Class: Puppet::Application::Device
- Inherits:
-
Puppet::Application
- Object
- Puppet::Application
- Puppet::Application::Device
- Defined in:
- lib/puppet/application/device.rb
Constant Summary
Constants inherited from Puppet::Application
Constants included from Util
Util::AbsolutePathPosix, Util::AbsolutePathWindows, Util::DEFAULT_POSIX_MODE, Util::DEFAULT_WINDOWS_MODE, Util::PUPPET_STACK_INSERTION_FRAME, Util::RFC_3986_URI_REGEX
Constants included from Util::POSIX
Util::POSIX::LOCALE_ENV_VARS, Util::POSIX::USER_ENV_VARS
Constants included from Util::SymbolicFileMode
Util::SymbolicFileMode::SetGIDBit, Util::SymbolicFileMode::SetUIDBit, Util::SymbolicFileMode::StickyBit, Util::SymbolicFileMode::SymbolicMode, Util::SymbolicFileMode::SymbolicSpecialToBit
Instance Attribute Summary collapse
-
#agent ⇒ Object
Returns the value of attribute agent.
-
#args ⇒ Object
Returns the value of attribute args.
-
#host ⇒ Object
Returns the value of attribute host.
Attributes inherited from Puppet::Application
Instance Method Summary collapse
- #app_defaults ⇒ Object
- #find_resources(type, name) ⇒ Object
- #help ⇒ Object
- #main ⇒ Object
- #parse_args(args) ⇒ Object
- #preinit ⇒ Object
- #setup ⇒ Object
- #setup_context ⇒ Object
- #summary ⇒ Object
Methods inherited from Puppet::Application
[], available_application_names, banner, clear!, clear?, clear_everything_for_tests, #configure_indirector_routes, controlled_run, #deprecate, #deprecated?, environment_mode, exit, find, get_environment_mode, #handle_logdest_arg, #handlearg, #initialize, #initialize_app_defaults, interrupted?, #log_runtime_environment, #name, option, option_parser_commands, #parse_options, restart!, restart_requested?, #run, #run_command, run_mode, #set_log_level, #setup_logs, stop!, stop_requested?, try_load_class
Methods included from Util
absolute_path?, benchmark, chuser, clear_environment, default_env, deterministic_rand, deterministic_rand_int, exit_on_fail, format_backtrace_array, format_puppetstack_frame, get_env, get_environment, logmethods, merge_environment, path_to_uri, pretty_backtrace, replace_file, resolve_stackframe, safe_posix_fork, set_env, skip_external_facts, symbolizehash, thinmark, uri_encode, uri_query_encode, uri_to_path, uri_unescape, which, withenv, withumask
Methods included from Util::POSIX
#get_posix_field, #gid, groups_of, #idfield, #methodbyid, #methodbyname, #search_posix_field, #uid
Methods included from Util::SymbolicFileMode
#display_mode, #normalize_symbolic_mode, #symbolic_mode_to_int, #valid_symbolic_mode?
Constructor Details
This class inherits a constructor from Puppet::Application
Instance Attribute Details
#agent ⇒ Object
Returns the value of attribute agent.
10 11 12 |
# File 'lib/puppet/application/device.rb', line 10 def agent @agent end |
#args ⇒ Object
Returns the value of attribute args.
10 11 12 |
# File 'lib/puppet/application/device.rb', line 10 def args @args end |
#host ⇒ Object
Returns the value of attribute host.
10 11 12 |
# File 'lib/puppet/application/device.rb', line 10 def host @host end |
Instance Method Details
#app_defaults ⇒ Object
12 13 14 15 16 17 18 19 |
# File 'lib/puppet/application/device.rb', line 12 def app_defaults super.merge({ :catalog_terminus => :rest, :catalog_cache_terminus => :json, :node_terminus => :rest, :facts_terminus => :network_device, }) end |
#find_resources(type, name) ⇒ Object
401 402 403 404 405 406 407 408 409 |
# File 'lib/puppet/application/device.rb', line 401 def find_resources(type, name) key = [type, name].join('/') if name [ Puppet::Resource.indirection.find( key ) ] else Puppet::Resource.indirection.search( key, {} ) end end |
#help ⇒ Object
86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 |
# File 'lib/puppet/application/device.rb', line 86 def help "\npuppet-device(8) -- \#{summary}\n========\n\nSYNOPSIS\n--------\nRetrieves catalogs from the Puppet master and applies them to remote devices.\n\nThis subcommand can be run manually; or periodically using cron,\na scheduled task, or a similar tool.\n\n\nUSAGE\n-----\npuppet device [-h|--help] [-v|--verbose] [-d|--debug]\n [-l|--logdest syslog|<file>|console] [--detailed-exitcodes]\n [--deviceconfig <file>] [-w|--waitforcert <seconds>]\n [--libdir <directory>]\n [-a|--apply <file>] [-f|--facts] [-r|--resource <type> [name]]\n [-t|--target <device>] [--user=<user>] [-V|--version]\n\n\nDESCRIPTION\n-----------\nDevices require a proxy Puppet agent to request certificates, collect facts,\nretrieve and apply catalogs, and store reports.\n\n\nUSAGE NOTES\n-----------\nDevices managed by the puppet-device subcommand on a Puppet agent are\nconfigured in device.conf, which is located at $confdir/device.conf by default,\nand is configurable with the $deviceconfig setting.\n\nThe device.conf file is an INI-like file, with one section per device:\n\n[<DEVICE_CERTNAME>]\ntype <TYPE>\nurl <URL>\ndebug\n\nThe section name specifies the certname of the device.\n\nThe values for the type and url properties are specific to each type of device.\n\nThe optional debug property specifies transport-level debugging,\nand is limited to telnet and ssh transports.\n\nSee https://puppet.com/docs/puppet/latest/config_file_device.html for details.\n\n\nOPTIONS\n-------\nNote that any setting that's valid in the configuration file is also a valid\nlong argument. For example, 'server' is a valid configuration parameter, so\nyou can specify '--server <servername>' as an argument.\n\n* --help, -h:\nPrint this help message\n\n* --verbose, -v:\nTurn on verbose reporting.\n\n* --debug, -d:\nEnable full debugging.\n\n* --logdest, -l:\nWhere to send log messages. Choose between 'syslog' (the POSIX syslog\nservice), 'console', or the path to a log file. If debugging or verbosity is\nenabled, this defaults to 'console'. Otherwise, it defaults to 'syslog'.\nMultiple destinations can be set using a comma separated list\n(eg: `/path/file1,console,/path/file2`)\"\n\nA path ending with '.json' will receive structured output in JSON format. The\nlog file will not have an ending ']' automatically written to it due to the\nappending nature of logging. It must be appended manually to make the content\nvalid JSON.\n\n* --detailed-exitcodes:\nProvide transaction information via exit codes. If this is enabled, an exit\ncode of '1' means at least one device had a compile failure, an exit code of\n'2' means at least one device had resource changes, and an exit code of '4'\nmeans at least one device had resource failures. Exit codes of '3', '5', '6',\nor '7' means that a bitwise combination of the preceding exit codes happened.\n\n* --deviceconfig:\nPath to the device config file for puppet device.\nDefault: $confdir/device.conf\n\n* --waitforcert, -w:\nThis option only matters for targets that do not yet have certificates\nand it is enabled by default, with a value of 120 (seconds). This causes\n+puppet device+ to poll the server every 2 minutes and ask it to sign a\ncertificate request. This is useful for the initial setup of a target.\nYou can turn off waiting for certificates by specifying a time of 0.\n\n* --libdir:\nOverride the per-device libdir with a local directory. Specifying a libdir also\ndisables pluginsync. This is useful for testing.\n\nA path ending with '.jsonl' will receive structured output in JSON Lines\nformat.\n\n* --apply:\nApply a manifest against a remote target. Target must be specified.\n\n* --facts:\nDisplays the facts of a remote target. Target must be specified.\n\n* --resource:\nDisplays a resource state as Puppet code, roughly equivalent to\n`puppet resource`. Can be filtered by title. Requires --target be specified.\n\n* --target:\nTarget a specific device/certificate in the device.conf. Doing so will perform a\ndevice run against only that device/certificate.\n\n* --to_yaml:\nOutput found resources in yaml format, suitable to use with Hiera and\ncreate_resources.\n\n* --user:\nThe user to run as.\n\n\nEXAMPLE\n-------\n $ puppet device --target remotehost --verbose\n\nAUTHOR\n------\nBrice Figureau\n\n\nCOPYRIGHT\n---------\nCopyright (c) 2011-2018 Puppet Inc., LLC\nLicensed under the Apache 2.0 License\n HELP\nend\n" |
#main ⇒ Object
230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 |
# File 'lib/puppet/application/device.rb', line 230 def main if [:resource] and ![:target] raise _("resource command requires target") end if [:facts] and ![:target] raise _("facts command requires target") end unless [:apply].nil? raise _("missing argument: --target is required when using --apply") if [:target].nil? raise _("%{file} does not exist, cannot apply") % { file: [:apply] } unless File.file?([:apply]) end libdir = Puppet[:libdir] vardir = Puppet[:vardir] confdir = Puppet[:confdir] ssldir = Puppet[:ssldir] certname = Puppet[:certname] env = Puppet::Node::Environment.remote(Puppet[:environment]) returns = Puppet.override(:current_environment => env, :loaders => Puppet::Pops::Loaders.new(env)) do # find device list require 'puppet/util/network_device/config' devices = Puppet::Util::NetworkDevice::Config.devices.dup if [:target] devices.select! { |key, value| key == [:target] } end if devices.empty? if [:target] raise _("Target device / certificate '%{target}' not found in %{config}") % { target: [:target], config: Puppet[:deviceconfig] } else Puppet.err _("No device found in %{config}") % { config: Puppet[:deviceconfig] } exit(1) end end devices.collect do |devicename,device| pool = Puppet.runtime[:http].pool Puppet.override(:http_pool => pool) do # TODO when we drop support for ruby < 2.5 we can remove the extra block here begin device_url = URI.parse(device.url) # Handle nil scheme & port scheme = "#{device_url.scheme}://" if device_url.scheme port = ":#{device_url.port}" if device_url.port # override local $vardir and $certname Puppet[:ssldir] = ::File.join(Puppet[:deviceconfdir], device.name, 'ssl') Puppet[:confdir] = ::File.join(Puppet[:devicedir], device.name) Puppet[:libdir] = [:libdir] || ::File.join(Puppet[:devicedir], device.name, 'lib') Puppet[:vardir] = ::File.join(Puppet[:devicedir], device.name) Puppet[:certname] = device.name ssl_context = nil # create device directory under $deviceconfdir Puppet::FileSystem.dir_mkpath(Puppet[:ssldir]) unless Puppet::FileSystem.dir_exist?(Puppet[:ssldir]) # this will reload and recompute default settings and create device-specific sub vardir Puppet.settings.use :main, :agent, :ssl # Workaround for PUP-8736: store ssl certs outside the cache directory to prevent accidental removal and keep the old path as symlink optssldir = File.join(Puppet[:confdir], 'ssl') Puppet::FileSystem.symlink(Puppet[:ssldir], optssldir) unless Puppet::FileSystem.exist?(optssldir) unless [:resource] || [:facts] || [:apply] # Since it's too complicated to fix properly in the default settings, we workaround for PUP-9642 here. # See https://github.com/puppetlabs/puppet/pull/7483#issuecomment-483455997 for details. # This has to happen after `settings.use` above, so the directory is created and before `setup_host` below, where the SSL # routines would fail with access errors if Puppet.features.root? && !Puppet::Util::Platform.windows? user = Puppet::Type.type(:user).new(name: Puppet[:user]).exists? ? Puppet[:user] : nil group = Puppet::Type.type(:group).new(name: Puppet[:group]).exists? ? Puppet[:group] : nil Puppet.debug("Fixing perms for #{user}:#{group} on #{Puppet[:confdir]}") FileUtils.chown(user, group, Puppet[:confdir]) if user || group end ssl_context = setup_context unless [:libdir] Puppet.override(ssl_context: ssl_context) do Puppet::Configurer::PluginHandler.new.download_plugins(env) if Puppet::Configurer.should_pluginsync? end end end # this inits the device singleton, so that the facts terminus # and the various network_device provider can use it Puppet::Util::NetworkDevice.init(device) if [:resource] type, name = parse_args(command_line.args) Puppet.info _("retrieving resource: %{resource} from %{target} at %{scheme}%{url_host}%{port}%{url_path}") % { resource: type, target: device.name, scheme: scheme, url_host: device_url.host, port: port, url_path: device_url.path } resources = find_resources(type, name) if [:to_yaml] data = resources.map do |resource| resource.prune_parameters(:parameters_to_include => @extra_params).to_hiera_hash end.inject(:merge!) text = YAML.dump(type.downcase => data) else text = resources.map do |resource| resource.prune_parameters(:parameters_to_include => @extra_params).to_manifest.force_encoding(Encoding.default_external) end.join("\n") end (puts text) 0 elsif [:facts] Puppet.info _("retrieving facts from %{target} at %{scheme}%{url_host}%{port}%{url_path}") % { resource: type, target: device.name, scheme: scheme, url_host: device_url.host, port: port, url_path: device_url.path } remote_facts = Puppet::Node::Facts.indirection.find(name, :environment => env) # Give a proper name to the facts remote_facts.name = remote_facts.values['clientcert'] renderer = Puppet::Network::FormatHandler.format(:console) puts renderer.render(remote_facts) 0 elsif [:apply] # avoid reporting to server Puppet::Transaction::Report.indirection.terminus_class = :yaml Puppet::Resource::Catalog.indirection.cache_class = nil require 'puppet/application/apply' begin Puppet[:node_terminus] = :plain Puppet[:catalog_terminus] = :compiler Puppet[:catalog_cache_terminus] = nil Puppet[:facts_terminus] = :network_device Puppet.override(:network_device => true) do Puppet::Application::Apply.new(Puppet::Util::CommandLine.new('puppet', ["apply", [:apply]])).run_command end end else Puppet.info _("starting applying configuration to %{target} at %{scheme}%{url_host}%{port}%{url_path}") % { target: device.name, scheme: scheme, url_host: device_url.host, port: port, url_path: device_url.path } overrides = {} overrides[:ssl_context] = ssl_context if ssl_context Puppet.override(overrides) do configurer = Puppet::Configurer.new configurer.run(:network_device => true, :pluginsync => false) end end rescue => detail Puppet.log_exception(detail) # If we rescued an error, then we return 1 as the exit code 1 ensure pool.close Puppet[:libdir] = libdir Puppet[:vardir] = vardir Puppet[:confdir] = confdir Puppet[:ssldir] = ssldir Puppet[:certname] = certname end end end end if ! returns or returns.compact.empty? exit(1) elsif [:detailed_exitcodes] # Bitwise OR the return codes together, puppet style exit(returns.compact.reduce(:|)) elsif returns.include? 1 exit(1) else exit(0) end end |
#parse_args(args) ⇒ Object
393 394 395 396 397 398 399 |
# File 'lib/puppet/application/device.rb', line 393 def parse_args(args) type = args.shift or raise _("You must specify the type to display") Puppet::Type.type(type) or raise _("Could not find type %{type}") % { type: type } name = args.shift [type, name] end |
#preinit ⇒ Object
21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 |
# File 'lib/puppet/application/device.rb', line 21 def preinit # Do an initial trap, so that cancels don't get a stack trace. Signal.trap(:INT) do $stderr.puts _("Cancelling startup") exit(0) end { :apply => nil, :waitforcert => nil, :detailed_exitcodes => false, :verbose => false, :debug => false, :centrallogs => false, :setdest => false, :resource => false, :facts => false, :target => nil, :to_yaml => false, }.each do |opt,val| [opt] = val end @args = {} end |
#setup ⇒ Object
417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 |
# File 'lib/puppet/application/device.rb', line 417 def setup setup_logs Puppet::SSL::Oids.register_puppet_oids # setup global device-specific defaults; creates all necessary directories, etc Puppet.settings.use :main, :agent, :device, :ssl if [:apply] || [:facts] || [:resource] Puppet::Util::Log.newdestination(:console) else args[:Server] = Puppet[:server] if [:centrallogs] logdest = args[:Server] logdest += ":" + args[:Port] if args.include?(:Port) Puppet::Util::Log.newdestination(logdest) end Puppet::Transaction::Report.indirection.terminus_class = :rest if Puppet[:catalog_cache_terminus] Puppet::Resource::Catalog.indirection.cache_class = Puppet[:catalog_cache_terminus].intern end end end |
#setup_context ⇒ Object
411 412 413 414 415 |
# File 'lib/puppet/application/device.rb', line 411 def setup_context waitforcert = [:waitforcert] || (Puppet[:onetime] ? 0 : Puppet[:waitforcert]) sm = Puppet::SSL::StateMachine.new(waitforcert: waitforcert) sm.ensure_client_certificate end |
#summary ⇒ Object
82 83 84 |
# File 'lib/puppet/application/device.rb', line 82 def summary _("Manage remote network devices") end |