Class: Puppet::Application::Master
Constant Summary
DOCPATTERN
Instance Attribute Summary
#command_line, #options
Instance Method Summary
collapse
[], banner, clear!, clear?, #configure_indirector_routes, controlled_run, exit, find, #handlearg, #initialize, interrupted?, #name, option, option_parser_commands, #parse_options, restart!, restart_requested?, #run, run_mode, #set_run_mode, should_not_parse_config, should_parse_config, should_parse_config?, #should_parse_config?, stop!, stop_requested?
Methods included from Util
absolute_path?, activerecord_version, benchmark, binread, chuser, classproxy, #execfail, #execpipe, execute, execute_posix, execute_windows, logmethods, memory, path_to_uri, proxy, recmkdir, secure_open, symbolize, symbolizehash, symbolizehash!, synchronize_on, thinmark, #threadlock, uri_to_path, wait_for_output, which, withumask
#get_posix_field, #gid, #idfield, #methodbyid, #methodbyname, #search_posix_field, #uid
Instance Method Details
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
|
# File 'lib/puppet/application/master.rb', line 148
def compile
Puppet::Util::Log.newdestination :console
raise ArgumentError, "Cannot render compiled catalogs without pson support" unless Puppet.features.pson?
begin
unless catalog = Puppet::Resource::Catalog.indirection.find(options[:node])
raise "Could not compile catalog for #{options[:node]}"
end
jj catalog.to_resource
rescue => detail
$stderr.puts detail
exit(30)
end
exit(0)
end
|
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
|
# File 'lib/puppet/application/master.rb', line 33
def help
"\npuppet-master(8) -- The puppet master daemon\n========\n\nSYNOPSIS\n--------\nThe central puppet server. Functions as a certificate authority by\ndefault.\n\n\nUSAGE\n-----\npuppet master [-D|--daemonize|--no-daemonize] [-d|--debug] [-h|--help]\n[-l|--logdest <file>|console|syslog] [-v|--verbose] [-V|--version]\n[--compile <node-name>]\n\n\nDESCRIPTION\n-----------\nThis command starts an instance of puppet master, running as a daemon\nand using Ruby's built-in Webrick webserver. Puppet master can also be\nmanaged by other application servers; when this is the case, this\nexecutable is not used.\n\n\nOPTIONS\n-------\nNote that any configuration parameter that's valid in the configuration\nfile is also a valid long argument. For example, 'ssldir' is a valid\nconfiguration parameter, so you can specify '--ssldir <directory>' as an\nargument.\n\nSee the configuration file documentation at\nhttp://docs.puppetlabs.com/references/stable/configuration.html for the\nfull list of acceptable parameters. A commented list of all\nconfiguration options can also be generated by running puppet master\nwith '--genconfig'.\n\n* --daemonize:\nSend the process into the background. This is the default.\n\n* --no-daemonize:\nDo not send the process into the background.\n\n* --debug:\nEnable full debugging.\n\n* --help:\nPrint this help message.\n\n* --logdest:\nWhere to send messages. Choose between syslog, the console, and a log\nfile. Defaults to sending messages to syslog, or the console if\ndebugging or verbosity is enabled.\n\n* --verbose:\nEnable verbosity.\n\n* --version:\nPrint the puppet version number and exit.\n\n* --compile:\nCompile a catalogue and output it in JSON from the puppet master. Uses\nfacts contained in the $vardir/yaml/ directory to compile the catalog.\n\n\nEXAMPLE\n-------\npuppet master\n\nDIAGNOSTICS\n-----------\n\nWhen running as a standalone daemon, puppet master accepts the\nfollowing signals:\n\n* SIGHUP:\nRestart the puppet master server.\n* SIGINT and SIGTERM:\nShut down the puppet master server.\n\nAUTHOR\n------\nLuke Kanies\n\n\nCOPYRIGHT\n---------\nCopyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License\n\n HELP\nend\n"
|
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
|
# File 'lib/puppet/application/master.rb', line 164
def main
require 'etc'
require 'puppet/file_serving/content'
require 'puppet/file_serving/metadata'
xmlrpc_handlers = [:Status, :FileServer, :Master, :Report, :Filebucket]
xmlrpc_handlers << :CA if Puppet[:ca]
Puppet::SSL::Host.localhost
Puppet::SSL::Host.ca_location = :only if Puppet::SSL::CertificateAuthority.ca?
if Puppet.features.root?
begin
Puppet::Util.chuser
rescue => detail
puts detail.backtrace if Puppet[:trace]
$stderr.puts "Could not change user to #{Puppet[:user]}: #{detail}"
exit(39)
end
end
unless options[:rack]
require 'puppet/network/server'
@daemon.server = Puppet::Network::Server.new(:xmlrpc_handlers => xmlrpc_handlers)
@daemon.daemonize if Puppet[:daemonize]
else
require 'puppet/network/http/rack'
@app = Puppet::Network::HTTP::Rack.new(:xmlrpc_handlers => xmlrpc_handlers, :protocols => [:rest, :xmlrpc])
end
Puppet.notice "Starting Puppet master version #{Puppet.version}"
unless options[:rack]
@daemon.start
else
return @app
end
end
|
128
129
130
131
132
133
134
135
136
137
138
|
# File 'lib/puppet/application/master.rb', line 128
def preinit
Signal.trap(:INT) do
$stderr.puts "Cancelling startup"
exit(0)
end
require 'puppet/daemon'
@daemon = Puppet::Daemon.new
@daemon.argv = ARGV.dup
end
|
#run_command ⇒ Object
140
141
142
143
144
145
146
|
# File 'lib/puppet/application/master.rb', line 140
def run_command
if options[:node]
compile
else
main
end
end
|
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
|
# File 'lib/puppet/application/master.rb', line 208
def setup
raise Puppet::Error.new("Puppet master is not supported on Microsoft Windows") if Puppet.features.microsoft_windows?
if options[:debug] or options[:verbose]
if options[:debug]
Puppet::Util::Log.level = :debug
else
Puppet::Util::Log.level = :info
end
unless Puppet[:daemonize] or options[:rack]
Puppet::Util::Log.newdestination(:console)
options[:setdest] = true
end
end
Puppet::Util::Log.newdestination(:syslog) unless options[:setdest]
exit(Puppet.settings.print_configs ? 0 : 1) if Puppet.settings.print_configs?
Puppet.settings.use :main, :master, :ssl, :metrics
Puppet::Node.indirection.cache_class = :yaml
if Puppet::SSL::CertificateAuthority.ca?
Puppet::SSL::Host.ca_location = :local
Puppet.settings.use :ca
Puppet::SSL::CertificateAuthority.instance
else
Puppet::SSL::Host.ca_location = :none
end
end
|