Class: Puppet::Configurer

Inherits:
Object show all
Includes:
FactHandler, Util
Defined in:
lib/puppet/configurer.rb

Defined Under Namespace

Modules: FactHandler Classes: Downloader, PluginHandler

Constant Summary

Constants included from Util

Util::AbsolutePathPosix, Util::AbsolutePathWindows, Util::DEFAULT_POSIX_MODE, Util::DEFAULT_WINDOWS_MODE, Util::PUPPET_STACK_INSERTION_FRAME, Util::RFC_3986_URI_REGEX

Constants included from Util::POSIX

Util::POSIX::LOCALE_ENV_VARS, Util::POSIX::USER_ENV_VARS

Constants included from Util::SymbolicFileMode

Util::SymbolicFileMode::SetGIDBit, Util::SymbolicFileMode::SetUIDBit, Util::SymbolicFileMode::StickyBit, Util::SymbolicFileMode::SymbolicMode, Util::SymbolicFileMode::SymbolicSpecialToBit

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Util

absolute_path?, benchmark, chuser, clear_environment, default_env, deterministic_rand, deterministic_rand_int, exit_on_fail, format_backtrace_array, format_puppetstack_frame, get_env, get_environment, logmethods, merge_environment, path_to_uri, pretty_backtrace, replace_file, resolve_stackframe, safe_posix_fork, set_env, skip_external_facts, symbolizehash, thinmark, uri_encode, uri_query_encode, uri_to_path, uri_unescape, which, withenv, withumask

Methods included from Util::POSIX

#get_posix_field, #gid, groups_of, #idfield, #methodbyid, #methodbyname, #search_posix_field, #uid

Methods included from Util::SymbolicFileMode

#display_mode, #normalize_symbolic_mode, #symbolic_mode_to_int, #valid_symbolic_mode?

Methods included from FactHandler

#encode_facts, #facts_for_uploading, #find_facts

Constructor Details

#initialize(transaction_uuid = nil, job_id = nil) ⇒ Configurer



53
54
55
56
57
58
59
60
61
62
63
64
# File 'lib/puppet/configurer.rb', line 53

def initialize(transaction_uuid = nil, job_id = nil)
  @running = false
  @splayed = false
  @running_failure = false
  @cached_catalog_status = 'not_used'
  @environment = Puppet[:environment]
  @transaction_uuid = transaction_uuid || SecureRandom.uuid
  @job_id = job_id
  @static_catalog = true
  @checksum_type = Puppet[:supported_checksum_types]
  @handler = Puppet::Configurer::PluginHandler.new()
end

Instance Attribute Details

#environmentObject (readonly)

Returns the value of attribute environment.



17
18
19
# File 'lib/puppet/configurer.rb', line 17

def environment
  @environment
end

Class Method Details

.should_pluginsync?Boolean



24
25
26
27
28
29
30
# File 'lib/puppet/configurer.rb', line 24

def self.should_pluginsync?
  if Puppet[:use_cached_catalog]
    false
  else
    true
  end
end

.to_sObject

Provide more helpful strings to the logging that the Agent does



20
21
22
# File 'lib/puppet/configurer.rb', line 20

def self.to_s
  _("Puppet configuration client")
end

Instance Method Details

#apply_catalog(catalog, options) ⇒ Object

Apply supplied catalog and return associated application report



190
191
192
193
194
195
196
197
198
199
200
201
202
# File 'lib/puppet/configurer.rb', line 190

def apply_catalog(catalog, options)
  report = options[:report]
  report.configuration_version = catalog.version

  benchmark(:notice, _("Applied catalog in %{seconds} seconds")) do
    apply_catalog_time = thinmark do
      catalog.apply(options)
    end
    options[:report].add_times(:catalog_application, apply_catalog_time)
  end

  report
end

#convert_catalog(result, duration, facts, options = {}) ⇒ Object

Convert a plain resource catalog into our full host catalog.



109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
# File 'lib/puppet/configurer.rb', line 109

def convert_catalog(result, duration, facts, options = {})
  catalog = nil

  catalog_conversion_time = thinmark do
    # Will mutate the result and replace all Deferred values with resolved values
    if facts
      Puppet::Pops::Evaluator::DeferredResolver.resolve_and_replace(facts, result, Puppet.lookup(:current_environment))
    end

    catalog = result.to_ral
    catalog.finalize
    catalog.retrieval_duration = duration

    if Puppet[:write_catalog_summary]
      catalog.write_class_file
      catalog.write_resource_file
    end
  end
  options[:report].add_times(:convert_catalog, catalog_conversion_time) if options[:report]

  catalog
end

#execute_postrun_commandObject



32
33
34
# File 'lib/puppet/configurer.rb', line 32

def execute_postrun_command
  execute_from_setting(:postrun_command)
end

#execute_prerun_commandObject



36
37
38
# File 'lib/puppet/configurer.rb', line 36

def execute_prerun_command
  execute_from_setting(:prerun_command)
end

#get_facts(options) ⇒ Object



132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
# File 'lib/puppet/configurer.rb', line 132

def get_facts(options)
  if options[:pluginsync]
    plugin_sync_time = thinmark do
      remote_environment_for_plugins = Puppet::Node::Environment.remote(@environment)
      download_plugins(remote_environment_for_plugins)

      Puppet::GettextConfig.reset_text_domain('agent')
      Puppet::ModuleTranslations.load_from_vardir(Puppet[:vardir])
    end
    options[:report].add_times(:plugin_sync, plugin_sync_time) if options[:report]
  end

  facts_hash = {}
  facts = nil
  if Puppet::Resource::Catalog.indirection.terminus_class == :rest
    # This is a bit complicated.  We need the serialized and escaped facts,
    # and we need to know which format they're encoded in.  Thus, we
    # get a hash with both of these pieces of information.
    #
    # facts_for_uploading may set Puppet[:node_name_value] as a side effect
    facter_time = thinmark do
      facts = find_facts
      facts_hash = encode_facts(facts) # encode for uploading # was: facts_for_uploading
    end
    options[:report].add_times(:fact_generation, facter_time) if options[:report]
  end
  [facts_hash, facts]
end

#init_storageObject

Initialize and load storage



41
42
43
44
45
46
47
48
49
50
51
# File 'lib/puppet/configurer.rb', line 41

def init_storage
    Puppet::Util::Storage.load
rescue => detail
  Puppet.log_exception(detail, _("Removing corrupt state file %{file}: %{detail}") % { file: Puppet[:statefile], detail: detail })
  begin
    Puppet::FileSystem.unlink(Puppet[:statefile])
    retry
  rescue => detail
    raise Puppet::Error.new(_("Cannot remove %{file}: %{detail}") % { file: Puppet[:statefile], detail: detail }, detail)
  end
end

#prepare_and_retrieve_catalog(cached_catalog, facts, options, query_options) ⇒ Object



161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
# File 'lib/puppet/configurer.rb', line 161

def prepare_and_retrieve_catalog(cached_catalog, facts, options, query_options)
  # set report host name now that we have the fact
  options[:report].host = Puppet[:node_name_value]

  query_options[:transaction_uuid] = @transaction_uuid
  query_options[:job_id] = @job_id
  query_options[:static_catalog] = @static_catalog

  # Query params don't enforce ordered evaluation, so munge this list into a
  # dot-separated string.
  query_options[:checksum_type] = @checksum_type.join('.')

  # apply passes in ral catalog
  catalog = cached_catalog || options[:catalog]
  unless catalog
    # retrieve_catalog returns resource catalog
    catalog = retrieve_catalog(facts, query_options)
    Puppet.err _("Could not retrieve catalog; skipping run") unless catalog
  end
  catalog
end

#prepare_and_retrieve_catalog_from_cache(options = {}) ⇒ Object



183
184
185
186
187
# File 'lib/puppet/configurer.rb', line 183

def prepare_and_retrieve_catalog_from_cache(options = {})
  result = retrieve_catalog_from_cache({:transaction_uuid => @transaction_uuid, :static_catalog => @static_catalog})
  Puppet.info _("Using cached catalog from environment '%{catalog_env}'") % { catalog_env: result.environment } if result
  result
end

#resubmit_factstrue, false

Submit updated facts to the Puppet Server

This method will clear all current fact values, load a fresh set of fact data, and then submit it to the Puppet Server.



566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
# File 'lib/puppet/configurer.rb', line 566

def resubmit_facts
  Puppet.runtime[:facter].clear
  facts = find_facts

  client = Puppet.runtime[:http]
  session = client.create_session
  puppet = session.route_to(:puppet)

  Puppet.info(_("Uploading facts for %{node} to %{server}") % {
                node: facts.name,
                server: puppet.url.hostname})

  puppet.put_facts(facts.name, facts: facts, environment: Puppet.lookup(:current_environment).name.to_s)

  return true
rescue => detail
  Puppet.log_exception(detail, _("Failed to submit facts: %{detail}") %
                               { detail: detail })

  return false
end

#retrieve_catalog(facts, query_options) ⇒ Object

Get the remote catalog, yo. Returns nil if no catalog can be found.



67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
# File 'lib/puppet/configurer.rb', line 67

def retrieve_catalog(facts, query_options)
  query_options ||= {}
  if Puppet[:use_cached_catalog] || @running_failure
    result = retrieve_catalog_from_cache(query_options)
  end

  if result
    if Puppet[:use_cached_catalog]
      @cached_catalog_status = 'explicitly_requested'
    elsif @running_failure
      @cached_catalog_status = 'on_failure'
    end

    Puppet.info _("Using cached catalog from environment '%{environment}'") % { environment: result.environment }
  else
    result = retrieve_new_catalog(facts, query_options)

    if !result
      if !Puppet[:usecacheonfailure]
        Puppet.warning _("Not using cache on failed catalog")
        return nil
      end

      result = retrieve_catalog_from_cache(query_options)

      if result
        # don't use use cached catalog if it doesn't match server specified environment
        if result.environment != @environment
          Puppet.err _("Not using cached catalog because its environment '%{catalog_env}' does not match '%{local_env}'") % { catalog_env: result.environment, local_env: @environment }
          return nil
        end

        @cached_catalog_status = 'on_failure'
        Puppet.info _("Using cached catalog from environment '%{catalog_env}'") % { catalog_env: result.environment }
      end
    end
  end

  result
end

#run(options = {}) ⇒ Object

The code that actually runs the catalog. This just passes any options on to the catalog, which accepts :tags and :ignoreschedules.



207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
# File 'lib/puppet/configurer.rb', line 207

def run(options = {})
  pool = Puppet.runtime[:http].pool
  # We create the report pre-populated with default settings for
  # environment and transaction_uuid very early, this is to ensure
  # they are sent regardless of any catalog compilation failures or
  # exceptions.
  options[:report] ||= Puppet::Transaction::Report.new(nil, @environment, @transaction_uuid, @job_id, options[:start_time] || Time.now)
  report = options[:report]
  init_storage

  Puppet::Util::Log.newdestination(report)

  completed = nil
  begin
    Puppet.override(:http_pool => pool) do
      # Skip failover logic if the server_list setting is empty
      do_failover = Puppet.settings[:server_list] && !Puppet.settings[:server_list].empty?

      # When we are passed a catalog, that means we're in apply
      # mode. We shouldn't try to do any failover in that case.
      if options[:catalog].nil? && do_failover
        server, port = find_functional_server
        if server.nil?
          detail = _("Could not select a functional puppet server from server_list: '%{server_list}'") % { server_list: Puppet.settings.value(:server_list, Puppet[:environment].to_sym, true) }
          if Puppet[:usecacheonfailure]
            options[:pluginsync] = false
            @running_failure = true

            server = Puppet[:server_list].first[0]
            port = Puppet[:server_list].first[1] || Puppet[:serverport]

            Puppet.err(detail)
          else
            raise Puppet::Error, detail
          end
        else
          #TRANSLATORS 'server_list' is the name of a setting and should not be translated
          Puppet.debug _("Selected puppet server from the `server_list` setting: %{server}:%{port}") % { server: server, port: port }
          report.server_used = "#{server}:#{port}"
        end
        Puppet.override(server: server, serverport: port) do
          completed = run_internal(options)
        end
      else
        completed = run_internal(options)
      end
    end
  ensure
    pool.close
  end

  completed ? report.exit_status : nil
end

#save_last_run_summary(report) ⇒ Object



549
550
551
552
553
554
555
556
# File 'lib/puppet/configurer.rb', line 549

def save_last_run_summary(report)
  mode = Puppet.settings.setting(:lastrunfile).mode
  Puppet::Util.replace_file(Puppet[:lastrunfile], mode) do |fh|
    fh.print YAML.dump(report.raw_summary)
  end
rescue => detail
  Puppet.log_exception(detail, _("Could not save last run local report: %{detail}") % { detail: detail })
end

#send_report(report) ⇒ Object



534
535
536
537
538
539
540
541
542
543
544
545
546
547
# File 'lib/puppet/configurer.rb', line 534

def send_report(report)
  puts report.summary if Puppet[:summarize]
  save_last_run_summary(report)
  if Puppet[:report]
    remote = Puppet::Node::Environment.remote(@environment)
    begin
      Puppet::Transaction::Report.indirection.save(report, nil, ignore_cache: true, environment: remote)
    ensure
      Puppet::Transaction::Report.indirection.save(report, nil, ignore_terminus: true, environment: remote)
    end
  end
rescue => detail
  Puppet.log_exception(detail, _("Could not send report: %{detail}") % { detail: detail })
end

#valid_server_environment?Boolean



438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
# File 'lib/puppet/configurer.rb', line 438

def valid_server_environment?
  session = Puppet.lookup(:http_session)
  begin
    fs = session.route_to(:fileserver)
    fs.get_file_metadatas(path: URI(Puppet[:pluginsource]).path, recurse: :false, environment: @environment)
    true
  rescue Puppet::HTTP::ResponseError => detail
    if detail.response.code == 404
      Puppet.notice(_("Environment '%{environment}' not found on server, skipping initial pluginsync.") % { environment: @environment })
    else
      Puppet.log_exception(detail, detail.message)
    end
    false
  rescue => detail
    Puppet.log_exception(detail, detail.message)
    false
  end
end