Class: Puppet::Environments::Cached::Entry Private
- Defined in:
- lib/puppet/environments.rb
Overview
This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.
Never evicting entry
Direct Known Subclasses
Instance Attribute Summary collapse
- #value ⇒ Object readonly private
Instance Method Summary collapse
- #expired?(now) ⇒ Boolean private
- #guard ⇒ Object private
-
#guarded? ⇒ Boolean
private
These are not protected with a lock, because all of the Cached methods are protected.
-
#initialize(value) ⇒ Entry
constructor
private
A new instance of Entry.
- #label ⇒ Object private
- #touch ⇒ Object private
- #unguard ⇒ Object private
Constructor Details
#initialize(value) ⇒ Entry
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns a new instance of Entry.
531 532 533 534 |
# File 'lib/puppet/environments.rb', line 531 def initialize(value) @value = value @guards = 0 end |
Instance Attribute Details
#value ⇒ Object (readonly)
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
529 530 531 |
# File 'lib/puppet/environments.rb', line 529 def value @value end |
Instance Method Details
#expired?(now) ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
539 540 541 |
# File 'lib/puppet/environments.rb', line 539 def expired?(now) false end |
#guard ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
553 554 555 |
# File 'lib/puppet/environments.rb', line 553 def guard @guards += 1 end |
#guarded? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
These are not protected with a lock, because all of the Cached methods are protected.
549 550 551 |
# File 'lib/puppet/environments.rb', line 549 def guarded? @guards > 0 end |
#label ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
543 544 545 |
# File 'lib/puppet/environments.rb', line 543 def label "" end |
#touch ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
536 537 |
# File 'lib/puppet/environments.rb', line 536 def touch end |
#unguard ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
557 558 559 |
# File 'lib/puppet/environments.rb', line 557 def unguard @guards -= 1 end |