Class: Puppet::Environments::Directories Private

Inherits:
Object
  • Object
show all
Includes:
EnvironmentLoader
Defined in:
lib/puppet/environments.rb

Overview

This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.

Reads environments from a directory on disk. Each environment is represented as a sub-directory. The environment’s manifest setting is the manifest directory of the environment directory. The environment’s modulepath setting is the global modulepath (from the [master] section for the master) prepended with the modules directory of the environment directory.

API:

  • private

Class Method Summary collapse

Instance Method Summary collapse

Methods included from EnvironmentLoader

#clear_all, #get!

Constructor Details

#initialize(environment_dir, global_module_path) ⇒ Directories

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns a new instance of Directories.

API:

  • private



152
153
154
155
156
157
# File 'lib/puppet/environments.rb', line 152

def initialize(environment_dir, global_module_path)
  @environment_dir =  Puppet::FileSystem.expand_path(environment_dir)
  @global_module_path = global_module_path ?
    global_module_path.map { |p| Puppet::FileSystem.expand_path(p) } :
    nil
end

Class Method Details

.from_path(path, global_module_path) ⇒ Array<Puppet::Environments::Directories>

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Generate an array of directory loaders from a path string.

API:

  • private

Parameters:

  • path to environment directories

  • the global modulepath setting

Returns:

  • An array of configured directory loaders.



164
165
166
167
168
169
# File 'lib/puppet/environments.rb', line 164

def self.from_path(path, global_module_path)
  environments = path.split(File::PATH_SEPARATOR)
  environments.map do |dir|
    Puppet::Environments::Directories.new(dir, global_module_path)
  end
end

Instance Method Details

#get(name) ⇒ Puppet::Node::Environment?

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Find a named environment

API:

  • private

Parameters:

  • The name of environment to find

Returns:

  • the requested environment or nil if it wasn’t found



186
187
188
189
190
# File 'lib/puppet/environments.rb', line 186

def get(name)
  if valid_directory?(File.join(@environment_dir, name.to_s))
    create_environment(name)
  end
end

#get_conf(name) ⇒ Puppet::Setting::EnvironmentConf?

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Attempt to obtain the initial configuration for the environment. Not all loaders can provide this.

API:

  • private

Parameters:

  • The name of the environment whose configuration we are looking up

Returns:

  • the configuration for the requested environment, or nil if not found or no configuration is available



193
194
195
196
197
198
199
# File 'lib/puppet/environments.rb', line 193

def get_conf(name)
  envdir = File.join(@environment_dir, name.to_s)
  if valid_directory?(envdir)
    return Puppet::Settings::EnvironmentConf.load_from(envdir, @global_module_path)
  end
  nil
end

#listArray<Puppet::Node::Environment>

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns All of the environments known to the loader.

API:

  • private

Returns:

  • All of the environments known to the loader



177
178
179
180
181
182
183
# File 'lib/puppet/environments.rb', line 177

def list
  valid_directories.collect do |envdir|
    name = Puppet::FileSystem.basename_string(envdir).intern

    create_environment(name)
  end
end

#search_pathsArray<String>

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

A list of indicators of where the loader is getting its environments from.

API:

  • private

Returns:

  • The URIs of the load locations



172
173
174
# File 'lib/puppet/environments.rb', line 172

def search_paths
  ["file://#{@environment_dir}"]
end