Module: Puppet::FFI::POSIX::Constants

Extended by:
FFI::Library
Defined in:
lib/puppet/ffi/posix/constants.rb

Constant Summary collapse

MAXIMUM_NUMBER_OF_GROUPS =

Maximum number of supplementary groups (groups that a user can be in plus its primary group) (64 + 1 primary group) Chosen a reasonable middle number from the list https://www.j3e.de/ngroups.html

65