Method: Puppet::Node#add_extra_facts

Defined in:
lib/puppet/node.rb

#add_extra_facts(extra_facts) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Add extra facts, such as facts given to lookup on the command line The extra facts will override existing ones.

Parameters:

  • extra_facts (Hash{String=>Object})

    the facts to tadd



169
170
171
172
173
# File 'lib/puppet/node.rb', line 169

def add_extra_facts(extra_facts)
  @facts.add_extra_values(extra_facts)
  @parameters.merge!(extra_facts)
  nil
end