Module: Puppet::Parser::Files

Defined in:
lib/puppet/parser/files.rb

Overview

This is a silly central module for finding different kinds of files while parsing. This code doesn’t really belong in the Puppet::Module class, but it doesn’t really belong anywhere else, either.

Class Method Summary collapse

Class Method Details

.find_manifests_in_modules(pattern, environment) ⇒ Array(String, Array<String>)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Return a list of manifests as absolute filenames matching the given pattern.

Parameters:

  • pattern (String)

    A reference for a file in a module. It is the format “<modulename>/<file glob>”

  • environment (Puppet::Node::Environment)

    the environment of modules

Returns:



19
20
21
22
23
24
25
26
27
28
29
30
# File 'lib/puppet/parser/files.rb', line 19

def find_manifests_in_modules(pattern, environment)
  module_name, file_pattern = split_file_path(pattern)
  begin
    if mod = Puppet::Module.find(module_name, environment)
      return [mod.name, mod.match_manifests(file_pattern)]
    end
  rescue Puppet::Module::InvalidName
    # one of the modules being loaded might have an invalid name and so
    # looking for one might blow up since we load them lazily.
  end
  [nil, []]
end

.find_template(template, environment = nil) ⇒ Object

Find the concrete file denoted by file. If file is absolute, return it directly. Otherwise try to find it as a template in a module. If that fails, return it relative to the templatedir config param. In all cases, an absolute path is returned, which does not necessarily refer to an existing file



38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
# File 'lib/puppet/parser/files.rb', line 38

def find_template(template, environment = nil)
  if template == File.expand_path(template)
    return template
  end

  if template_paths = templatepath(environment)
    # If we can find the template in :templatedir, we return that.
    template_paths.collect { |path|
      File::join(path, template)
    }.each do |f|
      return f if FileTest.exist?(f)
    end
  end

  # check in the default template dir, if there is one
  if td_file = find_template_in_module(template, environment)
    return td_file
  end

  nil
end

.find_template_in_module(template, environment = nil) ⇒ Object



60
61
62
63
64
65
66
67
68
69
70
71
72
# File 'lib/puppet/parser/files.rb', line 60

def find_template_in_module(template, environment = nil)
  path, file = split_file_path(template)

  # Because templates don't have an assumed template name, like manifests do,
  # we treat templates with no name as being templates in the main template
  # directory.
  return nil unless file

  if mod = Puppet::Module.find(path, environment) and t = mod.template(file)
    return t
  end
  nil
end

.split_file_path(path) ⇒ Object

Split the path into the module and the rest of the path, or return nil if the path is empty or absolute (starts with a /).



85
86
87
88
89
90
91
# File 'lib/puppet/parser/files.rb', line 85

def split_file_path(path)
  if path == "" or Puppet::Util.absolute_path?(path)
    nil
  else
    path.split(File::SEPARATOR, 2)
  end
end

.templatepath(environment = nil) ⇒ Object

Return an array of paths by splitting the templatedir config parameter.



76
77
78
79
80
81
# File 'lib/puppet/parser/files.rb', line 76

def templatepath(environment = nil)
  dirs = Puppet.settings.value(:templatedir, environment).split(File::PATH_SEPARATOR)
  dirs.select do |p|
    File::directory?(p)
  end
end