Class: Puppet::Pops::Adapters::OriginAdapter

Inherits:
Puppet::Pops::Adaptable::Adapter show all
Defined in:
lib/puppet/pops/adapters.rb

Overview

An origin adapter adapts an object with where it came from. This origin describes the resource (a file, etc.) where source text originates. Instances of SourcePosAdapter is then used on other objects in a model to describe their relative position versus the origin.

See Also:

  • Utils#find_adapter

Instance Attribute Summary collapse

Method Summary

Methods inherited from Puppet::Pops::Adaptable::Adapter

adapt, adapt_new, associate_adapter, clear, create_adapter, get, instance_var_name

Instance Attribute Details

#originString

Returns the origin of the adapted (usually a filename).

Returns:

  • (String)

    the origin of the adapted (usually a filename)



22
23
24
# File 'lib/puppet/pops/adapters.rb', line 22

def origin
  @origin
end