Class: Puppet::Pops::Loader::ModuleLoaders::FileBased Private
- Inherits:
-
AbstractPathBasedModuleLoader
- Object
- Loader
- BaseLoader
- AbstractPathBasedModuleLoader
- Puppet::Pops::Loader::ModuleLoaders::FileBased
- Defined in:
- lib/puppet/pops/loader/module_loaders.rb
This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.
Direct Known Subclasses
Constant Summary
Constants inherited from Loader
Instance Attribute Summary collapse
- #path_index ⇒ Object readonly private
- #smart_paths ⇒ Object readonly private
Attributes inherited from AbstractPathBasedModuleLoader
#module_name, #path, #private_loader
Attributes inherited from BaseLoader
Attributes inherited from Loader
Instance Method Summary collapse
- #add_to_index(smart_path) ⇒ Object private
- #existing_path(effective_path) ⇒ Object private
- #existing_paths(effective_path) ⇒ Object private
- #get_contents(effective_path) ⇒ Object private
-
#initialize(parent_loader, loaders, module_name, path, loader_name, loadables = LOADABLE_KINDS) ⇒ FileBased
constructor
private
Create a kind of ModuleLoader for one module (Puppet Module, or module like).
- #meaningful_to_search?(smart_path) ⇒ Boolean private
-
#relative_paths(smart_path) ⇒ Array<String>
private
Return all paths that matches the given smart path.
- #to_s ⇒ Object private
Methods inherited from AbstractPathBasedModuleLoader
#discover, #find, #get_source_ref, #global?, #instantiate, #lib_root?, #loadables
Methods inherited from BaseLoader
#add_entry, #discover, #get_entry, #load_typed, #loaded_entry, #promote_entry, #remove_entry, #set_entry
Methods inherited from Loader
#[], #discover, #find, #get_entry, #inspect, #load, #load_typed, #loadables, #loaded_entry, #parent, #private_loader, #set_entry
Constructor Details
#initialize(parent_loader, loaders, module_name, path, loader_name, loadables = LOADABLE_KINDS) ⇒ FileBased
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Create a kind of ModuleLoader for one module (Puppet Module, or module like)
417 418 419 420 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 417 def initialize(parent_loader, loaders, module_name, path, loader_name, loadables = LOADABLE_KINDS) super @path_index = Set.new end |
Instance Attribute Details
#path_index ⇒ Object (readonly)
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
408 409 410 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 408 def path_index @path_index end |
#smart_paths ⇒ Object (readonly)
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
407 408 409 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 407 def smart_paths @smart_paths end |
Instance Method Details
#add_to_index(smart_path) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
445 446 447 448 449 450 451 452 453 454 455 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 445 def add_to_index(smart_path) found = Dir.glob(File.join(smart_path.generic_path, '**', "*#{smart_path.extension}")) # The reason for not always rejecting directories here is performance (avoid extra stat calls). The # false positives (directories with a matching extension) is an error in any case and will be caught # later. found = found.reject { |file_name| File.directory?(file_name) } if smart_path.extension.empty? @path_index.merge(found) found end |
#existing_path(effective_path) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
422 423 424 425 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 422 def existing_path(effective_path) # Optimized, checks index instead of visiting file system @path_index.include?(effective_path) ? effective_path : nil end |
#existing_paths(effective_path) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
427 428 429 430 431 432 433 434 435 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 427 def existing_paths(effective_path) dirname = File.dirname(effective_path) basename = File.basename(effective_path) # Select all paths matching `effective_path` up until an optional file extension @path_index.select do |path| File.basename(path, '.*') == basename && File.dirname(path) == dirname end end |
#get_contents(effective_path) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
457 458 459 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 457 def get_contents(effective_path) Puppet::FileSystem.read(effective_path, :encoding => 'utf-8') end |
#meaningful_to_search?(smart_path) ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
437 438 439 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 437 def meaningful_to_search?(smart_path) ! add_to_index(smart_path).empty? end |
#relative_paths(smart_path) ⇒ Array<String>
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Return all paths that matches the given smart path. The returned paths are relative to the #generic_path of the given smart path.
This method relies on the cache and does not perform any file system access
468 469 470 471 472 473 474 475 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 468 def relative_paths(smart_path) root = smart_path.generic_path found = [] @path_index.each do |path| found << Pathname(path).relative_path_from(Pathname(root)).to_s if smart_path.valid_path?(path) end found end |
#to_s ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
441 442 443 |
# File 'lib/puppet/pops/loader/module_loaders.rb', line 441 def to_s() "(ModuleLoader::FileBased '#{loader_name}' '#{module_name}')" end |