Class: Puppet::Pops::Model::Injection::BindingEntry
- Inherits:
-
PopsObject
- Object
- RGen::MetamodelBuilder::MMBase
- PopsObject
- Puppet::Pops::Model::Injection::BindingEntry
- Defined in:
- lib/puppet/pops/model/model.rb
Overview
A binding entry binds one capability generically or named, specifies default bindings or composition of other bindings.