Class: Puppet::Pops::Model::Injection::Producer
- Inherits:
-
Expression
- Object
- RGen::MetamodelBuilder::MMBase
- PopsObject
- Expression
- Puppet::Pops::Model::Injection::Producer
- Defined in:
- lib/puppet/pops/model/model.rb
Overview
A producer expression produces an instance of a type. The instance is initialized from an expression (or from the current scope if this expression is missing). – new. to handle production of injections