Class: Puppet::Pops::Model::ParenthesizedExpression
- Inherits:
-
UnaryExpression
- Object
- RGen::MetamodelBuilder::MMBase
- PopsObject
- Expression
- UnaryExpression
- Puppet::Pops::Model::ParenthesizedExpression
- Defined in:
- lib/puppet/pops/model/model.rb
Overview
A class that simply evaluates to the contained expression. It is of value in order to preserve user entered parentheses in transformations, and transformations from model to source.