Method: Puppet::Pops::Types::TypeFactory.ruby

Defined in:
lib/puppet/pops/types/type_factory.rb

.ruby(o) ⇒ Object .ruby(o) ⇒ Object

Note:

To get the type for the class’ class use ‘TypeCalculator.infer©`

Produces a type for a class or infers a type for something that is not a class

Overloads:

  • .ruby(o) ⇒ Object

    Parameters:

    • o (Class)

      produces the type corresponding to the class (e.g. Integer becomes PIntegerType)

  • .ruby(o) ⇒ Object

    Parameters:

    • o (Object)

      produces the type corresponding to the instance class (e.g. 3 becomes PIntegerType)



583
584
585
586
587
588
589
# File 'lib/puppet/pops/types/type_factory.rb', line 583

def self.ruby(o)
  if o.is_a?(Class)
    @type_calculator.type(o)
  else
    PRuntimeType.new(:ruby, o.class.name)
  end
end