Module: Puppet::Provider::Mount

Defined in:
lib/puppet/provider/mount.rb

Overview

A module just to store the mount/unmount methods. Individual providers still need to add the mount commands manually.

Instance Method Summary collapse

Instance Method Details

#mountObject

This only works when the mount point is synced to the fstab.



7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
# File 'lib/puppet/provider/mount.rb', line 7

def mount
  args = []

  # In general we do not have to pass mountoptions because we always
  # flush /etc/fstab before attempting to mount. But old code suggests
  # that MacOS always needs the mount options to be explicitly passed to
  # the mount command
  if Facter.value(:kernel) == 'Darwin'
    args << "-o" << self.options if self.options and self.options != :absent
  end
  args << resource[:name]

  mountcmd(*args)
  case get(:ensure)
  when :absent; set(:ensure => :ghost)
  when :unmounted; set(:ensure => :mounted)
  end
end

#mounted?Boolean

Is the mount currently mounted?



48
49
50
# File 'lib/puppet/provider/mount.rb', line 48

def mounted?
  [:mounted, :ghost].include?(get(:ensure))
end

#remountObject



26
27
28
29
30
31
32
33
34
# File 'lib/puppet/provider/mount.rb', line 26

def remount
  info "Remounting"
  if resource[:remounts] == :true
    mountcmd "-o", "remount", resource[:name]
  else
    unmount
    mount
  end
end

#unmountObject

This only works when the mount point is synced to the fstab.



37
38
39
40
41
42
43
44
45
# File 'lib/puppet/provider/mount.rb', line 37

def unmount
  umount(resource[:name])

  # Update property hash for future queries (e.g. refresh is called)
  case get(:ensure)
  when :mounted; set(:ensure => :unmounted)
  when :ghost; set(:ensure => :absent)
  end
end