Class: Puppet::Resource::Catalog

Inherits:
Graph::SimpleGraph show all
Extended by:
Indirector, Util::Pson
Includes:
Util::Tagging
Defined in:
lib/puppet/resource/catalog.rb

Overview

This class models a node catalog. It is the thing meant to be passed from server to client, and it contains all of the information in the catalog, including the resources and the relationships between them.

API:

  • public

Defined Under Namespace

Classes: ActiveRecord, Compiler, DuplicateResourceError, Json, Msgpack, Queue, Rest, StaticCompiler, StoreConfigs, Yaml

Constant Summary

Constants included from Indirector

Indirector::BadNameRegexp

Constants included from Util::Tagging

Util::Tagging::ValidTagRegex

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Indirector

configure_routes, indirects

Methods included from Util::Pson

pson_create

Methods included from Util::Tagging

#raw_tagged?, #tag, #tagged?, #tags, #tags=

Methods inherited from Graph::SimpleGraph

#add_edge, #add_relationship, #add_vertex, #adjacent, #dependencies, #dependents, #direct_dependencies_of, #direct_dependents_of, #directed?, #dotty, #downstream_from_vertex, #each_edge, #edge?, #edges, #edges_between, #find_cycles_in_graph, #instance_variable_get, #leaves, #matching_edges, #path_between, #paths_in_cycle, #remove_edge!, #remove_vertex!, #report_cycles_in_graph, #reversal, #size, #tarjan, #to_a, #to_dot, #to_dot_graph, #to_yaml_properties, #tree_from_vertex, #upstream_from_vertex, #vertex?, #vertices, #walk, #write_cycles_to_graph, #yaml_initialize

Constructor Details

#initialize(name = nil, environment = Puppet::Node::Environment::NONE) ⇒ Catalog

Returns a new instance of Catalog.

API:

  • public



239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
# File 'lib/puppet/resource/catalog.rb', line 239

def initialize(name = nil, environment = Puppet::Node::Environment::NONE)
  super()
  @name = name
  @classes = []
  @resource_table = {}
  @resources = []
  @relationship_graph = nil

  @host_config = true
  @environment_instance = environment
  @environment = environment.to_s

  @aliases = {}

  if block_given?
    yield(self)
    finalize
  end
end

Instance Attribute Details

#client_versionObject

Some metadata to help us compile and generally respond to the current state.

API:

  • public



47
48
49
# File 'lib/puppet/resource/catalog.rb', line 47

def client_version
  @client_version
end

#environmentObject

A String representing the environment for this catalog

API:

  • public



50
51
52
# File 'lib/puppet/resource/catalog.rb', line 50

def environment
  @environment
end

#environment_instanceObject

The actual environment instance that was used during compilation

API:

  • public



53
54
55
# File 'lib/puppet/resource/catalog.rb', line 53

def environment_instance
  @environment_instance
end

#from_cacheObject

Whether this catalog was retrieved from the cache, which affects whether it is written back out again.

API:

  • public



44
45
46
# File 'lib/puppet/resource/catalog.rb', line 44

def from_cache
  @from_cache
end

#host_configObject

Whether this is a host catalog, which behaves very differently. In particular, reports are sent, graphs are made, and state is stored in the state database. If this is set incorrectly, then you often end up in infinite loops, because catalogs are used to make things that the host catalog needs.

API:

  • public



40
41
42
# File 'lib/puppet/resource/catalog.rb', line 40

def host_config
  @host_config
end

#nameObject

The host name this is a catalog for.

API:

  • public



26
27
28
# File 'lib/puppet/resource/catalog.rb', line 26

def name
  @name
end

#retrieval_durationObject

How long this catalog took to retrieve. Used for reporting stats.

API:

  • public



33
34
35
# File 'lib/puppet/resource/catalog.rb', line 33

def retrieval_duration
  @retrieval_duration
end

#server_versionObject

Some metadata to help us compile and generally respond to the current state.

API:

  • public



47
48
49
# File 'lib/puppet/resource/catalog.rb', line 47

def server_version
  @server_version
end

#versionObject

The catalog version. Used for testing whether a catalog is up to date.

API:

  • public



30
31
32
# File 'lib/puppet/resource/catalog.rb', line 30

def version
  @version
end

Class Method Details

.from_data_hash(data) ⇒ Object

API:

  • public



324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
# File 'lib/puppet/resource/catalog.rb', line 324

def self.from_data_hash(data)
  result = new(data['name'], Puppet::Node::Environment::NONE)

  if tags = data['tags']
    result.tag(*tags)
  end

  if version = data['version']
    result.version = version
  end

  if environment = data['environment']
    result.environment = environment
    result.environment_instance = Puppet::Node::Environment.remote(environment.to_sym)
  end

  if resources = data['resources']
    result.add_resource(*resources.collect do |res|
      Puppet::Resource.from_data_hash(res)
    end)
  end

  if edges = data['edges']
    edges.each do |edge_hash|
      edge = Puppet::Relationship.from_data_hash(edge_hash)
      unless source = result.resource(edge.source)
        raise ArgumentError, "Could not intern from data: Could not find relationship source #{edge.source.inspect}"
      end
      edge.source = source

      unless target = result.resource(edge.target)
        raise ArgumentError, "Could not intern from data: Could not find relationship target #{edge.target.inspect}"
      end
      edge.target = target

      result.add_edge(edge)
    end
  end

  if classes = data['classes']
    result.add_class(*classes)
  end

  result
end

.from_pson(data) ⇒ Object

API:

  • public



370
371
372
373
# File 'lib/puppet/resource/catalog.rb', line 370

def self.from_pson(data)
  Puppet.deprecation_warning("from_pson is being removed in favour of from_data_hash.")
  self.from_data_hash(data)
end

Instance Method Details

#add_class(*classes) ⇒ Object

Add classes to our class list.

API:

  • public



56
57
58
59
60
61
62
63
# File 'lib/puppet/resource/catalog.rb', line 56

def add_class(*classes)
  classes.each do |klass|
    @classes << klass
  end

  # Add the class names as tags, too.
  tag(*classes)
end

#add_resource(*resources) ⇒ Object

API:

  • public



70
71
72
73
74
# File 'lib/puppet/resource/catalog.rb', line 70

def add_resource(*resources)
  resources.each do |resource|
    add_one_resource(resource)
  end
end

#alias(resource, key) ⇒ Object

Create an alias for a resource.

API:

  • public



117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
# File 'lib/puppet/resource/catalog.rb', line 117

def alias(resource, key)
  resource.ref =~ /^(.+)\[/
  class_name = $1 || resource.class.name

  newref = [class_name, key].flatten

  if key.is_a? String
    ref_string = "#{class_name}[#{key}]"
    return if ref_string == resource.ref
  end

  # LAK:NOTE It's important that we directly compare the references,
  # because sometimes an alias is created before the resource is
  # added to the catalog, so comparing inside the below if block
  # isn't sufficient.
  if existing = @resource_table[newref]
    return if existing == resource
    resource_declaration = " at #{resource.file}:#{resource.line}" if resource.file and resource.line
    existing_declaration = " at #{existing.file}:#{existing.line}" if existing.file and existing.line
    msg = "Cannot alias #{resource.ref} to #{key.inspect}#{resource_declaration}; resource #{newref.inspect} already declared#{existing_declaration}"
    raise ArgumentError, msg
  end
  @resource_table[newref] = resource
  @aliases[resource.ref] ||= []
  @aliases[resource.ref] << newref
end

#apply(options = {}) {|transaction| ... } ⇒ Puppet::Transaction

Apply our catalog to the local host.

Parameters:

  • (defaults to: {})

    a hash of options

Options Hash (options):

  • :report (Puppet::Transaction::Report)

    The report object to log this transaction to. This is optional, and the resulting transaction will create a report if not supplied.

  • :tags (Array[String])

    Tags used to filter the transaction. If supplied then only resources tagged with any of these tags will be evaluated.

  • :ignoreschedules (Boolean)

    Ignore schedules when evaluating resources

  • :for_network_device (Boolean)

    Whether this catalog is for a network device

Yields:

  • (transaction)

Returns:

  • the transaction created for this application

API:

  • public



162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
# File 'lib/puppet/resource/catalog.rb', line 162

def apply(options = {})
  Puppet::Util::Storage.load if host_config?

  transaction = create_transaction(options)

  begin
    transaction.report.as_logging_destination do
      transaction.evaluate
    end
  ensure
    # Don't try to store state unless we're a host config
    # too recursive.
    Puppet::Util::Storage.store if host_config?
  end

  yield transaction if block_given?

  transaction
end

#classesObject

API:

  • public



211
212
213
# File 'lib/puppet/resource/catalog.rb', line 211

def classes
  @classes.dup
end

#clear(remove_resources = true) ⇒ Object

API:

  • public



198
199
200
201
202
203
204
205
206
207
208
209
# File 'lib/puppet/resource/catalog.rb', line 198

def clear(remove_resources = true)
  super()
  # We have to do this so that the resources clean themselves up.
  @resource_table.values.each { |resource| resource.remove } if remove_resources
  @resource_table.clear
  @resources = []

  if @relationship_graph
    @relationship_graph.clear
    @relationship_graph = nil
  end
end

#container_of(resource) ⇒ A Resource?

Returns the resource that contains the given resource.

Parameters:

  • a resource in the catalog

Returns:

  • the resource that contains the given resource

API:

  • public



79
80
81
# File 'lib/puppet/resource/catalog.rb', line 79

def container_of(resource)
  adjacent(resource, :direction => :in)[0]
end

#create_resource(type, options) ⇒ Object

Create a new resource and register it in the catalog.

API:

  • public



216
217
218
219
220
221
222
223
224
# File 'lib/puppet/resource/catalog.rb', line 216

def create_resource(type, options)
  unless klass = Puppet::Type.type(type)
    raise ArgumentError, "Unknown resource type #{type}"
  end
  return unless resource = klass.new(options)

  add_resource(resource)
  resource
end

#filter(&block) ⇒ Object

filter out the catalog, applying block to each resource. If the block result is false, the resource will be kept otherwise it will be skipped

API:

  • public



415
416
417
418
419
420
421
422
423
424
425
426
427
428
# File 'lib/puppet/resource/catalog.rb', line 415

def filter(&block)
  # to_catalog must take place in a context where current_environment is set to the same env as the
  # environment set in the catalog (if it is set)
  # See PUP-3755
  if environment_instance
    Puppet.override({:current_environment => environment_instance}) do
      to_catalog :to_resource, &block
    end
  else
    # If catalog has no environment_instance, hope that the caller has made sure the context has the
    # correct current_environment
    to_catalog :to_resource, &block
  end
end

#finalizeObject

Make sure all of our resources are “finished”.

API:

  • public



227
228
229
230
231
232
233
# File 'lib/puppet/resource/catalog.rb', line 227

def finalize
  make_default_resources

  @resource_table.values.each { |resource| resource.finish }

  write_graph(:resources)
end

#host_config?Boolean

Returns:

API:

  • public



235
236
237
# File 'lib/puppet/resource/catalog.rb', line 235

def host_config?
  host_config
end

#make_default_resourcesObject

Make the default objects necessary for function.

API:

  • public



260
261
262
263
264
265
266
267
268
269
270
271
# File 'lib/puppet/resource/catalog.rb', line 260

def make_default_resources
  # We have to add the resources to the catalog, or else they won't get cleaned up after
  # the transaction.

  # First create the default scheduling objects
  Puppet::Type.type(:schedule).mkdefaultschedules.each { |res| add_resource(res) unless resource(res.ref) }

  # And filebuckets
  if bucket = Puppet::Type.type(:filebucket).mkdefaultbucket
    add_resource(bucket) unless resource(bucket.ref)
  end
end

#relationship_graph(given_prioritizer = nil) ⇒ Puppet::Graph::RelationshipGraph

The relationship_graph form of the catalog. This contains all of the dependency edges that are used for determining order.

Parameters:

  • (defaults to: nil)

    The prioritization strategy to use when constructing the relationship graph. Defaults the being determined by the ordering setting.

Returns:

API:

  • public



190
191
192
193
194
195
196
# File 'lib/puppet/resource/catalog.rb', line 190

def relationship_graph(given_prioritizer = nil)
  if @relationship_graph.nil?
    @relationship_graph = Puppet::Graph::RelationshipGraph.new(given_prioritizer || prioritizer)
    @relationship_graph.populate_from(self)
  end
  @relationship_graph
end

#remove_resource(*resources) ⇒ Object

Remove the resource from our catalog. Notice that we also call ‘remove’ on the resource, at least until resource classes no longer maintain references to the resource instances.

API:

  • public



276
277
278
279
280
281
282
283
284
285
286
287
288
289
# File 'lib/puppet/resource/catalog.rb', line 276

def remove_resource(*resources)
  resources.each do |resource|
    title_key = title_key_for_ref(resource.ref)
    @resource_table.delete(title_key)
    if aliases = @aliases[resource.ref]
      aliases.each { |res_alias| @resource_table.delete(res_alias) }
      @aliases.delete(resource.ref)
    end
    remove_vertex!(resource) if vertex?(resource)
    @relationship_graph.remove_vertex!(resource) if @relationship_graph and @relationship_graph.vertex?(resource)
    @resources.delete(title_key)
    resource.remove
  end
end

#resource(type, title = nil) ⇒ Object

Look a resource up by its reference (e.g., File).

API:

  • public



292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
# File 'lib/puppet/resource/catalog.rb', line 292

def resource(type, title = nil)
  # Always create a resource reference, so that it always
  # canonicalizes how we are referring to them.
  attributes = { :environment => environment_instance }
  if title
    res = Puppet::Resource.new(type, title, attributes)
  else
    # If they didn't provide a title, then we expect the first
    # argument to be of the form 'Class[name]', which our
    # Reference class canonicalizes for us.
    res = Puppet::Resource.new(nil, type, attributes)
  end
  res.catalog = self
  title_key      = [res.type, res.title.to_s]
  uniqueness_key = [res.type, res.uniqueness_key].flatten
  @resource_table[title_key] || @resource_table[uniqueness_key]
end

#resource_keysObject

API:

  • public



314
315
316
# File 'lib/puppet/resource/catalog.rb', line 314

def resource_keys
  @resource_table.keys
end

#resource_refsObject

API:

  • public



310
311
312
# File 'lib/puppet/resource/catalog.rb', line 310

def resource_refs
  resource_keys.collect{ |type, name| name.is_a?( String ) ? "#{type}[#{name}]" : nil}.compact
end

#resourcesObject

API:

  • public



318
319
320
321
322
# File 'lib/puppet/resource/catalog.rb', line 318

def resources
  @resources.collect do |key|
    @resource_table[key]
  end
end

#title_key_for_ref(ref) ⇒ Object

API:

  • public



65
66
67
68
# File 'lib/puppet/resource/catalog.rb', line 65

def title_key_for_ref( ref )
  ref =~ /^([-\w:]+)\[(.*)\]$/m
  [$1, $2]
end

#to_data_hashObject

API:

  • public



375
376
377
378
379
380
381
382
383
384
385
# File 'lib/puppet/resource/catalog.rb', line 375

def to_data_hash
  {
    'tags'      => tags,
    'name'      => name,
    'version'   => version,
    'environment' => environment.to_s,
    'resources' => @resources.collect { |v| @resource_table[v].to_pson_data_hash },
    'edges'     => edges.   collect { |e| e.to_pson_data_hash },
    'classes'   => classes
  }
end

#to_pson(*args) ⇒ Object

API:

  • public



398
399
400
# File 'lib/puppet/resource/catalog.rb', line 398

def to_pson(*args)
  to_pson_data_hash.to_pson(*args)
end

#to_pson_data_hashObject

API:

  • public



388
389
390
391
392
393
394
395
396
# File 'lib/puppet/resource/catalog.rb', line 388

def to_pson_data_hash
  {
    'document_type' => 'Catalog',
    'data'       => to_data_hash,
    'metadata' => {
      'api_version' => 1
      }
  }
end

#to_ralObject

Convert our catalog into a RAL catalog.

API:

  • public



403
404
405
# File 'lib/puppet/resource/catalog.rb', line 403

def to_ral
  to_catalog :to_ral
end

#to_resourceObject

Convert our catalog into a catalog of Puppet::Resource instances.

API:

  • public



408
409
410
# File 'lib/puppet/resource/catalog.rb', line 408

def to_resource
  to_catalog :to_resource
end

#write_class_fileObject

Store the classes in the classfile.

API:

  • public



431
432
433
434
435
436
437
# File 'lib/puppet/resource/catalog.rb', line 431

def write_class_file
  ::File.open(Puppet[:classfile], "w") do |f|
    f.puts classes.join("\n")
  end
rescue => detail
  Puppet.err "Could not create class file #{Puppet[:classfile]}: #{detail}"
end

#write_graph(name) ⇒ Object

Produce the graph files if requested.

API:

  • public



457
458
459
460
461
462
# File 'lib/puppet/resource/catalog.rb', line 457

def write_graph(name)
  # We only want to graph the main host catalog.
  return unless host_config?

  super
end

#write_resource_fileObject

Store the list of resources we manage

API:

  • public



440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
# File 'lib/puppet/resource/catalog.rb', line 440

def write_resource_file
  ::File.open(Puppet[:resourcefile], "w") do |f|
    to_print = resources.map do |resource|
      next unless resource.managed?
      if resource.name_var
        "#{resource.type}[#{resource[resource.name_var]}]"
      else
        "#{resource.ref.downcase}"
      end
    end.compact
    f.puts to_print.join("\n")
  end
rescue => detail
  Puppet.err "Could not create resource file #{Puppet[:resourcefile]}: #{detail}"
end