Class: Puppet::Resource::Catalog::Compiler

Inherits:
Indirector::Code show all
Includes:
Util
Defined in:
lib/puppet/indirector/catalog/compiler.rb

Constant Summary

Constants included from Util

Util::AbsolutePathPosix, Util::AbsolutePathWindows, Util::DEFAULT_POSIX_MODE, Util::DEFAULT_WINDOWS_MODE

Constants included from Util::Docs

Util::Docs::HEADER_LEVELS

Instance Attribute Summary collapse

Attributes included from Util::Docs

#doc, #nodoc

Instance Method Summary collapse

Methods included from Util

absolute_path?, activerecord_version, benchmark, binread, chuser, classproxy, #execfail, #execpipe, execute, execute_posix, execute_windows, logmethods, memory, path_to_uri, proxy, replace_file, safe_posix_fork, symbolize, symbolizehash, symbolizehash!, synchronize_on, thinmark, #threadlock, uri_to_path, wait_for_output, which, withumask

Methods included from Util::POSIX

#get_posix_field, #gid, #idfield, #methodbyid, #methodbyname, #search_posix_field, #uid

Methods inherited from Indirector::Terminus

abstract_terminus?, const2name, #indirection, indirection_name, inherited, mark_as_abstract_terminus, #model, model, #name, name2const, register_terminus_class, terminus_class, terminus_classes, #terminus_type, #validate, #validate_key, #validate_model

Methods included from Util::InstanceLoader

#instance_docs, #instance_hash, #instance_load, #instance_loader, #instance_loading?, #loaded_instance, #loaded_instances

Methods included from Util::Docs

#desc, #dochook, #doctable, #markdown_definitionlist, #markdown_header, #nodoc?, #pad, scrub

Constructor Details

#initializeCompiler

Returns a new instance of Compiler.



57
58
59
# File 'lib/puppet/indirector/catalog/compiler.rb', line 57

def initialize
  set_server_facts
end

Instance Attribute Details

#codeObject

Returns the value of attribute code.



12
13
14
# File 'lib/puppet/indirector/catalog/compiler.rb', line 12

def code
  @code
end

Instance Method Details

#extract_facts_from_request(request) ⇒ Object



14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
# File 'lib/puppet/indirector/catalog/compiler.rb', line 14

def extract_facts_from_request(request)
  return unless text_facts = request.options[:facts]
  unless format = request.options[:facts_format]
    raise ArgumentError, "Facts but no fact format provided for #{request.key}"
  end

  # If the facts were encoded as yaml, then the param reconstitution system
  # in Network::HTTP::Handler will automagically deserialize the value.
  if text_facts.is_a?(Puppet::Node::Facts)
    facts = text_facts
  else
    facts = Puppet::Node::Facts.convert_from(format, text_facts)
  end

  unless facts.name == request.key
    raise Puppet::Error, "Catalog for #{request.key.inspect} was requested with fact definition for the wrong node (#{facts.name.inspect})."
  end

  facts.add_timestamp
  Puppet::Node::Facts.indirection.save(facts)
end

#filter(catalog) ⇒ Object

filter-out a catalog to remove exported resources



52
53
54
55
# File 'lib/puppet/indirector/catalog/compiler.rb', line 52

def filter(catalog)
  return catalog.filter { |r| r.virtual? } if catalog.respond_to?(:filter)
  catalog
end

#find(request) ⇒ Object

Compile a node’s catalog.



37
38
39
40
41
42
43
44
45
46
47
48
49
# File 'lib/puppet/indirector/catalog/compiler.rb', line 37

def find(request)
  extract_facts_from_request(request)

  node = node_from_request(request)

  if catalog = compile(node)
    return catalog
  else
    # This shouldn't actually happen; we should either return
    # a config or raise an exception.
    return nil
  end
end

#networked?Boolean

Is our compiler part of a network, or are we just local?

Returns:

  • (Boolean)


62
63
64
# File 'lib/puppet/indirector/catalog/compiler.rb', line 62

def networked?
  Puppet.run_mode.master?
end