Class: Puppet::Settings

Inherits:
Object show all
Extended by:
Forwardable
Includes:
Enumerable
Defined in:
lib/puppet/settings.rb,
lib/puppet/settings/errors.rb

Overview

The class for handling configuration files.

Defined Under Namespace

Classes: AliasSetting, ArraySetting, AutosignSetting, BaseSetting, BooleanSetting, CertificateRevocationSetting, ChainedValues, ConfigFile, DirectorySetting, DurationSetting, EnumSetting, EnvironmentConf, FileOrDirectorySetting, FileSetting, HttpExtraHeadersSetting, IniFile, InterpolationError, ParseError, PathSetting, PrioritySetting, SearchPathElement, ServerListSetting, SettingsError, StringSetting, SymbolicEnumSetting, TTLSetting, TerminusSetting, ValidationError, ValueTranslator, Values, ValuesFromEnvironmentConf, ValuesFromSection

Constant Summary collapse

PuppetOptionParser =

local reference for convenience

Puppet::Util::CommandLine::PuppetOptionParser
REQUIRED_APP_SETTINGS =

These are the settings that every app is required to specify; there are reasonable defaults defined in application.rb.

[:logdir, :confdir, :vardir, :codedir]
ALLOWED_SECTION_NAMES =

The acceptable sections of the puppet.conf configuration file.

['main', 'server', 'master', 'agent', 'user'].freeze
NONE =
'none'.freeze
SETTING_TYPES =
{
    :string     => StringSetting,
    :file       => FileSetting,
    :directory  => DirectorySetting,
    :file_or_directory => FileOrDirectorySetting,
    :path       => PathSetting,
    :boolean    => BooleanSetting,
    :terminus   => TerminusSetting,
    :duration   => DurationSetting,
    :ttl        => TTLSetting,
    :array      => ArraySetting,
    :enum       => EnumSetting,
    :symbolic_enum   => SymbolicEnumSetting,
    :priority   => PrioritySetting,
    :autosign   => AutosignSetting,
    :server_list => ServerListSetting,
    :http_extra_headers => HttpExtraHeadersSetting,
    :certificate_revocation => CertificateRevocationSetting,
    :alias => AliasSetting
}

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Enumerable

#uniq

Constructor Details

#initializeSettings

Create a new collection of config settings.



130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
# File 'lib/puppet/settings.rb', line 130

def initialize
  @config = {}
  @shortnames = {}

  @created = []

  # Keep track of set values.
  @value_sets = {
    :cli => Values.new(:cli, @config),
    :memory => Values.new(:memory, @config),
    :application_defaults => Values.new(:application_defaults, @config),
    :overridden_defaults => Values.new(:overridden_defaults, @config),
  }
  @configuration_file = nil

  # And keep a per-environment cache
  @cache = Hash.new { |hash, key| hash[key] = {} }
  @values = Hash.new { |hash, key| hash[key] = {} }

  # The list of sections we've used.
  @used = []

  @hooks_to_call_on_application_initialization = []
  @deprecated_setting_names = []
  @deprecated_settings_that_have_been_configured = []

  @translate = Puppet::Settings::ValueTranslator.new
  @config_file_parser = Puppet::Settings::ConfigFile.new(@translate)
end

Instance Attribute Details

#files=(value) ⇒ Object

Sets the attribute files

Parameters:

  • value

    the value to set the attribute files to.



40
41
42
# File 'lib/puppet/settings.rb', line 40

def files=(value)
  @files = value
end

#timerObject (readonly)

Returns the value of attribute timer.



41
42
43
# File 'lib/puppet/settings.rb', line 41

def timer
  @timer
end

Class Method Details

.app_defaults_for_run_mode(run_mode) ⇒ Object

This method is intended for puppet internal use only; it is a convenience method that returns reasonable application default settings values for a given run_mode.



54
55
56
57
58
59
60
61
62
63
64
# File 'lib/puppet/settings.rb', line 54

def self.app_defaults_for_run_mode(run_mode)
  {
      :name     => run_mode.to_s,
      :run_mode => run_mode.name,
      :confdir  => run_mode.conf_dir,
      :codedir  => run_mode.code_dir,
      :vardir   => run_mode.var_dir,
      :rundir   => run_mode.run_dir,
      :logdir   => run_mode.log_dir,
  }
end

.clean_opt(opt, val) ⇒ Object

A utility method (public, is used by application.rb and perhaps elsewhere) that munges a command-line option string into the format that Puppet.settings expects. (This mostly has to deal with handling the "no-" prefix on flag/boolean options).

Parameters:

  • opt (String)

    the command line option that we are munging

  • val (String, TrueClass, FalseClass)

    the value for the setting (as determined by the OptionParser)



357
358
359
360
361
362
363
364
365
# File 'lib/puppet/settings.rb', line 357

def self.clean_opt(opt, val)
  # rewrite --[no-]option to --no-option if that's what was given
  if opt =~ /\[no-\]/ and !val
    opt = opt.gsub(/\[no-\]/,'no-')
  end
  # otherwise remove the [no-] prefix to not confuse everybody
  opt = opt.gsub(/\[no-\]/, '')
  [opt, val]
end

.default_certnameObject



66
67
68
69
70
71
72
73
74
75
# File 'lib/puppet/settings.rb', line 66

def self.default_certname()
  hostname = hostname_fact
  domain = domain_fact
  if domain and domain != ""
    fqdn = [hostname, domain].join(".")
  else
    fqdn = hostname
  end
  fqdn.to_s.gsub(/\.$/, '')
end

.default_config_file_nameObject



85
86
87
# File 'lib/puppet/settings.rb', line 85

def self.default_config_file_name
  "puppet.conf"
end

.domain_factObject



81
82
83
# File 'lib/puppet/settings.rb', line 81

def self.domain_fact()
  Puppet.runtime[:facter].value :domain
end

.hostname_factObject



77
78
79
# File 'lib/puppet/settings.rb', line 77

def self.hostname_fact()
  Puppet.runtime[:facter].value :hostname
end

Instance Method Details

#[](param) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Retrieve a config value

Parameters:

  • param (Symbol)

    the name of the setting

Returns:

  • (Object)

    the value of the setting



164
165
166
167
168
169
# File 'lib/puppet/settings.rb', line 164

def [](param)
  if @deprecated_setting_names.include?(param)
    issue_deprecation_warning(setting(param), "Accessing '#{param}' as a setting is deprecated.")
  end
  value(param)
end

#[]=(param, value) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Set a config value. This doesn't set the defaults, it sets the value itself.

Parameters:

  • param (Symbol)

    the name of the setting

  • value (Object)

    the new value of the setting



175
176
177
178
179
180
181
# File 'lib/puppet/settings.rb', line 175

def []=(param, value)
  if @deprecated_setting_names.include?(param)
    issue_deprecation_warning(setting(param), "Modifying '#{param}' as a setting is deprecated.")
  end
  @value_sets[:memory].set(param, value)
  unsafe_flush_cache
end

#addargs(options) ⇒ Object

Generate the list of valid arguments, in a format that GetoptLong can understand, and add them to the passed option list.



199
200
201
202
203
204
205
206
# File 'lib/puppet/settings.rb', line 199

def addargs(options)
  # Add all of the settings as valid options.
  self.each { |name, setting|
    setting.getopt_args.each { |args| options << args }
  }

  options
end

#app_defaults_initialized?Boolean

Returns:

  • (Boolean)


368
369
370
# File 'lib/puppet/settings.rb', line 368

def app_defaults_initialized?
  @app_defaults_initialized
end

#apply_metadata_from_section(section) ⇒ Object



707
708
709
710
711
712
713
714
# File 'lib/puppet/settings.rb', line 707

def (section)
  section.settings.each do |setting|
    type = @config[setting.name] if setting.has_metadata?
    if type
      type.set_meta(setting.meta)
    end
  end
end

#boolean?(param) ⇒ Boolean

Is our setting a boolean setting?

Returns:

  • (Boolean)


220
221
222
223
# File 'lib/puppet/settings.rb', line 220

def boolean?(param)
  param = param.to_sym
  @config.include?(param) and @config[param].kind_of?(BooleanSetting)
end

#clearObject

Remove all set values, potentially skipping cli values.



226
227
228
# File 'lib/puppet/settings.rb', line 226

def clear
  unsafe_clear
end

#clear_environment_settings(environment) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Clears all cached settings for a particular environment to ensure that changes to environment.conf are reflected in the settings if the environment timeout has expired.

param [String, Symbol] environment the name of environment to clear settings for



262
263
264
265
266
267
268
269
270
# File 'lib/puppet/settings.rb', line 262

def clear_environment_settings(environment)

  if environment.nil?
    return
  end

  @cache[environment.to_sym].clear
  @values[environment.to_sym] = {}
end

#clearusedObject



291
292
293
294
# File 'lib/puppet/settings.rb', line 291

def clearused
  @cache.clear
  @used = []
end

#configsearchpath(environment = nil, run_mode = preferred_run_mode) ⇒ Array<SearchPathElement>

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

The order in which to search for values, without defaults.

Parameters:

  • environment (String, Symbol) (defaults to: nil)

    symbolic reference to an environment name

  • run_mode (Symbol) (defaults to: preferred_run_mode)

    symbolic reference to a Puppet run mode

Returns:



828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
# File 'lib/puppet/settings.rb', line 828

def configsearchpath(environment = nil, run_mode = preferred_run_mode)
  searchpath = [
    SearchPathElement.new(:memory, :values),
    SearchPathElement.new(:cli, :values),
  ]
  searchpath << SearchPathElement.new(environment.intern, :environment) if environment

  if run_mode
    if [:master, :server].include?(run_mode)
      searchpath << SearchPathElement.new(:server, :section)
      searchpath << SearchPathElement.new(:master, :section)
    else
      searchpath << SearchPathElement.new(run_mode, :section)
    end
  end

  searchpath << SearchPathElement.new(:main, :section)
end

#define_settings(section, defs) ⇒ Object

Define a group of settings.

Parameters:

  • section (Symbol)

    a symbol to use for grouping multiple settings together into a conceptual unit. This value (and the conceptual separation) is not used very often; the main place where it will have a potential impact is when code calls Settings#use method. See docs on that method for further details, but basically that method just attempts to do any preparation that may be necessary before code attempts to leverage the value of a particular setting. This has the most impact for file/directory settings, where #use will attempt to "ensure" those files / directories.

  • defs (Hash[Hash])

    the settings to be defined. This argument is a hash of hashes; each key should be a symbol, which is basically the name of the setting that you are defining. The value should be another hash that specifies the parameters for the particular setting. Legal values include: [:default] => not required; this is the value for the setting if no other value is specified (via cli, config file, etc.) For string settings this may include "variables", demarcated with $ or ${} which will be interpolated with values of other settings. The default value may also be a Proc that will be called only once to evaluate the default when the setting's value is retrieved. [:desc] => required; a description of the setting, used in documentation / help generation [:type] => not required, but highly encouraged! This specifies the data type that the setting represents. If you do not specify it, it will default to "string". Legal values include: :string - A generic string setting :boolean - A boolean setting; values are expected to be "true" or "false" :file - A (single) file path; puppet may attempt to create this file depending on how the settings are used. This type also supports additional options such as "mode", "owner", "group" :directory - A (single) directory path; puppet may attempt to create this file depending on how the settings are used. This type also supports additional options such as "mode", "owner", "group" :path - This is intended to be used for settings whose value can contain multiple directory paths, represented as strings separated by the system path separator (e.g. system path, module path, etc.). [:mode] => an (optional) octal value to be used as the permissions/mode for :file and :directory settings [:owner] => optional owner username/uid for :file and :directory settings [:group] => optional group name/gid for :file and :directory settings



997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
# File 'lib/puppet/settings.rb', line 997

def define_settings(section, defs)
  section = section.to_sym
  call = []
  defs.each do |name, hash|
    raise ArgumentError, _("setting definition for '%{name}' is not a hash!") % { name: name } unless hash.is_a? Hash

    name = name.to_sym
    hash[:name] = name
    hash[:section] = section
    raise ArgumentError, _("Setting %{name} is already defined") % { name: name } if @config.include?(name)
    tryconfig = newsetting(hash)
    short = tryconfig.short
    if short
      other = @shortnames[short]
      if other
        raise ArgumentError, _("Setting %{name} is already using short name '%{short}'") % { name: other.name, short: short }
      end
      @shortnames[short] = tryconfig
    end
    @config[name] = tryconfig

    # Collect the settings that need to have their hooks called immediately.
    # We have to collect them so that we can be sure we're fully initialized before
    # the hook is called.
    if tryconfig.has_hook?
      if tryconfig.call_hook_on_define?
        call << tryconfig
      elsif tryconfig.call_hook_on_initialize?
        @hooks_to_call_on_application_initialization |= [ tryconfig ]
      end
    end

    @deprecated_setting_names << name if tryconfig.deprecated?
  end

  call.each do |setting|
    setting.handle(self.value(setting.name))
  end
end

#description(name) ⇒ Object

Return a value's description.



424
425
426
427
428
429
430
431
# File 'lib/puppet/settings.rb', line 424

def description(name)
  obj = @config[name.to_sym]
  if obj
    obj.desc
  else
    nil
  end
end

#eachsectionObject

Iterate over each section name.



436
437
438
439
440
441
442
443
444
445
# File 'lib/puppet/settings.rb', line 436

def eachsection
  yielded = []
  @config.each_value do |object|
    section = object.section
    unless yielded.include? section
      yield section
      yielded << section
    end
  end
end

#flush_cacheObject

Clear @cache, @used and the Environment.

Whenever an object is returned by Settings, a copy is stored in @cache. As long as Setting attributes that determine the content of returned objects remain unchanged, Settings can keep returning objects from @cache without re-fetching or re-generating them.

Whenever a Settings attribute changes, such as @values or @preferred_run_mode, this method must be called to clear out the caches so that updated objects will be returned.



282
283
284
# File 'lib/puppet/settings.rb', line 282

def flush_cache
  unsafe_flush_cache
end

#generate_configObject



535
536
537
538
# File 'lib/puppet/settings.rb', line 535

def generate_config
  puts to_config
  true
end

#generate_manifestObject



540
541
542
543
# File 'lib/puppet/settings.rb', line 540

def generate_manifest
  puts to_manifest
  true
end

#global_defaults_initialized?Boolean

Returns:

  • (Boolean)


296
297
298
# File 'lib/puppet/settings.rb', line 296

def global_defaults_initialized?()
  @global_defaults_initialized
end

#handlearg(opt, value = nil) ⇒ Object

Handle a command-line argument.



456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
# File 'lib/puppet/settings.rb', line 456

def handlearg(opt, value = nil)
  @cache.clear

  if value.is_a?(FalseClass)
    value = "false"
  elsif value.is_a?(TrueClass)
    value = "true"
  end

  value &&= @translate[value]
  str = opt.sub(/^--/,'')

  bool = true
  newstr = str.sub(/^no-/, '')
  if newstr != str
    str = newstr
    bool = false
  end
  str = str.intern

  if @config[str].is_a?(Puppet::Settings::BooleanSetting)
    if value == "" or value.nil?
      value = bool
    end
  end

  s = @config[str]
  if s
    @deprecated_settings_that_have_been_configured << s if s.completely_deprecated?
  end

  @value_sets[:cli].set(str, value)
  unsafe_flush_cache
end

#include?(name) ⇒ Boolean

Returns:

  • (Boolean)


491
492
493
494
# File 'lib/puppet/settings.rb', line 491

def include?(name)
  name = name.intern if name.is_a? String
  @config.include?(name)
end

#initialize_app_defaults(app_defaults) ⇒ Object



372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
# File 'lib/puppet/settings.rb', line 372

def initialize_app_defaults(app_defaults)
  REQUIRED_APP_SETTINGS.each do |key|
    raise SettingsError, "missing required app default setting '#{key}'" unless app_defaults.has_key?(key)
  end

  app_defaults.each do |key, value|
    if key == :run_mode
      self.preferred_run_mode = value
    else
      @value_sets[:application_defaults].set(key, value)
      unsafe_flush_cache
    end
  end
  
  call_hooks_deferred_to_application_initialization
  issue_deprecations

  REQUIRED_APP_SETTINGS.each do |key|
    create_ancestors(Puppet[key])
  end

  @app_defaults_initialized = true
end

#initialize_global_settings(args = [], require_config = true) ⇒ Object

Raises:



300
301
302
303
304
305
306
307
308
309
310
311
312
# File 'lib/puppet/settings.rb', line 300

def initialize_global_settings(args = [], require_config = true)
  raise Puppet::DevError, _("Attempting to initialize global default settings more than once!") if global_defaults_initialized?

  # The first two phases of the lifecycle of a puppet application are:
  # 1) Parse the command line options and handle any of them that are
  #    registered, defined "global" puppet settings (mostly from defaults.rb).
  # 2) Parse the puppet config file(s).

  parse_global_options(args)
  parse_config_files(require_config)

  @global_defaults_initialized = true
end

#optparse_addargs(options) ⇒ Object

Generate the list of valid arguments, in a format that OptionParser can understand, and add them to the passed option list.



210
211
212
213
214
215
216
217
# File 'lib/puppet/settings.rb', line 210

def optparse_addargs(options)
  # Add all of the settings as valid options.
  self.each { |name, setting|
    options << setting.optparse_args
  }

  options
end

#override_default(param, value) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Create a new default value for the given setting. The default overrides are higher precedence than the defaults given in defaults.rb, but lower precedence than any other values for the setting. This allows one setting a to change the default of setting b, but still allow a user to provide a value for setting b.

Parameters:

  • param (Symbol)

    the name of the setting

  • value (Object)

    the new default value for the setting



192
193
194
195
# File 'lib/puppet/settings.rb', line 192

def override_default(param, value)
  @value_sets[:overridden_defaults].set(param, value)
  unsafe_flush_cache
end

#parse_config(text, file = "text") ⇒ Object



576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
# File 'lib/puppet/settings.rb', line 576

def parse_config(text, file = "text")
  begin
    data = @config_file_parser.parse_file(file, text, ALLOWED_SECTION_NAMES)
  rescue => detail
    Puppet.log_exception(detail, "Could not parse #{file}: #{detail}")
    return
  end

  # If we get here and don't have any data, we just return and don't muck with the current state of the world.
  return if data.nil?

  # If we get here then we have some data, so we need to clear out any
  # previous settings that may have come from config files.
  unsafe_clear(false, false)

  # Screen settings which have been deprecated and removed from puppet.conf
  # but are still valid on the command line and/or in environment.conf
  screen_non_puppet_conf_settings(data)

  # Make note of deprecated settings we will warn about later in initialization
  record_deprecations_from_puppet_conf(data)

  # And now we can repopulate with the values from our last parsing of the config files.
  @configuration_file = data

  # Determine our environment, if we have one.
  if @config[:environment]
    env = self.value(:environment).to_sym
  else
    env = NONE
  end

  # Call any hooks we should be calling.
  value_sets = value_sets_for(env, preferred_run_mode)
  @config.values.select(&:has_hook?).each do |setting|
    value_sets.each do |source|
      if source.include?(setting.name)
        # We still have to use value to retrieve the value, since
        # we want the fully interpolated value, not $vardir/lib or whatever.
        # This results in extra work, but so few of the settings
        # will have associated hooks that it ends up being less work this
        # way overall.
        if setting.call_hook_on_initialize?
          @hooks_to_call_on_application_initialization |= [ setting ]
        else
          setting.handle(ChainedValues.new(
            preferred_run_mode,
            env,
            value_sets,
            @config).interpolate(setting.name))
        end
        break
      end
    end
  end

  call_hooks_deferred_to_application_initialization :ignore_interpolation_dependency_errors => true
  
end

#parse_file(file, allowed_sections = []) ⇒ Puppet::Settings::ConfigFile::Conf

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

This method just turns a file into a new ConfigFile::Conf instance

Parameters:

  • file (String)

    absolute path to the configuration file

Returns:



1226
1227
1228
# File 'lib/puppet/settings.rb', line 1226

def parse_file(file, allowed_sections = [])
  @config_file_parser.parse_file(file, read_file(file), allowed_sections)
end

#patch_value(param, value, type) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Patches the value for a param in a section. This method is required to support the use case of unifying --dns-alt-names and --dns_alt_names in the certificate face. Ideally this should be cleaned up. See PUP-3684 for more information. For regular use of setting a value, the method []= should be used.



961
962
963
964
965
966
# File 'lib/puppet/settings.rb', line 961

def patch_value(param, value, type)
  if @value_sets[type]
    @value_sets[type].set(param, value)
    unsafe_flush_cache
  end
end

#persection(section) ⇒ Object

Iterate across all of the objects in a given section.



769
770
771
772
773
774
775
776
# File 'lib/puppet/settings.rb', line 769

def persection(section)
  section = section.to_sym
  self.each { |name, obj|
    if obj.section == section
      yield obj
    end
  }
end

#preferred_run_modeObject

The currently configured run mode that is preferred for constructing the application configuration.



556
557
558
# File 'lib/puppet/settings.rb', line 556

def preferred_run_mode
  @preferred_run_mode_name || :user
end

#preferred_run_mode=(mode) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

PRIVATE! This only exists because we need a hook to validate the run mode when it's being set, and it should never, ever, ever, ever be called from outside of this file. This method is also called when --run_mode MODE is used on the command line to set the default

Parameters:

  • mode (String|Symbol)

    the name of the mode to have in effect

Raises:



566
567
568
569
570
571
572
573
574
# File 'lib/puppet/settings.rb', line 566

def preferred_run_mode=(mode)
  mode = mode.to_s.downcase.intern
  raise ValidationError, "Invalid run mode '#{mode}'" unless [:server, :master, :agent, :user].include?(mode)
  @preferred_run_mode_name = mode
  # Changing the run mode has far-reaching consequences. Flush any cached
  # settings so they will be re-generated.
  flush_cache
  mode
end

Prints the contents of a config file with the available config settings, or it prints a single value of a config setting.



498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
# File 'lib/puppet/settings.rb', line 498

def print_config_options
  if Puppet::Util::Log.sendlevel?(:info)
    Puppet::Util::Log.newdestination(:console)
    message = (_("Using --configprint is deprecated. Use 'puppet config <subcommand>' instead."))
    Puppet.deprecation_warning(message)
  end

  env = value(:environment)
  val = value(:configprint)
  if val == "all"
    hash = {}
    each do |name, obj|
      val = value(name,env)
      val = val.inspect if val == ""
      hash[name] = val
    end
    hash.sort { |a,b| a[0].to_s <=> b[0].to_s }.each do |name, v|
      puts "#{name} = #{v}"
    end
  else
    val.split(/\s*,\s*/).sort.each do |v|
      if include?(v)
        #if there is only one value, just print it for back compatibility
        if v == val
          puts value(val,env)
          break
        end
        puts "#{v} = #{value(v,env)}"
      else
        puts "invalid setting: #{v}"
        return false
      end
    end
  end
  true
end


545
546
547
548
549
# File 'lib/puppet/settings.rb', line 545

def print_configs
  return print_config_options if value(:configprint) != ""
  return generate_config if value(:genconfig)
  generate_manifest if value(:genmanifest)
end

Returns:

  • (Boolean)


551
552
553
# File 'lib/puppet/settings.rb', line 551

def print_configs?
  (value(:configprint) != "" || value(:genconfig) || value(:genmanifest)) && true
end

#reparse_config_filesObject

Reparse our config file, if necessary.



779
780
781
782
783
784
785
786
787
788
# File 'lib/puppet/settings.rb', line 779

def reparse_config_files
  if files
    filename = any_files_changed?
    if filename
      Puppet.notice "Config file #{filename} changed; triggering re-parse of all config files."
      parse_config_files
      reuse
    end
  end
end

#reuseObject



813
814
815
816
817
818
# File 'lib/puppet/settings.rb', line 813

def reuse
  return unless defined?(@used)
  new = @used
  @used = []
  self.use(*new)
end

#searchpath(environment = nil, run_mode = preferred_run_mode) ⇒ Array<SearchPathElement>

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

The order in which to search for values.

Parameters:

  • environment (String, Symbol) (defaults to: nil)

    symbolic reference to an environment name

  • run_mode (Symbol) (defaults to: preferred_run_mode)

    symbolic reference to a Puppet run mode

Returns:



853
854
855
856
857
# File 'lib/puppet/settings.rb', line 853

def searchpath(environment = nil, run_mode = preferred_run_mode)
  searchpath = configsearchpath(environment, run_mode)
  searchpath << SearchPathElement.new(:application_defaults, :values)
  searchpath << SearchPathElement.new(:overridden_defaults, :values)
end

#searchpath_values(source) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Get values from a search path entry.



908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
# File 'lib/puppet/settings.rb', line 908

def searchpath_values(source)
  case source.type
  when :values
    @value_sets[source.name]
  when :section
    section = @configuration_file.sections[source.name] if @configuration_file
    if section
      ValuesFromSection.new(source.name, section)
    end
  when :environment
    ValuesFromEnvironmentConf.new(source.name)
  else
    raise Puppet::DevError, _("Unknown searchpath case: %{source_type} for the %{source} settings path element.") % { source_type: source.type, source: source}
  end
end

#service_group_available?Boolean

Returns:

  • (Boolean)


875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
# File 'lib/puppet/settings.rb', line 875

def service_group_available?
  return @service_group_available if defined?(@service_group_available)

  if self[:group]
    group = Puppet::Type.type(:group).new :name => self[:group], :audit => :ensure

    if group.suitable?
      @service_group_available = group.exists?
    else
      raise Puppet::Error, (_("Cannot manage group permissions, because the provider for '%{name}' is not functional") % { name: group })
    end
  else
    @service_group_available = false
  end
end

#service_user_available?Boolean

Returns:

  • (Boolean)


859
860
861
862
863
864
865
866
867
868
869
870
871
872
873
# File 'lib/puppet/settings.rb', line 859

def service_user_available?
  return @service_user_available if defined?(@service_user_available)

  if self[:user]
    user = Puppet::Type.type(:user).new :name => self[:user], :audit => :ensure

    if user.suitable?
      @service_user_available = user.exists?
    else
      raise Puppet::Error, (_("Cannot manage owner permissions, because the provider for '%{name}' is not functional") % { name: user })
    end
  else
    @service_user_available = false
  end
end

#set_by_cli(param) ⇒ Object?

Allow later inspection to determine if the setting was set on the command line, or through some other code path. Used for the dns_alt_names option during cert generate. --daniel 2011-10-18

Parameters:

  • param (String, Symbol)

    the setting to look up

Returns:

  • (Object, nil)

    the value of the setting or nil if unset



897
898
899
900
# File 'lib/puppet/settings.rb', line 897

def set_by_cli(param)
  param = param.to_sym
  @value_sets[:cli].lookup(param)
end

#set_by_cli?(param) ⇒ Boolean

Returns:

  • (Boolean)


902
903
904
# File 'lib/puppet/settings.rb', line 902

def set_by_cli?(param)
  !!set_by_cli(param)
end

#set_by_config?(param, environment = nil, run_mode = preferred_run_mode) ⇒ Boolean

Allow later inspection to determine if the setting was set by user config, rather than a default setting.

Returns:

  • (Boolean)


926
927
928
929
930
931
932
933
934
# File 'lib/puppet/settings.rb', line 926

def set_by_config?(param, environment = nil, run_mode = preferred_run_mode)
  param = param.to_sym
  configsearchpath(environment, run_mode).any? do |source|
    vals = searchpath_values(source)
    if vals
      vals.lookup(param)
    end
  end
end

#set_in_section(param, section) ⇒ Object?

Allow later inspection to determine if the setting was set in a specific section

Parameters:

  • param (String, Symbol)

    the setting to look up

  • section (Symbol)

    the section in which to look up the setting

Returns:

  • (Object, nil)

    the value of the setting or nil if unset



942
943
944
945
946
947
948
# File 'lib/puppet/settings.rb', line 942

def set_in_section(param, section)
  param = param.to_sym
  vals = searchpath_values(SearchPathElement.new(section, :section))
  if vals
    vals.lookup(param)
  end
end

#set_in_section?(param, section) ⇒ Boolean

Returns:

  • (Boolean)


950
951
952
# File 'lib/puppet/settings.rb', line 950

def set_in_section?(param, section)
  !!set_in_section(param, section)
end

#setting(param) ⇒ Puppet::Settings::BaseSetting

Returns a given setting by name

Parameters:

  • name (Symbol)

    The name of the setting to fetch

Returns:



450
451
452
453
# File 'lib/puppet/settings.rb', line 450

def setting(param)
  param = param.to_sym
  @config[param]
end

#stringify_settings(section, settings = :all) ⇒ Object



89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
# File 'lib/puppet/settings.rb', line 89

def stringify_settings(section, settings = :all)
  values_from_the_selected_section =
    values(nil, section.to_sym)

  loader_settings = {
    :environmentpath => values_from_the_selected_section.interpolate(:environmentpath),
    :basemodulepath => values_from_the_selected_section.interpolate(:basemodulepath),
  }

  Puppet.override(Puppet.base_context(loader_settings),
                  _("New environment loaders generated from the requested section.")) do
    # And now we can lookup values that include those from environments configured from
    # the requested section
    values = values(Puppet[:environment].to_sym, section.to_sym)

    to_be_rendered = {}
    settings = Puppet.settings.to_a.collect(&:first) if settings == :all
    settings.sort.each do |setting_name|
      to_be_rendered[setting_name] = values.print(setting_name.to_sym)
    end

    stringifyhash(to_be_rendered)
  end
end

#stringifyhash(hash) ⇒ Object



114
115
116
117
118
119
120
121
122
123
124
125
126
127
# File 'lib/puppet/settings.rb', line 114

def stringifyhash(hash)
  newhash = {}
  hash.each do |key, val|
    key = key.to_s
    if val.is_a? Hash
      newhash[key] = stringifyhash(val)
    elsif val.is_a? Symbol
      newhash[key] = val.to_s
    else
      newhash[key] = val
    end
  end
  newhash
end

#to_catalog(*sections) ⇒ Object

Convert the settings we manage into a catalog full of resources that model those settings.



1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
# File 'lib/puppet/settings.rb', line 1038

def to_catalog(*sections)
  sections = nil if sections.empty?

  catalog = Puppet::Resource::Catalog.new("Settings", Puppet::Node::Environment::NONE)
  @config.keys.find_all { |key| @config[key].is_a?(FileSetting) }.each do |key|
    file = @config[key]
    next if file.value.nil?
    next unless (sections.nil? or sections.include?(file.section))
    resource = file.to_resource
    next unless resource
    next if catalog.resource(resource.ref)

    Puppet.debug {"Using settings: adding file resource '#{key}': '#{resource.inspect}'"}

    catalog.add_resource(resource)
  end

  add_user_resources(catalog, sections)
  add_environment_resources(catalog, sections)

  catalog
end

#to_configObject

Convert our list of config settings into a configuration file.



1062
1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
1080
1081
1082
1083
1084
1085
1086
1087
# File 'lib/puppet/settings.rb', line 1062

def to_config
  str = %{The configuration file for #{Puppet.run_mode.name}.  Note that this file
is likely to have unused settings in it; any setting that's
valid anywhere in Puppet can be in any config file, even if it's not used.

Every section can specify three special parameters: owner, group, and mode.
These parameters affect the required permissions of any files specified after
their specification.  Puppet will sometimes use these parameters to check its
own configured state, so they can be used to make Puppet a bit more self-managing.

The file format supports octothorpe-commented lines, but not partial-line comments.

Generated on #{Time.now}.

}.gsub(/^/, "# ")

#         Add a section heading that matches our name.
  str += "[#{preferred_run_mode}]\n"
  eachsection do |section|
    persection(section) do |obj|
      str += obj.to_config + "\n" unless obj.name == :genconfig
    end
  end

  return str
end

#to_manifestObject

Convert to a parseable manifest



1090
1091
1092
1093
1094
1095
# File 'lib/puppet/settings.rb', line 1090

def to_manifest
  catalog = to_catalog
  catalog.resource_refs.collect do |ref|
    catalog.resource(ref).to_manifest
  end.join("\n\n")
end

#use(*sections) ⇒ Object

Create the necessary objects to use a section. This is idempotent; you can 'use' a section as many times as you want.



1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
# File 'lib/puppet/settings.rb', line 1099

def use(*sections)
  Puppet.warning(":master section deprecated in favor of :server section") if sections.include?(:master)

  # add :server if sections include :master or :master if sections include :server
  sections |= [:master, :server] if (sections & [:master, :server]).any?

  sections = sections.collect { |s| s.to_sym }
  sections = sections.reject { |s| @used.include?(s) }

  return if sections.empty?

  Puppet.debug { "Applying settings catalog for sections #{sections.join(', ')}" }

  begin
    catalog = to_catalog(*sections).to_ral
  rescue => detail
    Puppet.log_and_raise(detail, "Could not create resources for managing Puppet's files and directories in sections #{sections.inspect}: #{detail}")
  end

  catalog.host_config = false
  catalog.apply do |transaction|
    if transaction.any_failed?
      report = transaction.report
      status_failures = report.resource_statuses.values.select { |r| r.failed? }
      status_fail_msg = status_failures.
        collect(&:events).
        flatten.
        select { |event| event.status == 'failure' }.
        collect { |event| "#{event.resource}: #{event.message}" }.join("; ")

      raise "Got #{status_failures.length} failure(s) while initializing: #{status_fail_msg}"
    end
  end

  sections.each { |s| @used << s }
  @used.uniq!
end

#valid?(param) ⇒ Boolean

Returns:

  • (Boolean)


1137
1138
1139
1140
# File 'lib/puppet/settings.rb', line 1137

def valid?(param)
  param = param.to_sym
  @config.has_key?(param)
end

#value(param, environment = nil, bypass_interpolation = false) ⇒ Object

Find the correct value using our search path.

Parameters:

  • param (String, Symbol)

    The value to look up

  • environment (String, Symbol) (defaults to: nil)

    The environment to check for the value

  • bypass_interpolation (true, false) (defaults to: false)

    Whether to skip interpolation

Returns:

  • (Object)

    The looked up value

Raises:



1166
1167
1168
1169
# File 'lib/puppet/settings.rb', line 1166

def value(param, environment = nil, bypass_interpolation = false)
  environment &&= environment.to_sym
  value_sym(param.to_sym, environment, bypass_interpolation)
end

#value_sym(param, environment = nil, bypass_interpolation = false) ⇒ Object

Find the correct value using symbols and our search path.

Parameters:

  • param (Symbol)

    The value to look up

  • environment (Symbol) (defaults to: nil)

    The environment to check for the value

  • bypass_interpolation (true, false) (defaults to: false)

    Whether to skip interpolation

Returns:

  • (Object)

    The looked up value

Raises:



1180
1181
1182
1183
1184
1185
1186
1187
1188
1189
1190
1191
1192
1193
1194
1195
1196
1197
1198
1199
# File 'lib/puppet/settings.rb', line 1180

def value_sym(param, environment = nil, bypass_interpolation = false)
  # Check the cache first.  It needs to be a per-environment
  # cache so that we don't spread values from one env
  # to another.
  cached_env = @cache[environment || NONE]

  # Avoid two lookups in cache_env unless val is nil. When it is, it's important
  # to check if the key is included so that further processing (that will result
  # in nil again) is avoided.
  val = cached_env[param]
  return val if !val.nil? || cached_env.include?(param)

  # Short circuit to nil for undefined settings.
  return nil unless @config.include?(param)

  vals = values(environment, preferred_run_mode)
  val = bypass_interpolation ? vals.lookup(param) : vals.interpolate(param)
  cached_env[param] = val
  val
end

#values(environment, section) ⇒ Puppet::Settings::ChainedValues

Retrieve an object that can be used for looking up values of configuration settings.

Parameters:

  • environment (Symbol)

    The name of the environment in which to lookup

  • section (Symbol)

    The name of the configuration section in which to lookup

Returns:



1149
1150
1151
1152
1153
1154
1155
# File 'lib/puppet/settings.rb', line 1149

def values(environment, section)
  @values[environment][section] ||= ChainedValues.new(
    section,
    environment,
    value_sets_for(environment, section),
    @config)
end

#which_configuration_fileObject

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

TODO:

this code duplicates Util::RunMode#which_dir as described in #16637

(#15337) All of the logic to determine the configuration file to use should be centralized into this method. The simplified approach is:

  1. If there is an explicit configuration file, use that. (--confdir or --config)
  2. If we're running as a root process, use the system puppet.conf (usually /etc/puppetlabs/puppet/puppet.conf)
  3. Otherwise, use the user puppet.conf (usually ~/.puppetlabs/etc/puppet/puppet.conf)


1214
1215
1216
1217
1218
1219
1220
# File 'lib/puppet/settings.rb', line 1214

def which_configuration_file
  if explicit_config_file? or Puppet.features.root? then
    return main_config_file
  else
    return user_config_file
  end
end