Method: Puppet::Settings#values

Defined in:
lib/puppet/settings.rb

#values(environment, section) ⇒ Puppet::Settings::ChainedValues

Retrieve an object that can be used for looking up values of configuration settings.

Parameters:

  • environment (Symbol)

    The name of the environment in which to lookup

  • section (Symbol)

    The name of the configuration section in which to lookup

Returns:



1182
1183
1184
1185
1186
1187
1188
1189
# File 'lib/puppet/settings.rb', line 1182

def values(environment, section)
  @values[environment][section] ||= ChainedValues.new(
    section,
    environment,
    value_sets_for(environment, section),
    @config
  )
end