Class: Puppet::Transaction::EventManager Private

Inherits:
Object
  • Object
show all
Defined in:
lib/puppet/transaction/event_manager.rb

Overview

This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.

API:

  • private

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(transaction) ⇒ EventManager

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns a new instance of EventManager.

API:

  • private



6
7
8
9
10
# File 'lib/puppet/transaction/event_manager.rb', line 6

def initialize(transaction)
  @transaction = transaction
  @event_queues = {}
  @events = []
end

Instance Attribute Details

#eventsObject (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



4
5
6
# File 'lib/puppet/transaction/event_manager.rb', line 4

def events
  @events
end

#transactionObject (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



4
5
6
# File 'lib/puppet/transaction/event_manager.rb', line 4

def transaction
  @transaction
end

Instance Method Details

#dequeue_events_for_resource(target, callback) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



66
67
68
69
# File 'lib/puppet/transaction/event_manager.rb', line 66

def dequeue_events_for_resource(target, callback)
  target.info "Unscheduling #{callback} on #{target}"
  @event_queues[target][callback] = {} if @event_queues[target]
end

#process_events(resource) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Respond to any queued events for this resource.

API:

  • private



17
18
19
20
21
22
23
24
25
26
27
28
29
# File 'lib/puppet/transaction/event_manager.rb', line 17

def process_events(resource)
  restarted = false
  queued_events(resource) do |callback, events|
    r = process_callback(resource, callback, events)
    restarted ||= r
  end

  if restarted
    queue_events(resource, [resource.event(:name => :restarted, :status => "success")])

    transaction.resource_status(resource).restarted = true
  end
end

#queue_events(resource, events) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Queue events for other resources to respond to. All of these events have to be from the same resource.

API:

  • private



33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
# File 'lib/puppet/transaction/event_manager.rb', line 33

def queue_events(resource, events)
  #@events += events

  # Do some basic normalization so we're not doing so many
  # graph queries for large sets of events.
  events.inject({}) do |collection, event|
    collection[event.name] ||= []
    collection[event.name] << event
    collection
  end.collect do |name, list|
    # It doesn't matter which event we use - they all have the same source
    # and name here.
    event = list[0]

    # Collect the targets of any subscriptions to those events.  We pass
    # the parent resource in so it will override the source in the events,
    # since eval_generated children can't have direct relationships.
    received = (event.name != :restarted)
    relationship_graph.matching_edges(event, resource).each do |edge|
      received ||= true unless edge.target.is_a?(Puppet::Type.type(:whit))
      next unless method = edge.callback
      next unless edge.target.respond_to?(method)

      queue_events_for_resource(resource, edge.target, method, list)
    end
    @events << event if received

    queue_events_for_resource(resource, resource, :refresh, [event]) if resource.self_refresh? and ! resource.deleting?
  end

  dequeue_events_for_resource(resource, :refresh) if events.detect { |e| e.invalidate_refreshes }
end

#queue_events_for_resource(source, target, callback, events) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
# File 'lib/puppet/transaction/event_manager.rb', line 71

def queue_events_for_resource(source, target, callback, events)
  whit = Puppet::Type.type(:whit)

  # The message that a resource is refreshing the completed-whit for its own class
  # is extremely counter-intuitive. Basically everything else is easy to understand,
  # if you suppress the whit-lookingness of the whit resources
  refreshing_c_whit = target.is_a?(whit) && target.name =~ /^completed_/

  if refreshing_c_whit
    source.debug "The container #{target} will propagate my #{callback} event"
  else
    source.info "Scheduling #{callback} of #{target}"
  end

  @event_queues[target] ||= {}
  @event_queues[target][callback] ||= []
  @event_queues[target][callback].concat(events)
end

#queued_events(resource) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



90
91
92
93
94
95
# File 'lib/puppet/transaction/event_manager.rb', line 90

def queued_events(resource)
  return unless callbacks = @event_queues[resource]
  callbacks.each do |callback, events|
    yield callback, events unless events.empty?
  end
end

#relationship_graphObject

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



12
13
14
# File 'lib/puppet/transaction/event_manager.rb', line 12

def relationship_graph
  transaction.relationship_graph
end