Class: Puppet::Context::TrustedInformation Private

Inherits:
Object
  • Object
show all
Defined in:
lib/puppet/context/trusted_information.rb

Overview

This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.

API:

  • private

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(authenticated, certname, extensions) ⇒ TrustedInformation

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns a new instance of TrustedInformation.

API:

  • private



30
31
32
33
34
35
36
37
38
39
40
41
42
# File 'lib/puppet/context/trusted_information.rb', line 30

def initialize(authenticated, certname, extensions)
  @authenticated = authenticated.freeze
  @certname = certname.freeze
  @extensions = extensions.freeze
  if @certname
    hostname, domain = @certname.split('.', 2)
  else
    hostname = nil
    domain = nil
  end
  @hostname = hostname.freeze
  @domain = domain.freeze
end

Instance Attribute Details

#authenticatedString, Boolean (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

one of ‘remote’, ‘local’, or false, where ‘remote’ is authenticated via cert, ‘local’ is trusted by virtue of running on the same machine (not a remote request), and false is an unauthenticated remote request.

Returns:

API:

  • private



8
9
10
# File 'lib/puppet/context/trusted_information.rb', line 8

def authenticated
  @authenticated
end

#certnameString (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

The validated certificate name used for the request

Returns:

API:

  • private



13
14
15
# File 'lib/puppet/context/trusted_information.rb', line 13

def certname
  @certname
end

#domainString (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

The domain name derived from the validated certificate name

Returns:

API:

  • private



23
24
25
# File 'lib/puppet/context/trusted_information.rb', line 23

def domain
  @domain
end

#extensionsHash{Object => Object} (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Extra information that comes from the trusted certificate’s extensions.

Returns:

API:

  • private



18
19
20
# File 'lib/puppet/context/trusted_information.rb', line 18

def extensions
  @extensions
end

#hostnameString (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

The hostname derived from the validated certificate name

Returns:

API:

  • private



28
29
30
# File 'lib/puppet/context/trusted_information.rb', line 28

def hostname
  @hostname
end

Class Method Details

.local(node) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



60
61
62
63
64
65
# File 'lib/puppet/context/trusted_information.rb', line 60

def self.local(node)
  # Always trust local data by picking up the available parameters.
  client_cert = node ? node.parameters['clientcert'] : nil

  new('local', client_cert, {})
end

.remote(authenticated, node_name, certificate) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
# File 'lib/puppet/context/trusted_information.rb', line 44

def self.remote(authenticated, node_name, certificate)
  if authenticated
    extensions = {}
    if certificate.nil?
      Puppet.info('TrustedInformation expected a certificate, but none was given.')
    else
      extensions = Hash[certificate.custom_extensions.collect do |ext|
        [ext['oid'].freeze, ext['value'].freeze]
      end]
    end
    new('remote', node_name, extensions)
  else
    new(false, nil, {})
  end
end

Instance Method Details

#to_hObject

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

API:

  • private



67
68
69
70
71
72
73
74
75
# File 'lib/puppet/context/trusted_information.rb', line 67

def to_h
  {
    'authenticated'.freeze => authenticated,
    'certname'.freeze => certname,
    'extensions'.freeze => extensions,
    'hostname'.freeze => hostname,
    'domain'.freeze => domain,
  }.freeze
end