Class: Puppet::Indirector::REST
- Includes:
- Network::HTTP::API::V1
- Defined in:
- lib/puppet/indirector/rest.rb
Overview
Access objects via REST
Direct Known Subclasses
FileBucketFile::Rest, CertificateStatus::Rest, FileContent::Rest, FileMetadata::Rest, InstrumentationData::Rest, InstrumentationListener::Rest, InstrumentationProbe::Rest, Puppet::Indirector::ResourceType::Rest, Status::Rest, Node::Facts::InventoryService, Node::Facts::Rest, Node::Rest, Resource::Catalog::Rest, Resource::Rest, Run::Rest, SSL::Certificate::Rest, SSL::CertificateRequest::Rest, SSL::CertificateRevocationList::Rest, Transaction::Report::Rest
Constant Summary
Constants included from Network::HTTP::API::V1
Network::HTTP::API::V1::METHOD_MAP
Constants included from Util
Util::AbsolutePathPosix, Util::AbsolutePathWindows, Util::DEFAULT_POSIX_MODE, Util::DEFAULT_WINDOWS_MODE
Constants included from Util::POSIX
Util::POSIX::LOCALE_ENV_VARS, Util::POSIX::USER_ENV_VARS
Constants included from Util::SymbolicFileMode
Util::SymbolicFileMode::SetGIDBit, Util::SymbolicFileMode::SetUIDBit, Util::SymbolicFileMode::StickyBit, Util::SymbolicFileMode::SymbolicMode, Util::SymbolicFileMode::SymbolicSpecialToBit
Constants included from Util::Docs
Class Attribute Summary collapse
- .port_setting ⇒ Object readonly
- .server_setting ⇒ Object readonly
Attributes included from Util::Docs
Class Method Summary collapse
- .port ⇒ Object
- .server ⇒ Object
- .srv_service ⇒ Object
-
.use_port_setting(setting) ⇒ Object
Specify the setting that we should use to get the port.
-
.use_server_setting(setting) ⇒ Object
Specify the setting that we should use to get the server name.
-
.use_srv_service(service) ⇒ Object
Specify the service to use when doing SRV record lookup.
Instance Method Summary collapse
- #add_profiling_header(headers) ⇒ Object
- #destroy(request) ⇒ Object
-
#do_request(request) ⇒ Object
Encapsulate call to request.do_request with the arguments from this class Then yield to the code block that was called in We certainly could have retained the full request.do_request(…) { |r| … } but this makes the code much cleaner and we only then actually make the call to request.do_request from here, thus if we change what we pass or how we get it, we only need to change it here.
- #find(request) ⇒ Object
- #head(request) ⇒ Object
-
#headers ⇒ Object
Provide appropriate headers.
- #http_delete(request, path, headers = nil, *args) ⇒ Object
- #http_get(request, path, headers = nil, *args) ⇒ Object
- #http_head(request, path, headers = nil, *args) ⇒ Object
- #http_post(request, path, data, headers = nil, *args) ⇒ Object
- #http_put(request, path, data, headers = nil, *args) ⇒ Object
- #http_request(method, request, *args) ⇒ Object
- #network(request) ⇒ Object
- #save(request) ⇒ Object
- #search(request) ⇒ Object
- #validate_key(request) ⇒ Object
Methods included from Network::HTTP::API::V1
#indirection2uri, #indirection_method, #plurality, #pluralize, #request_to_uri_and_body, #uri2indirection
Methods inherited from Terminus
abstract_terminus?, #allow_remote_requests?, const2name, #indirection, indirection_name, inherited, #initialize, mark_as_abstract_terminus, model, #model, #name, name2const, register_terminus_class, terminus_class, terminus_classes, #terminus_type, #validate, #validate_model
Methods included from Util::InstanceLoader
#instance_docs, #instance_hash, #instance_load, #instance_loader, #instance_loading?, #loaded_instance, #loaded_instances
Methods included from Util
absolute_path?, activerecord_version, benchmark, binread, chuser, classproxy, deterministic_rand, execfail, execpipe, execute, exit_on_fail, logmethods, memory, path_to_uri, pretty_backtrace, proxy, replace_file, safe_posix_fork, symbolizehash, thinmark, uri_to_path, which, withenv, withumask
Methods included from Util::POSIX
#get_posix_field, #gid, #idfield, #methodbyid, #methodbyname, #search_posix_field, #uid
Methods included from Util::SymbolicFileMode
#normalize_symbolic_mode, #symbolic_mode_to_int, #valid_symbolic_mode?
Methods included from Util::Docs
#desc, #dochook, #doctable, #markdown_definitionlist, #markdown_header, #nodoc?, #pad, scrub
Constructor Details
This class inherits a constructor from Puppet::Indirector::Terminus
Class Attribute Details
.port_setting ⇒ Object (readonly)
15 16 17 |
# File 'lib/puppet/indirector/rest.rb', line 15 def port_setting @port_setting end |
.server_setting ⇒ Object (readonly)
15 16 17 |
# File 'lib/puppet/indirector/rest.rb', line 15 def server_setting @server_setting end |
Class Method Details
.port ⇒ Object
41 42 43 |
# File 'lib/puppet/indirector/rest.rb', line 41 def self.port Puppet.settings[port_setting || :masterport].to_i end |
.server ⇒ Object
37 38 39 |
# File 'lib/puppet/indirector/rest.rb', line 37 def self.server Puppet.settings[server_setting || :server] end |
.srv_service ⇒ Object
33 34 35 |
# File 'lib/puppet/indirector/rest.rb', line 33 def self.srv_service @srv_service || :puppet end |
.use_port_setting(setting) ⇒ Object
Specify the setting that we should use to get the port.
24 25 26 |
# File 'lib/puppet/indirector/rest.rb', line 24 def self.use_port_setting(setting) @port_setting = setting end |
.use_server_setting(setting) ⇒ Object
Specify the setting that we should use to get the server name.
19 20 21 |
# File 'lib/puppet/indirector/rest.rb', line 19 def self.use_server_setting(setting) @server_setting = setting end |
.use_srv_service(service) ⇒ Object
Specify the service to use when doing SRV record lookup
29 30 31 |
# File 'lib/puppet/indirector/rest.rb', line 29 def self.use_srv_service(service) @srv_service = service end |
Instance Method Details
#add_profiling_header(headers) ⇒ Object
50 51 52 53 54 55 |
# File 'lib/puppet/indirector/rest.rb', line 50 def add_profiling_header(headers) if (Puppet[:profile]) headers[Puppet::Network::HTTP::HEADER_ENABLE_PROFILING] = "true" end headers end |
#destroy(request) ⇒ Object
139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 |
# File 'lib/puppet/indirector/rest.rb', line 139 def destroy(request) raise ArgumentError, "DELETE does not accept options" unless request..empty? response = do_request(request) do |request| http_delete(request, indirection2uri(request), headers) end if is_http_200?(response) check_master_version(response) content_type, body = parse_response(response) deserialize_destroy(content_type, body) else nil end end |
#do_request(request) ⇒ Object
Encapsulate call to request.do_request with the arguments from this class Then yield to the code block that was called in We certainly could have retained the full request.do_request(…) { |r| … } but this makes the code much cleaner and we only then actually make the call to request.do_request from here, thus if we change what we pass or how we get it, we only need to change it here.
177 178 179 |
# File 'lib/puppet/indirector/rest.rb', line 177 def do_request(request) request.do_request(self.class.srv_service, self.class.server, self.class.port) { |request| yield(request) } end |
#find(request) ⇒ Object
87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 |
# File 'lib/puppet/indirector/rest.rb', line 87 def find(request) uri, body = request_to_uri_and_body(request) uri_with_query_string = "#{uri}?#{body}" response = do_request(request) do |request| # WEBrick in Ruby 1.9.1 only supports up to 1024 character lines in an HTTP request # http://redmine.ruby-lang.org/issues/show/3991 if "GET #{uri_with_query_string} HTTP/1.1\r\n".length > 1024 http_post(request, uri, body, headers) else http_get(request, uri_with_query_string, headers) end end if is_http_200?(response) check_master_version(response) content_type, body = parse_response(response) result = deserialize_find(content_type, body) result.name = request.key if result.respond_to?(:name=) result else nil end end |
#head(request) ⇒ Object
112 113 114 115 116 117 118 119 120 121 122 123 |
# File 'lib/puppet/indirector/rest.rb', line 112 def head(request) response = do_request(request) do |request| http_head(request, indirection2uri(request), headers) end if is_http_200?(response) check_master_version(response) true else false end end |
#headers ⇒ Object
Provide appropriate headers.
46 47 48 |
# File 'lib/puppet/indirector/rest.rb', line 46 def headers add_accept_encoding({"Accept" => model.supported_formats.join(", ")}) end |
#http_delete(request, path, headers = nil, *args) ⇒ Object
74 75 76 |
# File 'lib/puppet/indirector/rest.rb', line 74 def http_delete(request, path, headers = nil, *args) http_request(:delete, request, path, add_profiling_header(headers), *args) end |
#http_get(request, path, headers = nil, *args) ⇒ Object
62 63 64 |
# File 'lib/puppet/indirector/rest.rb', line 62 def http_get(request, path, headers = nil, *args) http_request(:get, request, path, add_profiling_header(headers), *args) end |
#http_head(request, path, headers = nil, *args) ⇒ Object
70 71 72 |
# File 'lib/puppet/indirector/rest.rb', line 70 def http_head(request, path, headers = nil, *args) http_request(:head, request, path, add_profiling_header(headers), *args) end |
#http_post(request, path, data, headers = nil, *args) ⇒ Object
66 67 68 |
# File 'lib/puppet/indirector/rest.rb', line 66 def http_post(request, path, data, headers = nil, *args) http_request(:post, request, path, data, add_profiling_header(headers), *args) end |
#http_put(request, path, data, headers = nil, *args) ⇒ Object
78 79 80 |
# File 'lib/puppet/indirector/rest.rb', line 78 def http_put(request, path, data, headers = nil, *args) http_request(:put, request, path, data, add_profiling_header(headers), *args) end |
#http_request(method, request, *args) ⇒ Object
82 83 84 85 |
# File 'lib/puppet/indirector/rest.rb', line 82 def http_request(method, request, *args) conn = network(request) conn.send(method, *args) end |
#network(request) ⇒ Object
57 58 59 60 |
# File 'lib/puppet/indirector/rest.rb', line 57 def network(request) Puppet::Network::HttpPool.http_instance(request.server || self.class.server, request.port || self.class.port) end |
#save(request) ⇒ Object
155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 |
# File 'lib/puppet/indirector/rest.rb', line 155 def save(request) raise ArgumentError, "PUT does not accept options" unless request..empty? response = do_request(request) do |request| http_put(request, indirection2uri(request), request.instance.render, headers.merge({ "Content-Type" => request.instance.mime })) end if is_http_200?(response) check_master_version(response) content_type, body = parse_response(response) deserialize_save(content_type, body) else nil end end |
#search(request) ⇒ Object
125 126 127 128 129 130 131 132 133 134 135 136 137 |
# File 'lib/puppet/indirector/rest.rb', line 125 def search(request) response = do_request(request) do |request| http_get(request, indirection2uri(request), headers) end if is_http_200?(response) check_master_version(response) content_type, body = parse_response(response) deserialize_search(content_type, body) || [] else [] end end |
#validate_key(request) ⇒ Object
181 182 183 |
# File 'lib/puppet/indirector/rest.rb', line 181 def validate_key(request) # Validation happens on the remote end end |