Class: Puppet::Indirector::Request
- Defined in:
- lib/puppet/indirector/request.rb
Overview
This class encapsulates all of the information you need to make an Indirection call, and as a result also handles REST calls. It’s somewhat analogous to an HTTP Request object, except tuned for our Indirector.
Constant Summary collapse
- OPTION_ATTRIBUTES =
[:ip, :node, :authenticated, :ignore_terminus, :ignore_cache, :instance, :environment]
Instance Attribute Summary collapse
- #authenticated ⇒ Object
- #ignore_cache ⇒ Object
- #ignore_terminus ⇒ Object
- #indirection_name ⇒ Object
- #instance ⇒ Object
- #ip ⇒ Object
- #key ⇒ Object
- #method ⇒ Object
- #node ⇒ Object
- #options ⇒ Object
- #port ⇒ Object
- #protocol ⇒ Object
- #server ⇒ Object
- #uri ⇒ Object
Class Method Summary collapse
Instance Method Summary collapse
-
#authenticated? ⇒ Boolean
Is this an authenticated request?.
- #do_request(srv_service = :puppet, default_server = , default_port = , &block) ⇒ Object
- #encode_params(params) ⇒ Object
- #environment ⇒ Object
- #environment=(env) ⇒ Object
- #escaped_key ⇒ Object
- #expand_into_parameters(data) ⇒ Object
- #expand_primitive_types_into_parameters(data) ⇒ Object
-
#ignore_cache? ⇒ Boolean
LAK:NOTE This is a messy interface to the cache, and it’s only used by the Configurer class.
- #ignore_terminus? ⇒ Boolean
-
#indirection ⇒ Object
Look up the indirection based on the name provided.
-
#initialize(indirection_name, method, key, instance, options = {}) ⇒ Request
constructor
A new instance of Request.
- #model ⇒ Object
-
#plural? ⇒ Boolean
Are we trying to interact with multiple resources, or just one?.
-
#query_string ⇒ Object
Create the query string, if options are present.
- #remote? ⇒ Boolean
- #replace_arrays_with_yaml ⇒ Object
- #to_data_hash ⇒ Object
- #to_hash ⇒ Object
- #to_pson(*args) ⇒ Object
- #to_pson_data_hash ⇒ Object
- #to_s ⇒ Object
-
#use_cache? ⇒ Boolean
Should we allow use of the cached object?.
Constructor Details
#initialize(indirection_name, method, key, instance, options = {}) ⇒ Request
Returns a new instance of Request.
107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 |
# File 'lib/puppet/indirector/request.rb', line 107 def initialize(indirection_name, method, key, instance, = {}) @instance = instance ||= {} self.indirection_name = indirection_name self.method = method = .inject({}) { |hash, ary| hash[ary[0].to_sym] = ary[1]; hash } set_attributes() = if key # If the request key is a URI, then we need to treat it specially, # because it rewrites the key. We could otherwise strip server/port/etc # info out in the REST class, but it seemed bad design for the REST # class to rewrite the key. if key.to_s =~ /^\w+:\// and not Puppet::Util.absolute_path?(key.to_s) # it's a URI set_uri_key(key) else @key = key end end @key = @instance.name if ! @key and @instance end |
Instance Attribute Details
#authenticated ⇒ Object
11 12 13 |
# File 'lib/puppet/indirector/request.rb', line 11 def authenticated @authenticated end |
#ignore_cache ⇒ Object
11 12 13 |
# File 'lib/puppet/indirector/request.rb', line 11 def ignore_cache @ignore_cache end |
#ignore_terminus ⇒ Object
11 12 13 |
# File 'lib/puppet/indirector/request.rb', line 11 def ignore_terminus @ignore_terminus end |
#indirection_name ⇒ Object
15 16 17 |
# File 'lib/puppet/indirector/request.rb', line 15 def indirection_name @indirection_name end |
#instance ⇒ Object
11 12 13 |
# File 'lib/puppet/indirector/request.rb', line 11 def instance @instance end |
#protocol ⇒ Object
13 14 15 |
# File 'lib/puppet/indirector/request.rb', line 13 def protocol @protocol end |
Class Method Details
.from_pson(json) ⇒ Object
21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 |
# File 'lib/puppet/indirector/request.rb', line 21 def self.from_pson(json) raise ArgumentError, "No indirection name provided in json data" unless indirection_name = json['type'] raise ArgumentError, "No method name provided in json data" unless method = json['method'] raise ArgumentError, "No key provided in json data" unless key = json['key'] request = new(indirection_name, method, key, nil, json['attributes']) if instance = json['instance'] klass = Puppet::Indirector::Indirection.instance(request.indirection_name).model if instance.is_a?(klass) request.instance = instance else request.instance = klass.from_pson(instance) end end request end |
Instance Method Details
#authenticated? ⇒ Boolean
Is this an authenticated request?
73 74 75 76 |
# File 'lib/puppet/indirector/request.rb', line 73 def authenticated? # Double negative, so we just get true or false ! ! authenticated end |
#do_request(srv_service = :puppet, default_server = , default_port = , &block) ⇒ Object
239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 |
# File 'lib/puppet/indirector/request.rb', line 239 def do_request(srv_service=:puppet, default_server=Puppet.settings[:server], default_port=Puppet.settings[:masterport], &block) # We were given a specific server to use, so just use that one. # This happens if someone does something like specifying a file # source using a puppet:// URI with a specific server. return yield(self) if !self.server.nil? if Puppet.settings[:use_srv_records] Puppet::Network::Resolver.each_srv_record(Puppet.settings[:srv_domain], srv_service) do |srv_server, srv_port| begin self.server = srv_server self.port = srv_port return yield(self) rescue SystemCallError => e Puppet.warning "Error connecting to #{srv_server}:#{srv_port}: #{e.message}" end end end # ... Fall back onto the default server. Puppet.debug "No more servers left, falling back to #{default_server}:#{default_port}" if Puppet.settings[:use_srv_records] self.server = default_server self.port = default_port return yield(self) end |
#encode_params(params) ⇒ Object
218 219 220 221 222 |
# File 'lib/puppet/indirector/request.rb', line 218 def encode_params(params) params.collect do |key, value| "#{key}=#{CGI.escape(value.to_s)}" end.join("&") end |
#environment ⇒ Object
78 79 80 |
# File 'lib/puppet/indirector/request.rb', line 78 def environment @environment ||= Puppet::Node::Environment.new end |
#environment=(env) ⇒ Object
82 83 84 85 86 87 88 |
# File 'lib/puppet/indirector/request.rb', line 82 def environment=(env) @environment = if env.is_a?(Puppet::Node::Environment) env else Puppet::Node::Environment.new(env) end end |
#escaped_key ⇒ Object
90 91 92 |
# File 'lib/puppet/indirector/request.rb', line 90 def escaped_key URI.escape(key) end |
#expand_into_parameters(data) ⇒ Object
189 190 191 192 193 194 195 196 197 198 199 200 201 202 |
# File 'lib/puppet/indirector/request.rb', line 189 def (data) data.inject([]) do |params, key_value| key, value = key_value = case value when Array value.collect { |val| [key, val] } else [key_value] end params.concat(()) end end |
#expand_primitive_types_into_parameters(data) ⇒ Object
204 205 206 207 208 209 210 211 212 213 214 215 216 |
# File 'lib/puppet/indirector/request.rb', line 204 def (data) data.inject([]) do |params, key_value| key, value = key_value case value when nil params when true, false, String, Symbol, Fixnum, Bignum, Float params << [key, value] else raise ArgumentError, "HTTP REST queries cannot handle values of type '#{value.class}'" end end end |
#ignore_cache? ⇒ Boolean
LAK:NOTE This is a messy interface to the cache, and it’s only used by the Configurer class. I decided it was better to implement it now and refactor later, when we have a better design, than to spend another month coming up with a design now that might not be any better.
99 100 101 |
# File 'lib/puppet/indirector/request.rb', line 99 def ignore_cache? ignore_cache end |
#ignore_terminus? ⇒ Boolean
103 104 105 |
# File 'lib/puppet/indirector/request.rb', line 103 def ignore_terminus? ignore_terminus end |
#indirection ⇒ Object
Look up the indirection based on the name provided.
137 138 139 |
# File 'lib/puppet/indirector/request.rb', line 137 def indirection Puppet::Indirector::Indirection.instance(indirection_name) end |
#model ⇒ Object
146 147 148 149 |
# File 'lib/puppet/indirector/request.rb', line 146 def model raise ArgumentError, "Could not find indirection '#{indirection_name}'" unless i = indirection i.model end |
#plural? ⇒ Boolean
Are we trying to interact with multiple resources, or just one?
161 162 163 |
# File 'lib/puppet/indirector/request.rb', line 161 def plural? method == :search end |
#query_string ⇒ Object
Create the query string, if options are present.
166 167 168 169 170 171 172 173 174 175 176 177 178 |
# File 'lib/puppet/indirector/request.rb', line 166 def query_string return "" if .nil? || .empty? # For backward compatibility with older (pre-3.3) masters, # this puppet option allows serialization of query parameter # arrays as yaml. This can be removed when we remove yaml # support entirely. if Puppet.settings[:legacy_query_parameter_serialization] replace_arrays_with_yaml end "?" + encode_params((.to_a)) end |
#remote? ⇒ Boolean
264 265 266 |
# File 'lib/puppet/indirector/request.rb', line 264 def remote? self.node or self.ip end |
#replace_arrays_with_yaml ⇒ Object
180 181 182 183 184 185 186 187 |
# File 'lib/puppet/indirector/request.rb', line 180 def replace_arrays_with_yaml .each do |key, value| case value when Array [key] = YAML.dump(value) end end end |
#to_data_hash ⇒ Object
40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 |
# File 'lib/puppet/indirector/request.rb', line 40 def to_data_hash result = { 'type' => indirection_name, 'method' => method, 'key' => key } attributes = {} OPTION_ATTRIBUTES.each do |key| next unless value = send(key) attributes[key] = value end .each do |opt, value| attributes[opt] = value end result['attributes'] = attributes unless attributes.empty? result['instance'] = instance if instance result end |
#to_hash ⇒ Object
224 225 226 227 228 229 230 231 232 233 |
# File 'lib/puppet/indirector/request.rb', line 224 def to_hash result = .dup OPTION_ATTRIBUTES.each do |attribute| if value = send(attribute) result[attribute] = value end end result end |
#to_pson(*args) ⇒ Object
68 69 70 |
# File 'lib/puppet/indirector/request.rb', line 68 def to_pson(*args) to_pson_data_hash.to_pson(*args) end |
#to_pson_data_hash ⇒ Object
61 62 63 64 65 66 |
# File 'lib/puppet/indirector/request.rb', line 61 def to_pson_data_hash { 'document_type' => 'IndirectorRequest', 'data' => to_data_hash, } end |
#to_s ⇒ Object
235 236 237 |
# File 'lib/puppet/indirector/request.rb', line 235 def to_s return(uri ? uri : "/#{indirection_name}/#{key}") end |
#use_cache? ⇒ Boolean
Should we allow use of the cached object?
152 153 154 155 156 157 158 |
# File 'lib/puppet/indirector/request.rb', line 152 def use_cache? if defined?(@use_cache) ! ! use_cache else true end end |