Class: Puppet::Module
- Includes:
- Util::Logging
- Defined in:
- lib/puppet/module.rb,
lib/puppet/module/task.rb
Overview
Support for modules
Defined Under Namespace
Classes: Error, FaultyMetadata, IncompatibleModule, IncompatiblePlatform, InvalidFilePattern, InvalidName, MissingMetadata, MissingModule, Task, UnsupportedPlatform
Constant Summary collapse
- FILETYPES =
{ "manifests" => "manifests", "files" => "files", "templates" => "templates", "plugins" => "lib", "pluginfacts" => "facts.d", "locales" => "locales", }
Instance Attribute Summary collapse
-
#author ⇒ Object
Returns the value of attribute author.
-
#dependencies ⇒ Object
Returns the value of attribute dependencies.
-
#description ⇒ Object
Returns the value of attribute description.
-
#environment ⇒ Object
Returns the value of attribute environment.
-
#forge_name ⇒ Object
Returns the value of attribute forge_name.
-
#license ⇒ Object
Returns the value of attribute license.
-
#metadata ⇒ Object
readonly
Returns the value of attribute metadata.
-
#name ⇒ Object
readonly
Returns the value of attribute name.
-
#path ⇒ Object
readonly
Returns the value of attribute path.
-
#project_page ⇒ Object
Returns the value of attribute project_page.
-
#source ⇒ Object
Returns the value of attribute source.
-
#summary ⇒ Object
Returns the value of attribute summary.
-
#tasks ⇒ Object
readonly
Returns the value of attribute tasks.
-
#version ⇒ Object
Returns the value of attribute version.
Class Method Summary collapse
-
.find(modname, environment = nil) ⇒ Object
Find and return the
modulethatpathbelongs to. - .is_module_directory?(name, path) ⇒ Boolean
- .is_module_directory_name?(name) ⇒ Boolean
- .is_module_namespaced_name?(name) ⇒ Boolean
- .parse_range(range, strict) ⇒ Object private
Instance Method Summary collapse
- #==(other) ⇒ Object
- #all_manifests ⇒ Object
- #dependencies_as_modules ⇒ Object
- #has_external_facts? ⇒ Boolean
- #has_hiera_conf? ⇒ Boolean
- #has_metadata? ⇒ Boolean
-
#has_translations?(locale) ⇒ Boolean
Returns true if the module has translation files for the given locale.
- #hiera_conf_file ⇒ Object
-
#initialize(name, path, environment, strict_semver = true) ⇒ Module
constructor
A new instance of Module.
- #license_file ⇒ Object
- #load_metadata ⇒ Object
- #locale_directory ⇒ String
-
#match_manifests(rest) ⇒ Object
Return the list of manifests matching the given glob pattern, defaulting to ‘init.pp’ for empty modules.
- #metadata_file ⇒ Object
- #modulepath ⇒ Object
-
#plugin_directory ⇒ Object
Find all plugin directories.
- #plugin_fact_directory ⇒ Object
-
#puppetversion ⇒ Object
deprecated
Deprecated.
The puppetversion module metadata field is no longer used.
-
#puppetversion=(something) ⇒ Object
deprecated
Deprecated.
The puppetversion module metadata field is no longer used.
- #read_metadata ⇒ Object
- #required_by ⇒ Object
- #strict_semver? ⇒ Boolean
- #supports(name, version = nil) ⇒ Object
-
#task_file(name) ⇒ Object
This is a re-implementation of the Filetypes singular type method (e.g. ‘manifest(’my/manifest.pp’)‘. We don’t implement the full filetype “API” for tasks since tasks don’t map 1:1 onto files..
- #tasks_directory ⇒ Object
- #to_s ⇒ Object
-
#unmet_dependencies ⇒ Object
Identify and mark unmet dependencies.
-
#validate_puppet_version ⇒ Object
deprecated
Deprecated.
The puppetversion module metadata field is no longer used.
Methods included from Util::Logging
#clear_deprecation_warnings, #debug, #deprecation_warning, #format_exception, #get_deprecation_offender, #log_and_raise, #log_deprecations_to_file, #log_exception, #puppet_deprecation_warning, #send_log, setup_facter_logging!, #warn_once
Constructor Details
#initialize(name, path, environment, strict_semver = true) ⇒ Module
Returns a new instance of Module.
90 91 92 93 94 95 96 97 98 99 100 |
# File 'lib/puppet/module.rb', line 90 def initialize(name, path, environment, strict_semver = true) @name = name @path = path @strict_semver = strict_semver @environment = environment assert_validity @absolute_path_to_manifests = Puppet::FileSystem::PathPattern.absolute(manifests) end |
Instance Attribute Details
#author ⇒ Object
Returns the value of attribute author.
88 89 90 |
# File 'lib/puppet/module.rb', line 88 def @author end |
#dependencies ⇒ Object
Returns the value of attribute dependencies.
87 88 89 |
# File 'lib/puppet/module.rb', line 87 def dependencies @dependencies end |
#description ⇒ Object
Returns the value of attribute description.
88 89 90 |
# File 'lib/puppet/module.rb', line 88 def description @description end |
#environment ⇒ Object
Returns the value of attribute environment.
84 85 86 |
# File 'lib/puppet/module.rb', line 84 def environment @environment end |
#forge_name ⇒ Object
Returns the value of attribute forge_name.
87 88 89 |
# File 'lib/puppet/module.rb', line 87 def forge_name @forge_name end |
#license ⇒ Object
Returns the value of attribute license.
88 89 90 |
# File 'lib/puppet/module.rb', line 88 def license @license end |
#metadata ⇒ Object (readonly)
Returns the value of attribute metadata.
84 85 86 |
# File 'lib/puppet/module.rb', line 84 def @metadata end |
#name ⇒ Object (readonly)
Returns the value of attribute name.
84 85 86 |
# File 'lib/puppet/module.rb', line 84 def name @name end |
#path ⇒ Object (readonly)
Returns the value of attribute path.
84 85 86 |
# File 'lib/puppet/module.rb', line 84 def path @path end |
#project_page ⇒ Object
Returns the value of attribute project_page.
88 89 90 |
# File 'lib/puppet/module.rb', line 88 def project_page @project_page end |
#source ⇒ Object
Returns the value of attribute source.
88 89 90 |
# File 'lib/puppet/module.rb', line 88 def source @source end |
#summary ⇒ Object
Returns the value of attribute summary.
88 89 90 |
# File 'lib/puppet/module.rb', line 88 def summary @summary end |
#tasks ⇒ Object (readonly)
Returns the value of attribute tasks.
84 85 86 |
# File 'lib/puppet/module.rb', line 84 def tasks @tasks end |
#version ⇒ Object
Returns the value of attribute version.
88 89 90 |
# File 'lib/puppet/module.rb', line 88 def version @version end |
Class Method Details
.find(modname, environment = nil) ⇒ Object
Find and return the module that path belongs to. If path is absolute, or if there is no module whose name is the first component of path, return nil
32 33 34 35 36 37 |
# File 'lib/puppet/module.rb', line 32 def self.find(modname, environment = nil) return nil unless modname # Unless a specific environment is given, use the current environment env = environment ? Puppet.lookup(:environments).get!(environment) : Puppet.lookup(:current_environment) env.module(modname) end |
.is_module_directory?(name, path) ⇒ Boolean
39 40 41 42 43 44 |
# File 'lib/puppet/module.rb', line 39 def self.is_module_directory?(name, path) # it must be a directory fullpath = File.join(path, name) return false unless Puppet::FileSystem.directory?(fullpath) return is_module_directory_name?(name) end |
.is_module_directory_name?(name) ⇒ Boolean
46 47 48 49 50 |
# File 'lib/puppet/module.rb', line 46 def self.is_module_directory_name?(name) # it must match an installed module name according to forge validator return true if name =~ /^[a-z][a-z0-9_]*$/ return false end |
.is_module_namespaced_name?(name) ⇒ Boolean
52 53 54 55 56 |
# File 'lib/puppet/module.rb', line 52 def self.is_module_namespaced_name?(name) # it must match the full module name according to forge validator return true if name =~ /^[a-zA-Z0-9]+[-][a-z][a-z0-9_]*$/ return false end |
.parse_range(range, strict) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 |
# File 'lib/puppet/module.rb', line 59 def self.parse_range(range, strict) @parse_range_method ||= SemanticPuppet::VersionRange.method(:parse) if @parse_range_method.arity == 1 @semver_gem_version ||= SemanticPuppet::Version.parse(SemanticPuppet::VERSION) # Give user a heads-up if the desired strict setting cannot be honored if strict if @semver_gem_version.major < 1 Puppet.warn_once('strict_version_ranges', 'version_range_cannot_be_strict', _('VersionRanges will never be strict when using non-vendored SemanticPuppet gem, version %{version}') % { version: @semver_gem_version}, :default, :default, :notice) end else if @semver_gem_version.major >= 1 Puppet.warn_once('strict_version_ranges', 'version_range_always_strict', _('VersionRanges will always be strict when using non-vendored SemanticPuppet gem, version %{version}') % { version: @semver_gem_version}, :default, :default, :notice) end end @parse_range_method.call(range) else @parse_range_method.call(range, strict) end end |
Instance Method Details
#==(other) ⇒ Object
428 429 430 431 432 433 |
# File 'lib/puppet/module.rb', line 428 def ==(other) self.name == other.name && self.version == other.version && self.path == other.path && self.environment == other.environment end |
#all_manifests ⇒ Object
267 268 269 270 271 |
# File 'lib/puppet/module.rb', line 267 def all_manifests return [] unless Puppet::FileSystem.exist?(manifests) Dir.glob(File.join(manifests, '**', '*.pp')) end |
#dependencies_as_modules ⇒ Object
334 335 336 337 338 339 340 341 342 343 |
# File 'lib/puppet/module.rb', line 334 def dependencies_as_modules dependent_modules = [] dependencies and dependencies.each do |dep| , dep_name = dep["name"].split('/') found_module = environment.module(dep_name) dependent_modules << found_module if found_module end dependent_modules end |
#has_external_facts? ⇒ Boolean
319 320 321 |
# File 'lib/puppet/module.rb', line 319 def has_external_facts? File.directory?(plugin_fact_directory) end |
#has_hiera_conf? ⇒ Boolean
287 288 289 |
# File 'lib/puppet/module.rb', line 287 def has_hiera_conf? hiera_conf_file.nil? ? false : Puppet::FileSystem.exist?(hiera_conf_file) end |
#has_metadata? ⇒ Boolean
116 117 118 119 120 121 122 123 |
# File 'lib/puppet/module.rb', line 116 def begin @metadata.is_a?(Hash) && !@metadata.empty? rescue Puppet::Module::MissingMetadata false end end |
#has_translations?(locale) ⇒ Boolean
Returns true if the module has translation files for the given locale.
315 316 317 |
# File 'lib/puppet/module.rb', line 315 def has_translations?(locale) return Puppet::FileSystem.exist?(File.join(locale_directory, locale)) end |
#hiera_conf_file ⇒ Object
280 281 282 283 284 285 |
# File 'lib/puppet/module.rb', line 280 def hiera_conf_file unless defined?(@hiera_conf_file) @hiera_conf_file = path.nil? ? nil : File.join(path, Puppet::Pops::Lookup::HieraConfig::CONFIG_FILE_NAME) end @hiera_conf_file end |
#license_file ⇒ Object
196 197 198 199 200 201 |
# File 'lib/puppet/module.rb', line 196 def license_file return @license_file if defined?(@license_file) return @license_file = nil unless path @license_file = File.join(path, "License") end |
#load_metadata ⇒ Object
222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 |
# File 'lib/puppet/module.rb', line 222 def return if instance_variable_defined?(:@metadata) @metadata = data = return if data.empty? @forge_name = data['name'].gsub('-', '/') if data['name'] [:source, :author, :version, :license, :dependencies].each do |attr| value = data[attr.to_s] raise MissingMetadata, "No #{attr} module metadata provided for #{self.name}" if value.nil? if attr == :dependencies unless value.is_a?(Array) raise MissingMetadata, "The value for the key dependencies in the file metadata.json of the module #{self.name} must be an array, not: '#{value}'" end value.each do |dep| name = dep['name'] dep['name'] = name.tr('-', '/') unless name.nil? dep['version_requirement'] ||= '>= 0.0.0' end end send(attr.to_s + "=", value) end end |
#locale_directory ⇒ String
305 306 307 |
# File 'lib/puppet/module.rb', line 305 def locale_directory subpath("locales") end |
#match_manifests(rest) ⇒ Object
Return the list of manifests matching the given glob pattern, defaulting to ‘init.pp’ for empty modules.
251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 |
# File 'lib/puppet/module.rb', line 251 def match_manifests(rest) if rest wanted_manifests = wanted_manifests_from(rest) searched_manifests = wanted_manifests.glob.reject { |f| FileTest.directory?(f) } else searched_manifests = [] end # (#4220) Always ensure init.pp in case class is defined there. init_manifest = manifest("init.pp") if !init_manifest.nil? && !searched_manifests.include?(init_manifest) searched_manifests.unshift(init_manifest) end searched_manifests end |
#metadata_file ⇒ Object
273 274 275 276 277 278 |
# File 'lib/puppet/module.rb', line 273 def return @metadata_file if defined?(@metadata_file) return @metadata_file = nil unless path @metadata_file = File.join(path, "metadata.json") end |
#modulepath ⇒ Object
291 292 293 |
# File 'lib/puppet/module.rb', line 291 def modulepath File.dirname(path) if path end |
#plugin_directory ⇒ Object
Find all plugin directories. This is used by the Plugins fileserving mount.
296 297 298 |
# File 'lib/puppet/module.rb', line 296 def plugin_directory subpath("lib") end |
#plugin_fact_directory ⇒ Object
300 301 302 |
# File 'lib/puppet/module.rb', line 300 def plugin_fact_directory subpath("facts.d") end |
#puppetversion ⇒ Object
The puppetversion module metadata field is no longer used.
103 104 105 |
# File 'lib/puppet/module.rb', line 103 def puppetversion nil end |
#puppetversion=(something) ⇒ Object
The puppetversion module metadata field is no longer used.
108 109 |
# File 'lib/puppet/module.rb', line 108 def puppetversion=(something) end |
#read_metadata ⇒ Object
203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 |
# File 'lib/puppet/module.rb', line 203 def md_file = md_file.nil? ? {} : JSON.parse(File.read(md_file, :encoding => 'utf-8')) rescue Errno::ENOENT {} rescue JSON::JSONError => e #TRANSLATORS 'metadata.json' is a specific file name and should not be translated. msg = _("%{name} has an invalid and unparsable metadata.json file. The parse error: %{error}") % { name: name, error: e. } case Puppet[:strict] when :off Puppet.debug(msg) when :warning Puppet.warning(msg) when :error raise FaultyMetadata, msg end {} end |
#required_by ⇒ Object
345 346 347 |
# File 'lib/puppet/module.rb', line 345 def required_by environment.module_requirements[self.forge_name] || {} end |
#strict_semver? ⇒ Boolean
435 436 437 |
# File 'lib/puppet/module.rb', line 435 def strict_semver? @strict_semver end |
#supports(name, version = nil) ⇒ Object
323 324 325 326 |
# File 'lib/puppet/module.rb', line 323 def supports(name, version = nil) @supports ||= [] @supports << [name, version] end |
#task_file(name) ⇒ Object
This is a re-implementation of the Filetypes singular type method (e.g. ‘manifest(’my/manifest.pp’)‘. We don’t implement the full filetype “API” for tasks since tasks don’t map 1:1 onto files.
180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 |
# File 'lib/puppet/module.rb', line 180 def task_file(name) # If 'file' is nil then they're asking for the base path. # This is used for things like fileserving. if name full_path = File.join(tasks_directory, name) else full_path = tasks_directory end if Puppet::FileSystem.exist?(full_path) return full_path else return nil end end |
#tasks_directory ⇒ Object
163 164 165 |
# File 'lib/puppet/module.rb', line 163 def tasks_directory subpath("tasks") end |
#to_s ⇒ Object
328 329 330 331 332 |
# File 'lib/puppet/module.rb', line 328 def to_s result = "Module #{name}" result += "(#{path})" if path result end |
#unmet_dependencies ⇒ Object
Identify and mark unmet dependencies. A dependency will be marked unmet for the following reasons:
* not installed and is thus considered missing
* installed and does not meet the version requirements for this module
* installed and doesn't use semantic versioning
Returns a list of hashes representing the details of an unmet dependency.
Example:
[
{
:reason => :missing,
:name => 'puppetlabs-mysql',
:version_constraint => 'v0.0.1',
:mod_details => {
:installed_version => '0.0.1'
}
:parent => {
:name => 'puppetlabs-bacula',
:version => 'v1.0.0'
}
}
]
375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 |
# File 'lib/puppet/module.rb', line 375 def unmet_dependencies unmet_dependencies = [] return unmet_dependencies unless dependencies dependencies.each do |dependency| name = dependency['name'] version_string = dependency['version_requirement'] || '>= 0.0.0' dep_mod = begin environment.module_by_forge_name(name) rescue nil end error_details = { :name => name, :version_constraint => version_string.gsub(/^(?=\d)/, "v"), :parent => { :name => self.forge_name, :version => self.version.gsub(/^(?=\d)/, "v") }, :mod_details => { :installed_version => dep_mod.nil? ? nil : dep_mod.version } } unless dep_mod error_details[:reason] = :missing unmet_dependencies << error_details next end if version_string begin required_version_semver_range = self.class.parse_range(version_string, @strict_semver) actual_version_semver = SemanticPuppet::Version.parse(dep_mod.version) rescue ArgumentError error_details[:reason] = :non_semantic_version unmet_dependencies << error_details next end unless required_version_semver_range.include? actual_version_semver error_details[:reason] = :version_mismatch unmet_dependencies << error_details next end end end unmet_dependencies end |
#validate_puppet_version ⇒ Object
The puppetversion module metadata field is no longer used.
112 113 114 |
# File 'lib/puppet/module.rb', line 112 def validate_puppet_version return end |