Class: Puppet::Module

Inherits:
Object show all
Includes:
Util::Logging
Defined in:
lib/puppet/module.rb

Overview

Support for modules

Defined Under Namespace

Classes: Error, IncompatibleModule, IncompatiblePlatform, InvalidFilePattern, InvalidName, MissingMetadata, MissingModule, UnsupportedPlatform

Constant Summary collapse

FILETYPES =
{
  "manifests" => "manifests",
  "files" => "files",
  "templates" => "templates",
  "plugins" => "lib",
  "pluginfacts" => "facts.d",
}

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Util::Logging

#clear_deprecation_warnings, #debug, #deprecation_warning, #format_exception, #get_deprecation_offender, #log_and_raise, #log_deprecations_to_file, #log_exception, #puppet_deprecation_warning, #send_log, setup_facter_logging!

Constructor Details

#initialize(name, path, environment) ⇒ Module

Returns a new instance of Module.



61
62
63
64
65
66
67
68
69
70
71
72
73
# File 'lib/puppet/module.rb', line 61

def initialize(name, path, environment)
  @name = name
  @path = path
  @environment = environment

  assert_validity

   if has_metadata?

  validate_puppet_version

  @absolute_path_to_manifests = Puppet::FileSystem::PathPattern.absolute(manifests)
end

Instance Attribute Details

#authorObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def author
  @author
end

#dependenciesObject



58
59
60
# File 'lib/puppet/module.rb', line 58

def dependencies
  @dependencies
end

#descriptionObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def description
  @description
end

#environmentObject



55
56
57
# File 'lib/puppet/module.rb', line 55

def environment
  @environment
end

#forge_nameObject



58
59
60
# File 'lib/puppet/module.rb', line 58

def forge_name
  @forge_name
end

#licenseObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def license
  @license
end

#metadataObject (readonly)



55
56
57
# File 'lib/puppet/module.rb', line 55

def 
  
end

#nameObject (readonly)



55
56
57
# File 'lib/puppet/module.rb', line 55

def name
  @name
end

#pathObject (readonly)



55
56
57
# File 'lib/puppet/module.rb', line 55

def path
  @path
end

#project_pageObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def project_page
  @project_page
end

#puppetversionObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def puppetversion
  @puppetversion
end

#sourceObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def source
  @source
end

#summaryObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def summary
  @summary
end

#versionObject



59
60
61
# File 'lib/puppet/module.rb', line 59

def version
  @version
end

Class Method Details

.find(modname, environment = nil) ⇒ Object

Find and return the module that path belongs to. If path is absolute, or if there is no module whose name is the first component of path, return nil



29
30
31
32
33
34
# File 'lib/puppet/module.rb', line 29

def self.find(modname, environment = nil)
  return nil unless modname
  # Unless a specific environment is given, use the current environment
  env = environment ? Puppet.lookup(:environments).get!(environment) : Puppet.lookup(:current_environment)
  env.module(modname)
end

.is_module_directory?(name, path) ⇒ Boolean

Returns:

  • (Boolean)


36
37
38
39
40
41
# File 'lib/puppet/module.rb', line 36

def self.is_module_directory?(name, path)
  # it must be a directory
  fullpath = File.join(path, name)
  return false unless Puppet::FileSystem.directory?(fullpath)
  return is_module_directory_name?(name)
end

.is_module_directory_name?(name) ⇒ Boolean

Returns:

  • (Boolean)


43
44
45
46
47
# File 'lib/puppet/module.rb', line 43

def self.is_module_directory_name?(name)
  # it must match an installed module name according to forge validator
  return true if name =~ /^[a-z][a-z0-9_]*$/
  return false
end

.is_module_namespaced_name?(name) ⇒ Boolean

Returns:

  • (Boolean)


49
50
51
52
53
# File 'lib/puppet/module.rb', line 49

def self.is_module_namespaced_name?(name)
  # it must match the full module name according to forge validator
  return true if name =~ /^[a-zA-Z0-9]+[-][a-z][a-z0-9_]*$/   
  return false
end

Instance Method Details

#==(other) ⇒ Object



315
316
317
318
319
320
# File 'lib/puppet/module.rb', line 315

def ==(other)
  self.name == other.name &&
  self.version == other.version &&
  self.path == other.path &&
  self.environment == other.environment
end

#all_manifestsObject



174
175
176
177
178
# File 'lib/puppet/module.rb', line 174

def all_manifests
  return [] unless Puppet::FileSystem.exist?(manifests)

  Dir.glob(File.join(manifests, '**', '*.pp'))
end

#dependencies_as_modulesObject



215
216
217
218
219
220
221
222
223
224
# File 'lib/puppet/module.rb', line 215

def dependencies_as_modules
  dependent_modules = []
  dependencies and dependencies.each do |dep|
    author, dep_name = dep["name"].split('/')
    found_module = environment.module(dep_name)
    dependent_modules << found_module if found_module
  end

  dependent_modules
end

#has_external_facts?Boolean

Returns:

  • (Boolean)


200
201
202
# File 'lib/puppet/module.rb', line 200

def has_external_facts?
  File.directory?(plugin_fact_directory)
end

#has_metadata?Boolean

Returns:

  • (Boolean)


75
76
77
78
79
80
81
82
83
84
85
86
87
88
# File 'lib/puppet/module.rb', line 75

def has_metadata?
  return false unless 

  return false unless Puppet::FileSystem.exist?()

  begin
     =  JSON.parse(File.read())
  rescue JSON::JSONError => e
    Puppet.debug("#{name} has an invalid and unparsable metadata.json file.  The parse error: #{e.message}")
    return false
  end

  return .is_a?(Hash) && !.keys.empty?
end

#license_fileObject



128
129
130
131
132
133
# File 'lib/puppet/module.rb', line 128

def license_file
  return @license_file if defined?(@license_file)

  return @license_file = nil unless path
  @license_file = File.join(path, "License")
end

#load_metadataObject



135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
# File 'lib/puppet/module.rb', line 135

def 
   = data = JSON.parse(File.read())
  @forge_name = data['name'].gsub('-', '/') if data['name']

  [:source, :author, :version, :license, :puppetversion, :dependencies].each do |attr|
    unless value = data[attr.to_s]
      unless attr == :puppetversion
        raise , "No #{attr} module metadata provided for #{self.name}"
      end
    end

    if attr == :dependencies
      value.each do |dep|
        dep['version_requirement'] ||= '>= 0.0.0'
      end
    end

    send(attr.to_s + "=", value)
  end
end

#match_manifests(rest) ⇒ Object

Return the list of manifests matching the given glob pattern, defaulting to ‘init.pp’ for empty modules.



158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
# File 'lib/puppet/module.rb', line 158

def match_manifests(rest)
  if rest
    wanted_manifests = wanted_manifests_from(rest)
    searched_manifests = wanted_manifests.glob.reject { |f| FileTest.directory?(f) }
  else
    searched_manifests = []
  end

  # (#4220) Always ensure init.pp in case class is defined there.
  init_manifest = manifest("init.pp")
  if !init_manifest.nil? && !searched_manifests.include?(init_manifest)
    searched_manifests.unshift(init_manifest)
  end
  searched_manifests
end

#metadata_fileObject



180
181
182
183
184
185
# File 'lib/puppet/module.rb', line 180

def 
  return  if defined?()

  return  = nil unless path
   = File.join(path, "metadata.json")
end

#modulepathObject



187
188
189
# File 'lib/puppet/module.rb', line 187

def modulepath
  File.dirname(path) if path
end

#plugin_directoryObject

Find all plugin directories. This is used by the Plugins fileserving mount.



192
193
194
# File 'lib/puppet/module.rb', line 192

def plugin_directory
  subpath("lib")
end

#plugin_fact_directoryObject



196
197
198
# File 'lib/puppet/module.rb', line 196

def plugin_fact_directory
  subpath("facts.d")
end

#required_byObject



226
227
228
# File 'lib/puppet/module.rb', line 226

def required_by
  environment.module_requirements[self.forge_name] || {}
end

#supports(name, version = nil) ⇒ Object



204
205
206
207
# File 'lib/puppet/module.rb', line 204

def supports(name, version = nil)
  @supports ||= []
  @supports << [name, version]
end

#to_sObject



209
210
211
212
213
# File 'lib/puppet/module.rb', line 209

def to_s
  result = "Module #{name}"
  result += "(#{path})" if path
  result
end

#unmet_dependenciesObject

Identify and mark unmet dependencies. A dependency will be marked unmet for the following reasons:

* not installed and is thus considered missing
* installed and does not meet the version requirements for this module
* installed and doesn't use semantic versioning

Returns a list of hashes representing the details of an unmet dependency.

Example:

[
  {
    :reason => :missing,
    :name   => 'puppetlabs-mysql',
    :version_constraint => 'v0.0.1',
    :mod_details => {
      :installed_version => '0.0.1'
    }
    :parent => {
      :name    => 'puppetlabs-bacula',
      :version => 'v1.0.0'
    }
  }
]


256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
# File 'lib/puppet/module.rb', line 256

def unmet_dependencies
  unmet_dependencies = []
  return unmet_dependencies unless dependencies

  dependencies.each do |dependency|
    name = dependency['name']
    forge_name = name.tr('-', '/')
    version_string = dependency['version_requirement'] || '>= 0.0.0'

    dep_mod = begin
      environment.module_by_forge_name(forge_name)
    rescue
      nil
    end

    error_details = {
      :name => name,
      :version_constraint => version_string.gsub(/^(?=\d)/, "v"),
      :parent => {
        :name => self.forge_name,
        :version => self.version.gsub(/^(?=\d)/, "v")
      },
      :mod_details => {
        :installed_version => dep_mod.nil? ? nil : dep_mod.version
      }
    }

    unless dep_mod
      error_details[:reason] = :missing
      unmet_dependencies << error_details
      next
    end

    if version_string
      begin
        required_version_semver_range = SemVer[version_string]
        actual_version_semver = SemVer.new(dep_mod.version)
      rescue ArgumentError
        error_details[:reason] = :non_semantic_version
        unmet_dependencies << error_details
        next
      end

      unless required_version_semver_range.include? actual_version_semver
        error_details[:reason] = :version_mismatch
        unmet_dependencies << error_details
        next
      end
    end
  end

  unmet_dependencies
end

#validate_puppet_versionObject

Raises:



310
311
312
313
# File 'lib/puppet/module.rb', line 310

def validate_puppet_version
  return unless puppetversion and puppetversion != Puppet.version
  raise IncompatibleModule, "Module #{self.name} is only compatible with Puppet version #{puppetversion}, not #{Puppet.version}"
end