Class: Puppet::Module
- Includes:
- Util::Logging
- Defined in:
- lib/puppet/module.rb
Overview
Support for modules
Defined Under Namespace
Classes: Error, IncompatibleModule, IncompatiblePlatform, InvalidFilePattern, InvalidName, MissingMetadata, MissingModule, UnsupportedPlatform
Constant Summary collapse
- TEMPLATES =
"templates"- FILES =
"files"- MANIFESTS =
"manifests"- PLUGINS =
"plugins"- FILETYPES =
[MANIFESTS, FILES, TEMPLATES, PLUGINS]
Instance Attribute Summary collapse
-
#author ⇒ Object
Returns the value of attribute author.
-
#dependencies ⇒ Object
Returns the value of attribute dependencies.
-
#description ⇒ Object
Returns the value of attribute description.
-
#environment ⇒ Object
Returns the value of attribute environment.
-
#forge_name ⇒ Object
Returns the value of attribute forge_name.
-
#license ⇒ Object
Returns the value of attribute license.
-
#name ⇒ Object
readonly
Returns the value of attribute name.
-
#project_page ⇒ Object
Returns the value of attribute project_page.
-
#puppetversion ⇒ Object
Returns the value of attribute puppetversion.
-
#source ⇒ Object
Returns the value of attribute source.
-
#summary ⇒ Object
Returns the value of attribute summary.
-
#version ⇒ Object
Returns the value of attribute version.
Class Method Summary collapse
-
.find(modname, environment = nil) ⇒ Object
Find and return the
modulethatpathbelongs to.
Instance Method Summary collapse
- #dependencies_as_modules ⇒ Object
- #exist? ⇒ Boolean
-
#file_directory ⇒ Object
Find the first ‘files’ directory.
- #has_local_changes? ⇒ Boolean
- #has_metadata? ⇒ Boolean
-
#initialize(name, options = {}) ⇒ Module
constructor
A new instance of Module.
- #license_file ⇒ Object
- #load_metadata ⇒ Object
- #local_changes ⇒ Object
-
#match_manifests(rest) ⇒ Object
Return the list of manifests matching the given glob pattern, defaulting to ‘init.pp,rb’ for empty modules.
- #metadata_file ⇒ Object
- #modulepath ⇒ Object
-
#path ⇒ Object
Find this module in the modulepath.
-
#plugin_directory ⇒ Object
Find all plugin directories.
- #required_by ⇒ Object
- #supports(name, version = nil) ⇒ Object
- #to_s ⇒ Object
-
#unmet_dependencies ⇒ Object
Identify and mark unmet dependencies.
- #validate_puppet_version ⇒ Object
Methods included from Util::Logging
#clear_deprecation_warnings, #deprecation_warning, #send_log
Constructor Details
#initialize(name, options = {}) ⇒ Module
Returns a new instance of Module.
50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 |
# File 'lib/puppet/module.rb', line 50 def initialize(name, = {}) @name = name @path = [:path] assert_validity if [:environment].is_a?(Puppet::Node::Environment) @environment = [:environment] else @environment = Puppet::Node::Environment.new([:environment]) end if validate_puppet_version end |
Instance Attribute Details
#author ⇒ Object
Returns the value of attribute author.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def end |
#dependencies ⇒ Object
Returns the value of attribute dependencies.
36 37 38 |
# File 'lib/puppet/module.rb', line 36 def dependencies @dependencies end |
#description ⇒ Object
Returns the value of attribute description.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def description @description end |
#environment ⇒ Object
Returns the value of attribute environment.
33 34 35 |
# File 'lib/puppet/module.rb', line 33 def environment @environment end |
#forge_name ⇒ Object
Returns the value of attribute forge_name.
36 37 38 |
# File 'lib/puppet/module.rb', line 36 def forge_name @forge_name end |
#license ⇒ Object
Returns the value of attribute license.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def license @license end |
#name ⇒ Object (readonly)
Returns the value of attribute name.
33 34 35 |
# File 'lib/puppet/module.rb', line 33 def name @name end |
#project_page ⇒ Object
Returns the value of attribute project_page.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def project_page @project_page end |
#puppetversion ⇒ Object
Returns the value of attribute puppetversion.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def puppetversion @puppetversion end |
#source ⇒ Object
Returns the value of attribute source.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def source @source end |
#summary ⇒ Object
Returns the value of attribute summary.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def summary @summary end |
#version ⇒ Object
Returns the value of attribute version.
37 38 39 |
# File 'lib/puppet/module.rb', line 37 def version @version end |
Class Method Details
.find(modname, environment = nil) ⇒ Object
Find and return the module that path belongs to. If path is absolute, or if there is no module whose name is the first component of path, return nil
28 29 30 31 |
# File 'lib/puppet/module.rb', line 28 def self.find(modname, environment = nil) return nil unless modname Puppet::Node::Environment.new(environment).module(modname) end |
Instance Method Details
#dependencies_as_modules ⇒ Object
186 187 188 189 190 191 192 193 194 195 |
# File 'lib/puppet/module.rb', line 186 def dependencies_as_modules dependent_modules = [] dependencies and dependencies.each do |dep| , dep_name = dep["name"].split('/') found_module = environment.module(dep_name) dependent_modules << found_module if found_module end dependent_modules end |
#exist? ⇒ Boolean
97 98 99 |
# File 'lib/puppet/module.rb', line 97 def exist? ! path.nil? end |
#file_directory ⇒ Object
Find the first ‘files’ directory. This is used by the XMLRPC fileserver.
102 103 104 |
# File 'lib/puppet/module.rb', line 102 def file_directory subpath("files") end |
#has_local_changes? ⇒ Boolean
201 202 203 204 |
# File 'lib/puppet/module.rb', line 201 def has_local_changes? changes = Puppet::ModuleTool::Applications::Checksummer.run(path) !changes.empty? end |
#has_metadata? ⇒ Boolean
39 40 41 42 43 44 45 46 47 48 |
# File 'lib/puppet/module.rb', line 39 def return false unless return false unless FileTest.exist?() = PSON.parse File.read() return .is_a?(Hash) && !.keys.empty? end |
#license_file ⇒ Object
106 107 108 109 110 111 |
# File 'lib/puppet/module.rb', line 106 def license_file return @license_file if defined?(@license_file) return @license_file = nil unless path @license_file = File.join(path, "License") end |
#load_metadata ⇒ Object
113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 |
# File 'lib/puppet/module.rb', line 113 def data = PSON.parse File.read() @forge_name = data['name'].gsub('-', '/') if data['name'] [:source, :author, :version, :license, :puppetversion, :dependencies].each do |attr| unless value = data[attr.to_s] unless attr == :puppetversion raise MissingMetadata, "No #{attr} module metadata provided for #{self.name}" end end # NOTICE: The fallback to `versionRequirement` is something we'd like to # not have to support, but we have a reasonable number of releases that # don't use `version_requirement`. When we can deprecate this, we should. if attr == :dependencies value.tap do |dependencies| dependencies.each do |dep| dep['version_requirement'] ||= dep['versionRequirement'] || '>= 0.0.0' end end end send(attr.to_s + "=", value) end end |
#local_changes ⇒ Object
206 207 208 |
# File 'lib/puppet/module.rb', line 206 def local_changes Puppet::ModuleTool::Applications::Checksummer.run(path) end |
#match_manifests(rest) ⇒ Object
Return the list of manifests matching the given glob pattern, defaulting to ‘init.pp,rb’ for empty modules.
141 142 143 144 145 146 147 148 149 150 151 152 |
# File 'lib/puppet/module.rb', line 141 def match_manifests(rest) if rest wanted_manifests = wanted_manifests_from(rest) searched_manifests = wanted_manifests.glob.reject { |f| FileTest.directory?(f) } else searched_manifests = [] end # (#4220) Always ensure init.pp in case class is defined there. init_manifests = [manifest("init.pp"), manifest("init.rb")].compact init_manifests + searched_manifests end |
#metadata_file ⇒ Object
154 155 156 157 158 159 |
# File 'lib/puppet/module.rb', line 154 def return if defined?() return = nil unless path = File.join(path, "metadata.json") end |
#modulepath ⇒ Object
166 167 168 |
# File 'lib/puppet/module.rb', line 166 def modulepath File.dirname(path) if path end |
#path ⇒ Object
Find this module in the modulepath.
162 163 164 |
# File 'lib/puppet/module.rb', line 162 def path @path ||= environment.modulepath.collect { |path| File.join(path, name) }.find { |d| FileTest.directory?(d) } end |
#plugin_directory ⇒ Object
Find all plugin directories. This is used by the Plugins fileserving mount.
171 172 173 |
# File 'lib/puppet/module.rb', line 171 def plugin_directory subpath("plugins") end |
#required_by ⇒ Object
197 198 199 |
# File 'lib/puppet/module.rb', line 197 def required_by environment.module_requirements[self.forge_name] || {} end |
#supports(name, version = nil) ⇒ Object
175 176 177 178 |
# File 'lib/puppet/module.rb', line 175 def supports(name, version = nil) @supports ||= [] @supports << [name, version] end |
#to_s ⇒ Object
180 181 182 183 184 |
# File 'lib/puppet/module.rb', line 180 def to_s result = "Module #{name}" result += "(#{path})" if path result end |
#unmet_dependencies ⇒ Object
Identify and mark unmet dependencies. A dependency will be marked unmet for the following reasons:
* not installed and is thus considered missing
* installed and does not meet the version requirements for this module
* installed and doesn't use semantic versioning
Returns a list of hashes representing the details of an unmet dependency.
Example:
[
{
:reason => :missing,
:name => 'puppetlabs-mysql',
:version_constraint => 'v0.0.1',
:mod_details => {
:installed_version => '0.0.1'
}
:parent => {
:name => 'puppetlabs-bacula',
:version => 'v1.0.0'
}
}
]
236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 |
# File 'lib/puppet/module.rb', line 236 def unmet_dependencies unmet_dependencies = [] return unmet_dependencies unless dependencies dependencies.each do |dependency| forge_name = dependency['name'] version_string = dependency['version_requirement'] || '>= 0.0.0' dep_mod = begin environment.module_by_forge_name(forge_name) rescue => e nil end error_details = { :name => forge_name, :version_constraint => version_string.gsub(/^(?=\d)/, "v"), :parent => { :name => self.forge_name, :version => self.version.gsub(/^(?=\d)/, "v") }, :mod_details => { :installed_version => dep_mod.nil? ? nil : dep_mod.version } } unless dep_mod error_details[:reason] = :missing unmet_dependencies << error_details next end if version_string begin required_version_semver_range = SemVer[version_string] actual_version_semver = SemVer.new(dep_mod.version) rescue ArgumentError error_details[:reason] = :non_semantic_version unmet_dependencies << error_details next end unless required_version_semver_range.include? actual_version_semver error_details[:reason] = :version_mismatch unmet_dependencies << error_details next end end end unmet_dependencies end |
#validate_puppet_version ⇒ Object
289 290 291 292 |
# File 'lib/puppet/module.rb', line 289 def validate_puppet_version return unless puppetversion and puppetversion != Puppet.version raise IncompatibleModule, "Module #{self.name} is only compatible with Puppet version #{puppetversion}, not #{Puppet.version}" end |