Method: Puppet::Pal::ScriptCompiler#list_plans
- Defined in:
- lib/puppet/pal/script_compiler.rb
#list_plans(filter_regex = nil, error_collector = nil) ⇒ Array<Puppet::Pops::Loader::TypedName>
Returns an array of TypedName objects for all plans, optionally filtered by a regular expression. The returned array has more information than just the leaf name - the typical thing is to just get the name as showing the following example.
Errors that occur during plan discovery will either be logged as warnings or collected by the optional error_collector array. When provided, it will receive DataTypes::Error instances describing each error in detail and no warnings will be logged.
36 37 38 |
# File 'lib/puppet/pal/script_compiler.rb', line 36 def list_plans(filter_regex = nil, error_collector = nil) list_loadable_kind(:plan, filter_regex, error_collector) end |