Method: Puppet::Plugins::SyntaxCheckers::SyntaxChecker#check
- Defined in:
- lib/puppet/plugins/syntax_checkers.rb
#check(text, syntax_identifier, acceptor, location_info) ⇒ Boolean
Checks the text for syntax issues and reports them to the given acceptor. This implementation is abstract, it raises NotImplementedError since a subclass should have implemented the method.
98 99 100 |
# File 'lib/puppet/plugins/syntax_checkers.rb', line 98 def check(text, syntax_identifier, acceptor, location_info) raise NotImplementedError, "The class #{self.class.name} should have implemented the method check()" end |