Class: Puppet::Pops::Adapters::OriginAdapter
- Inherits:
-
Puppet::Pops::Adaptable::Adapter
- Object
- Puppet::Pops::Adaptable::Adapter
- Puppet::Pops::Adapters::OriginAdapter
- Defined in:
- lib/puppet/pops/adapters.rb
Overview
An origin adapter adapts an object with where it came from. This origin describes the resource (a file, etc.) where source text originates. Instances of SourcePosAdapter is then used on other objects in a model to describe their relative position versus the origin.
Instance Attribute Summary collapse
-
#origin ⇒ String
The origin of the adapted (usually a filename).
Method Summary
Methods inherited from Puppet::Pops::Adaptable::Adapter
adapt, adapt_new, associate_adapter, clear, create_adapter, get, instance_var_name
Instance Attribute Details
#origin ⇒ String
Returns the origin of the adapted (usually a filename).
22 23 24 |
# File 'lib/puppet/pops/adapters.rb', line 22 def origin @origin end |