Class: Puppet::Pops::Types::PPatternType
- Inherits:
-
PScalarType
- Object
- RGen::MetamodelBuilder::MMBase
- Model::PopsObject
- PAbstractType
- PObjectType
- PScalarType
- Puppet::Pops::Types::PPatternType
- Defined in:
- lib/puppet/pops/types/types.rb
Overview
Represents a subtype of String that narrows the string to those matching the patterns If specified without a pattern it is basically the same as the String type.
Defined Under Namespace
Modules: ClassModule
Method Summary
Methods included from Containment
#eAllContainers, #eAllContents