Method: Puppet::Provider.execute
- Defined in:
- lib/puppet/provider.rb
.execute(*args) ⇒ Puppet::Util::Execution::ProcessOutput
Note:
Unfortunately, the default behavior for failonfail and combine (since 0.22.4 and 0.24.7, respectively) depend on whether options are specified or not. If specified, then failonfail and combine default to false (even when the options specified are neither failonfail nor combine). If no options are specified, then failonfail and combine default to true.
Executes the desired command, and return the status and output. def execute(command, options)
105 106 107 |
# File 'lib/puppet/provider.rb', line 105 def self.execute(*args) Puppet::Util::Execution.execute(*args) end |