Class: Puppet::Resource::Catalog
- Inherits:
-
Graph::SimpleGraph
- Object
- Graph::SimpleGraph
- Puppet::Resource::Catalog
- Extended by:
- Indirector, Util::Pson
- Includes:
- Util::Tagging
- Defined in:
- lib/puppet/resource/catalog.rb
Overview
This class models a node catalog. It is the thing meant to be passed from server to client, and it contains all of the information in the catalog, including the resources and the relationships between them.
Defined Under Namespace
Classes: ActiveRecord, Compiler, DuplicateResourceError, Json, Queue, Rest, StaticCompiler, StoreConfigs, Yaml
Constant Summary
Constants included from Indirector
Instance Attribute Summary collapse
-
#client_version ⇒ Object
Some metadata to help us compile and generally respond to the current state.
-
#environment ⇒ Object
The environment for this catalog.
-
#from_cache ⇒ Object
Whether this catalog was retrieved from the cache, which affects whether it is written back out again.
-
#host_config ⇒ Object
Whether this is a host catalog, which behaves very differently.
-
#name ⇒ Object
The host name this is a catalog for.
-
#retrieval_duration ⇒ Object
How long this catalog took to retrieve.
-
#server_version ⇒ Object
Some metadata to help us compile and generally respond to the current state.
-
#version ⇒ Object
The catalog version.
Class Method Summary collapse
Instance Method Summary collapse
-
#add_class(*classes) ⇒ Object
Add classes to our class list.
- #add_resource(*resources) ⇒ Object
-
#alias(resource, key) ⇒ Object
Create an alias for a resource.
-
#apply(options = {}) {|transaction| ... } ⇒ Puppet::Transaction
Apply our catalog to the local host.
- #classes ⇒ Object
- #clear(remove_resources = true) ⇒ Object
-
#container_of(resource) ⇒ A Resource?
The resource that contains the given resource.
-
#create_resource(type, options) ⇒ Object
Create a new resource and register it in the catalog.
-
#filter(&block) ⇒ Object
filter out the catalog, applying
blockto each resource. -
#finalize ⇒ Object
Make sure all of our resources are “finished”.
- #host_config? ⇒ Boolean
-
#initialize(name = nil) ⇒ Catalog
constructor
A new instance of Catalog.
-
#make_default_resources ⇒ Object
Make the default objects necessary for function.
-
#relationship_graph ⇒ Puppet::Graph::RelationshipGraph
The relationship_graph form of the catalog.
-
#remove_resource(*resources) ⇒ Object
Remove the resource from our catalog.
-
#resource(type, title = nil) ⇒ Object
Look a resource up by its reference (e.g., File).
- #resource_keys ⇒ Object
- #resource_refs ⇒ Object
- #resources ⇒ Object
- #title_key_for_ref(ref) ⇒ Object
- #to_data_hash ⇒ Object
- #to_pson(*args) ⇒ Object
- #to_pson_data_hash ⇒ Object
-
#to_ral ⇒ Object
Convert our catalog into a RAL catalog.
-
#to_resource ⇒ Object
Convert our catalog into a catalog of Puppet::Resource instances.
-
#write_class_file ⇒ Object
Store the classes in the classfile.
-
#write_graph(name) ⇒ Object
Produce the graph files if requested.
-
#write_resource_file ⇒ Object
Store the list of resources we manage.
Methods included from Indirector
Methods included from Util::Pson
Methods included from Util::Tagging
Methods inherited from Graph::SimpleGraph
#add_edge, #add_relationship, #add_vertex, #adjacent, #dependencies, #dependents, #direct_dependencies_of, #direct_dependents_of, #directed?, #dotty, #downstream_from_vertex, #each_edge, #edge?, #edges, #edges_between, #find_cycles_in_graph, #instance_variable_get, #leaves, #matching_edges, #path_between, #paths_in_cycle, #remove_edge!, #remove_vertex!, #report_cycles_in_graph, #reversal, #size, #tarjan, #to_a, #to_dot, #to_dot_graph, #to_yaml_properties, #tree_from_vertex, #upstream_from_vertex, #vertex?, #vertices, #walk, #write_cycles_to_graph, #yaml_initialize
Constructor Details
#initialize(name = nil) ⇒ Catalog
Returns a new instance of Catalog.
235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 |
# File 'lib/puppet/resource/catalog.rb', line 235 def initialize(name = nil) super() @name = name if name @classes = [] @resource_table = {} @resources = [] @relationship_graph = nil @host_config = true @aliases = {} if block_given? yield(self) finalize end end |
Instance Attribute Details
#client_version ⇒ Object
Some metadata to help us compile and generally respond to the current state.
47 48 49 |
# File 'lib/puppet/resource/catalog.rb', line 47 def client_version @client_version end |
#environment ⇒ Object
The environment for this catalog
50 51 52 |
# File 'lib/puppet/resource/catalog.rb', line 50 def environment @environment end |
#from_cache ⇒ Object
Whether this catalog was retrieved from the cache, which affects whether it is written back out again.
44 45 46 |
# File 'lib/puppet/resource/catalog.rb', line 44 def from_cache @from_cache end |
#host_config ⇒ Object
Whether this is a host catalog, which behaves very differently. In particular, reports are sent, graphs are made, and state is stored in the state database. If this is set incorrectly, then you often end up in infinite loops, because catalogs are used to make things that the host catalog needs.
40 41 42 |
# File 'lib/puppet/resource/catalog.rb', line 40 def host_config @host_config end |
#name ⇒ Object
The host name this is a catalog for.
26 27 28 |
# File 'lib/puppet/resource/catalog.rb', line 26 def name @name end |
#retrieval_duration ⇒ Object
How long this catalog took to retrieve. Used for reporting stats.
33 34 35 |
# File 'lib/puppet/resource/catalog.rb', line 33 def retrieval_duration @retrieval_duration end |
#server_version ⇒ Object
Some metadata to help us compile and generally respond to the current state.
47 48 49 |
# File 'lib/puppet/resource/catalog.rb', line 47 def server_version @server_version end |
#version ⇒ Object
The catalog version. Used for testing whether a catalog is up to date.
30 31 32 |
# File 'lib/puppet/resource/catalog.rb', line 30 def version @version end |
Class Method Details
.edge_from_pson(result, edge) ⇒ Object
351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 |
# File 'lib/puppet/resource/catalog.rb', line 351 def self.edge_from_pson(result, edge) # If no type information was presented, we manually find # the class. edge = Puppet::Relationship.from_pson(edge) if edge.is_a?(Hash) unless source = result.resource(edge.source) raise ArgumentError, "Could not convert from pson: Could not find relationship source #{edge.source.inspect}" end edge.source = source unless target = result.resource(edge.target) raise ArgumentError, "Could not convert from pson: Could not find relationship target #{edge.target.inspect}" end edge.target = target result.add_edge(edge) end |
.from_pson(data) ⇒ Object
316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 |
# File 'lib/puppet/resource/catalog.rb', line 316 def self.from_pson(data) result = new(data['name']) if = data['tags'] result.tag(*) end if version = data['version'] result.version = version end if environment = data['environment'] result.environment = environment end if resources = data['resources'] result.add_resource(*resources.collect do |res| Puppet::Resource.from_pson(res) end) end if edges = data['edges'] edges = PSON.parse(edges) if edges.is_a?(String) edges.each do |edge| edge_from_pson(result, edge) end end if classes = data['classes'] result.add_class(*classes) end result end |
Instance Method Details
#add_class(*classes) ⇒ Object
Add classes to our class list.
53 54 55 56 57 58 59 60 |
# File 'lib/puppet/resource/catalog.rb', line 53 def add_class(*classes) classes.each do |klass| @classes << klass end # Add the class names as tags, too. tag(*classes) end |
#add_resource(*resources) ⇒ Object
67 68 69 70 71 |
# File 'lib/puppet/resource/catalog.rb', line 67 def add_resource(*resources) resources.each do |resource| add_one_resource(resource) end end |
#alias(resource, key) ⇒ Object
Create an alias for a resource.
112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 |
# File 'lib/puppet/resource/catalog.rb', line 112 def alias(resource, key) resource.ref =~ /^(.+)\[/ class_name = $1 || resource.class.name newref = [class_name, key].flatten if key.is_a? String ref_string = "#{class_name}[#{key}]" return if ref_string == resource.ref end # LAK:NOTE It's important that we directly compare the references, # because sometimes an alias is created before the resource is # added to the catalog, so comparing inside the below if block # isn't sufficient. if existing = @resource_table[newref] return if existing == resource resource_declaration = " at #{resource.file}:#{resource.line}" if resource.file and resource.line existing_declaration = " at #{existing.file}:#{existing.line}" if existing.file and existing.line msg = "Cannot alias #{resource.ref} to #{key.inspect}#{resource_declaration}; resource #{newref.inspect} already declared#{existing_declaration}" raise ArgumentError, msg end @resource_table[newref] = resource @aliases[resource.ref] ||= [] @aliases[resource.ref] << newref end |
#apply(options = {}) {|transaction| ... } ⇒ Puppet::Transaction
Apply our catalog to the local host.
157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 |
# File 'lib/puppet/resource/catalog.rb', line 157 def apply( = {}) Puppet::Util::Storage.load if host_config? transaction = create_transaction() begin transaction.report.as_logging_destination do transaction.evaluate end rescue Puppet::Error => detail Puppet.log_exception(detail, "Could not apply complete catalog: #{detail}") rescue => detail Puppet.log_exception(detail, "Got an uncaught exception of type #{detail.class}: #{detail}") ensure # Don't try to store state unless we're a host config # too recursive. Puppet::Util::Storage.store if host_config? end yield transaction if block_given? transaction end |
#classes ⇒ Object
207 208 209 |
# File 'lib/puppet/resource/catalog.rb', line 207 def classes @classes.dup end |
#clear(remove_resources = true) ⇒ Object
194 195 196 197 198 199 200 201 202 203 204 205 |
# File 'lib/puppet/resource/catalog.rb', line 194 def clear(remove_resources = true) super() # We have to do this so that the resources clean themselves up. @resource_table.values.each { |resource| resource.remove } if remove_resources @resource_table.clear @resources = [] if @relationship_graph @relationship_graph.clear @relationship_graph = nil end end |
#container_of(resource) ⇒ A Resource?
Returns the resource that contains the given resource.
76 77 78 |
# File 'lib/puppet/resource/catalog.rb', line 76 def container_of(resource) adjacent(resource, :direction => :in)[0] end |
#create_resource(type, options) ⇒ Object
Create a new resource and register it in the catalog.
212 213 214 215 216 217 218 219 220 |
# File 'lib/puppet/resource/catalog.rb', line 212 def create_resource(type, ) unless klass = Puppet::Type.type(type) raise ArgumentError, "Unknown resource type #{type}" end return unless resource = klass.new() add_resource(resource) resource end |
#filter(&block) ⇒ Object
filter out the catalog, applying block to each resource. If the block result is false, the resource will be kept otherwise it will be skipped
408 409 410 |
# File 'lib/puppet/resource/catalog.rb', line 408 def filter(&block) to_catalog :to_resource, &block end |
#finalize ⇒ Object
Make sure all of our resources are “finished”.
223 224 225 226 227 228 229 |
# File 'lib/puppet/resource/catalog.rb', line 223 def finalize make_default_resources @resource_table.values.each { |resource| resource.finish } write_graph(:resources) end |
#host_config? ⇒ Boolean
231 232 233 |
# File 'lib/puppet/resource/catalog.rb', line 231 def host_config? host_config end |
#make_default_resources ⇒ Object
Make the default objects necessary for function.
254 255 256 257 258 259 260 261 262 263 264 265 |
# File 'lib/puppet/resource/catalog.rb', line 254 def make_default_resources # We have to add the resources to the catalog, or else they won't get cleaned up after # the transaction. # First create the default scheduling objects Puppet::Type.type(:schedule).mkdefaultschedules.each { |res| add_resource(res) unless resource(res.ref) } # And filebuckets if bucket = Puppet::Type.type(:filebucket).mkdefaultbucket add_resource(bucket) unless resource(bucket.ref) end end |
#relationship_graph ⇒ Puppet::Graph::RelationshipGraph
The relationship_graph form of the catalog. This contains all of the dependency edges that are used for determining order.
186 187 188 189 190 191 192 |
# File 'lib/puppet/resource/catalog.rb', line 186 def relationship_graph if @relationship_graph.nil? @relationship_graph = Puppet::Graph::RelationshipGraph.new(prioritizer) @relationship_graph.populate_from(self) end @relationship_graph end |
#remove_resource(*resources) ⇒ Object
Remove the resource from our catalog. Notice that we also call ‘remove’ on the resource, at least until resource classes no longer maintain references to the resource instances.
270 271 272 273 274 275 276 277 278 279 280 281 282 283 |
# File 'lib/puppet/resource/catalog.rb', line 270 def remove_resource(*resources) resources.each do |resource| title_key = title_key_for_ref(resource.ref) @resource_table.delete(title_key) if aliases = @aliases[resource.ref] aliases.each { |res_alias| @resource_table.delete(res_alias) } @aliases.delete(resource.ref) end remove_vertex!(resource) if vertex?(resource) @relationship_graph.remove_vertex!(resource) if @relationship_graph and @relationship_graph.vertex?(resource) @resources.delete(title_key) resource.remove end end |
#resource(type, title = nil) ⇒ Object
Look a resource up by its reference (e.g., File).
286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 |
# File 'lib/puppet/resource/catalog.rb', line 286 def resource(type, title = nil) # Always create a resource reference, so that it always # canonicalizes how we are referring to them. if title res = Puppet::Resource.new(type, title) else # If they didn't provide a title, then we expect the first # argument to be of the form 'Class[name]', which our # Reference class canonicalizes for us. res = Puppet::Resource.new(nil, type) end title_key = [res.type, res.title.to_s] uniqueness_key = [res.type, res.uniqueness_key].flatten @resource_table[title_key] || @resource_table[uniqueness_key] end |
#resource_keys ⇒ Object
306 307 308 |
# File 'lib/puppet/resource/catalog.rb', line 306 def resource_keys @resource_table.keys end |
#resource_refs ⇒ Object
302 303 304 |
# File 'lib/puppet/resource/catalog.rb', line 302 def resource_refs resource_keys.collect{ |type, name| name.is_a?( String ) ? "#{type}[#{name}]" : nil}.compact end |
#resources ⇒ Object
310 311 312 313 314 |
# File 'lib/puppet/resource/catalog.rb', line 310 def resources @resources.collect do |key| @resource_table[key] end end |
#title_key_for_ref(ref) ⇒ Object
62 63 64 65 |
# File 'lib/puppet/resource/catalog.rb', line 62 def title_key_for_ref( ref ) ref =~ /^([-\w:]+)\[(.*)\]$/m [$1, $2] end |
#to_data_hash ⇒ Object
368 369 370 371 372 373 374 375 376 377 378 |
# File 'lib/puppet/resource/catalog.rb', line 368 def to_data_hash { 'tags' => , 'name' => name, 'version' => version, 'environment' => environment.to_s, 'resources' => @resources.collect { |v| @resource_table[v].to_pson_data_hash }, 'edges' => edges. collect { |e| e.to_pson_data_hash }, 'classes' => classes } end |
#to_pson(*args) ⇒ Object
391 392 393 |
# File 'lib/puppet/resource/catalog.rb', line 391 def to_pson(*args) to_pson_data_hash.to_pson(*args) end |
#to_pson_data_hash ⇒ Object
381 382 383 384 385 386 387 388 389 |
# File 'lib/puppet/resource/catalog.rb', line 381 def to_pson_data_hash { 'document_type' => 'Catalog', 'data' => to_data_hash, 'metadata' => { 'api_version' => 1 } } end |
#to_ral ⇒ Object
Convert our catalog into a RAL catalog.
396 397 398 |
# File 'lib/puppet/resource/catalog.rb', line 396 def to_ral to_catalog :to_ral end |
#to_resource ⇒ Object
Convert our catalog into a catalog of Puppet::Resource instances.
401 402 403 |
# File 'lib/puppet/resource/catalog.rb', line 401 def to_resource to_catalog :to_resource end |
#write_class_file ⇒ Object
Store the classes in the classfile.
413 414 415 416 417 418 419 |
# File 'lib/puppet/resource/catalog.rb', line 413 def write_class_file ::File.open(Puppet[:classfile], "w") do |f| f.puts classes.join("\n") end rescue => detail Puppet.err "Could not create class file #{Puppet[:classfile]}: #{detail}" end |
#write_graph(name) ⇒ Object
Produce the graph files if requested.
439 440 441 442 443 444 |
# File 'lib/puppet/resource/catalog.rb', line 439 def write_graph(name) # We only want to graph the main host catalog. return unless host_config? super end |
#write_resource_file ⇒ Object
Store the list of resources we manage
422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 |
# File 'lib/puppet/resource/catalog.rb', line 422 def write_resource_file ::File.open(Puppet[:resourcefile], "w") do |f| to_print = resources.map do |resource| next unless resource.managed? if resource.name_var "#{resource.type}[#{resource[resource.name_var]}]" else "#{resource.ref.downcase}" end end.compact f.puts to_print.join("\n") end rescue => detail Puppet.err "Could not create resource file #{Puppet[:resourcefile]}: #{detail}" end |