Class: Puppet::Util::Lockfile
Overview
This class provides a simple API for managing a lock file whose contents are an (optional) String. In addition to querying the basic state (#locked?) of the lock, managing the lock (#lock, #unlock), the contents can be retrieved at any time while the lock is held (#lock_data). This can be used to store pids, messages, etc.
Direct Known Subclasses
Instance Attribute Summary collapse
- #file_path ⇒ Object readonly
Instance Method Summary collapse
-
#initialize(file_path) ⇒ Lockfile
constructor
A new instance of Lockfile.
-
#lock(lock_data = nil) ⇒ boolean
True if lock is successfully acquired, false otherwise.
-
#lock_data ⇒ String
Retrieve the (optional) lock data that was specified at the time the file was locked.
- #locked? ⇒ Boolean
- #unlock ⇒ Object
Constructor Details
#initialize(file_path) ⇒ Lockfile
Returns a new instance of Lockfile.
12 13 14 |
# File 'lib/puppet/util/lockfile.rb', line 12 def initialize(file_path) @file_path = file_path end |
Instance Attribute Details
#file_path ⇒ Object (readonly)
10 11 12 |
# File 'lib/puppet/util/lockfile.rb', line 10 def file_path @file_path end |
Instance Method Details
#lock(lock_data = nil) ⇒ boolean
Returns true if lock is successfully acquired, false otherwise.
25 26 27 28 29 30 |
# File 'lib/puppet/util/lockfile.rb', line 25 def lock(lock_data = nil) return false if locked? File.open(@file_path, 'w') { |fd| fd.print(lock_data) } true end |
#lock_data ⇒ String
Retrieve the (optional) lock data that was specified at the time the file
was locked.
49 50 51 |
# File 'lib/puppet/util/lockfile.rb', line 49 def lock_data return File.read(@file_path) if file_locked? end |
#locked? ⇒ Boolean
41 42 43 44 |
# File 'lib/puppet/util/lockfile.rb', line 41 def locked? # delegate logic to a more explicit private method file_locked? end |
#unlock ⇒ Object
32 33 34 35 36 37 38 39 |
# File 'lib/puppet/util/lockfile.rb', line 32 def unlock if locked? Puppet::FileSystem::File.unlink(@file_path) true else false end end |